@@ -69,9 +69,9 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
- * Processes a PDF content stream and executes certain operations. Provides a callback interface for clients that want
- * to do things with the stream.
- * 
+ * Processes a PDF content stream and executes certain operations. Provides a callback interface for
+ * clients that want to do things with the stream.
+ *
  * @author Ben Litchfield
  */
 public abstract class PDFStreamEngine
@@ -112,7 +112,8 @@ public abstract class PDFStreamEngine
     }
 
     /**
-     * Adds an operator processor to the engine if there isn't an operator already associated with the PDF operator.
+     * Adds an operator processor to the engine if there isn't an operator already associated with
+     * the PDF operator.
      *
      * @param op operator processor
      * @return true if the operator is added, false if not (there's already an operator associated)
@@ -194,7 +195,7 @@ public abstract class PDFStreamEngine
 
     /**
      * Processes a soft mask transparency group stream.
-     * 
+     *
      * @param group
      * @throws IOException
      */
@@ -209,7 +210,7 @@ public abstract class PDFStreamEngine
 
     /**
      * Processes a transparency group stream.
-     * 
+     *
      * @param group
      * @throws IOException
      */
@@ -224,37 +225,43 @@ public abstract class PDFStreamEngine
         PDResources parent = pushResources(group);
         Deque<PDGraphicsState> savedStack = saveGraphicsStack();
 
+        PDGraphicsState graphicsState = getGraphicsState();
         Matrix parentMatrix = initialMatrix;
 
         // the stream's initial matrix includes the parent CTM, e.g. this allows a scaled form
-        initialMatrix = getGraphicsState().getCurrentTransformationMatrix().clone();
+        initialMatrix = graphicsState.getCurrentTransformationMatrix().clone();
 
         // transform the CTM using the stream's matrix
-        getGraphicsState().getCurrentTransformationMatrix().concatenate(group.getMatrix());
+        graphicsState.getCurrentTransformationMatrix().concatenate(group.getMatrix());
 
         // Before execution of the transparency group XObject’s content stream,
         // the current blend mode in the graphics state shall be initialized to Normal,
         // the current stroking and nonstroking alpha constants to 1.0, and the current soft mask to None.
-        getGraphicsState().setBlendMode(BlendMode.NORMAL);
-        getGraphicsState().setAlphaConstant(1);
-        getGraphicsState().setNonStrokeAlphaConstants(1);
-        getGraphicsState().setSoftMask(null);
+        graphicsState.setBlendMode(BlendMode.NORMAL);
+        graphicsState.setAlphaConstant(1);
+        graphicsState.setNonStrokeAlphaConstants(1);
+        graphicsState.setSoftMask(null);
 
         // clip to bounding box
         clipToRect(group.getBBox());
 
-        processStreamOperators(group);
+        try
+        {
+            processStreamOperators(group);
+        }
+        finally
+        {
+            initialMatrix = parentMatrix;
 
-        initialMatrix = parentMatrix;
-
-        restoreGraphicsStack(savedStack);
-        popResources(parent);
+            restoreGraphicsStack(savedStack);
+            popResources(parent);
+        }
     }
 
     /**
      * Processes a Type 3 character stream.
      *
-     * @param charProc Type 3 character procedure
+     * @param charProc            Type 3 character procedure
      * @param textRenderingMatrix the Text Rendering Matrix
      */
     protected void processType3Stream(PDType3CharProc charProc, Matrix textRenderingMatrix)
@@ -273,7 +280,7 @@ public abstract class PDFStreamEngine
         getGraphicsState().setCurrentTransformationMatrix(textRenderingMatrix);
 
         // transform the CTM using the stream's matrix (this is the FontMatrix)
-        getGraphicsState().getCurrentTransformationMatrix().concatenate(charProc.getMatrix());
+        textRenderingMatrix.concatenate(charProc.getMatrix());
 
         // note: we don't clip to the BBox as it is often wrong, see PDFBOX-1917
 
@@ -283,14 +290,19 @@ public abstract class PDFStreamEngine
         Matrix textLineMatrixOld = textLineMatrix;
         textLineMatrix = new Matrix();
 
-        processStreamOperators(charProc);
+        try
+        {
+            processStreamOperators(charProc);
+        }
+        finally
+        {
+            // restore text matrices
+            textMatrix = textMatrixOld;
+            textLineMatrix = textLineMatrixOld;
 
-        // restore text matrices
-        textMatrix = textMatrixOld;
-        textLineMatrix = textLineMatrixOld;
-
-        restoreGraphicsStack(savedStack);
-        popResources(parent);
+            restoreGraphicsStack(savedStack);
+            popResources(parent);
+        }
     }
 
     /**
@@ -302,16 +314,17 @@ public abstract class PDFStreamEngine
     protected void processAnnotation(PDAnnotation annotation, PDAppearanceStream appearance)
             throws IOException
     {
-        PDResources parent = pushResources(appearance);
-        Deque<PDGraphicsState> savedStack = saveGraphicsStack();
-
         PDRectangle bbox = appearance.getBBox();
         PDRectangle rect = annotation.getRectangle();
-        Matrix matrix = appearance.getMatrix();
 
         // zero-sized rectangles are not valid
-        if (rect != null && rect.getWidth() > 0 && rect.getHeight() > 0 && bbox != null)
+        if (rect != null && rect.getWidth() > 0 && rect.getHeight() > 0 && bbox != null
+                && bbox.getWidth() > 0 && bbox.getHeight() > 0)
         {
+            PDResources parent = pushResources(appearance);
+            Deque<PDGraphicsState> savedStack = saveGraphicsStack();
+
+            Matrix matrix = appearance.getMatrix();
             // transformed appearance box fixme: may be an arbitrary shape
             Rectangle2D transformedBox = bbox.transform(matrix).getBounds2D();
 
@@ -338,19 +351,24 @@ public abstract class PDFStreamEngine
             // needed for patterns in appearance streams, e.g. PDFBOX-2182
             initialMatrix = aa.clone();
 
-            processStreamOperators(appearance);
+            try
+            {
+                processStreamOperators(appearance);
+            }
+            finally
+            {
+                restoreGraphicsStack(savedStack);
+                popResources(parent);
+            }
         }
-
-        restoreGraphicsStack(savedStack);
-        popResources(parent);
     }
 
     /**
      * Process the given tiling pattern.
      *
      * @param tilingPattern the tiling pattern
-     * @param color color to use, if this is an uncoloured pattern, otherwise null.
-     * @param colorSpace color space to use, if this is an uncoloured pattern, otherwise null.
+     * @param color         color to use, if this is an uncoloured pattern, otherwise null.
+     * @param colorSpace    color space to use, if this is an uncoloured pattern, otherwise null.
      */
     protected final void processTilingPattern(PDTilingPattern tilingPattern, PDColor color,
             PDColorSpace colorSpace) throws IOException
@@ -359,11 +377,12 @@ public abstract class PDFStreamEngine
     }
 
     /**
-     * Process the given tiling pattern. Allows the pattern matrix to be overridden for custom rendering.
+     * Process the given tiling pattern. Allows the pattern matrix to be overridden for custom
+     * rendering.
      *
      * @param tilingPattern the tiling pattern
-     * @param color color to use, if this is an uncoloured pattern, otherwise null.
-     * @param colorSpace color space to use, if this is an uncoloured pattern, otherwise null.
+     * @param color         color to use, if this is an uncoloured pattern, otherwise null.
+     * @param colorSpace    color space to use, if this is an uncoloured pattern, otherwise null.
      * @param patternMatrix the pattern matrix, may be overridden for custom rendering.
      */
     protected final void processTilingPattern(PDTilingPattern tilingPattern, PDColor color,
@@ -378,32 +397,43 @@ public abstract class PDFStreamEngine
         Deque<PDGraphicsState> savedStack = saveGraphicsStack();
 
         // save a clean state (new clipping path, line path, etc.)
-        Rectangle2D bbox = tilingPattern.getBBox().transform(patternMatrix).getBounds2D();
+        PDRectangle tilingBBox = tilingPattern.getBBox();
+        Rectangle2D bbox = tilingBBox.transform(patternMatrix).getBounds2D();
         PDRectangle rect = new PDRectangle((float) bbox.getX(), (float) bbox.getY(),
                 (float) bbox.getWidth(), (float) bbox.getHeight());
         graphicsStack.push(new PDGraphicsState(rect));
+        PDGraphicsState graphicsState = getGraphicsState();
 
         // non-colored patterns have to be given a color
         if (colorSpace != null)
         {
             color = new PDColor(color.getComponents(), colorSpace);
-            getGraphicsState().setNonStrokingColorSpace(colorSpace);
-            getGraphicsState().setNonStrokingColor(color);
-            getGraphicsState().setStrokingColorSpace(colorSpace);
-            getGraphicsState().setStrokingColor(color);
+            graphicsState.setNonStrokingColorSpace(colorSpace);
+            graphicsState.setNonStrokingColor(color);
+            graphicsState.setStrokingColorSpace(colorSpace);
+            graphicsState.setStrokingColor(color);
         }
 
         // transform the CTM using the stream's matrix
-        getGraphicsState().getCurrentTransformationMatrix().concatenate(patternMatrix);
+        graphicsState.getCurrentTransformationMatrix().concatenate(patternMatrix);
 
         // clip to bounding box
-        clipToRect(tilingPattern.getBBox());
+        clipToRect(tilingBBox);
 
-        processStreamOperators(tilingPattern);
-
-        initialMatrix = parentMatrix;
-        restoreGraphicsStack(savedStack);
-        popResources(parent);
+        Matrix textMatrixSave = textMatrix;
+        Matrix textLineMatrixSave = textLineMatrix;
+        try
+        {
+            processStreamOperators(tilingPattern);
+        }
+        finally
+        {
+            textMatrix = textMatrixSave;
+            textLineMatrix = textLineMatrixSave;
+            initialMatrix = parentMatrix;
+            restoreGraphicsStack(savedStack);
+            popResources(parent);
+        }
     }
 
     /**
@@ -422,8 +452,8 @@ public abstract class PDFStreamEngine
     }
 
     /**
-     * Returns the appearance stream to process for the given annotation. May be used to render a specific appearance
-     * such as "hover".
+     * Returns the appearance stream to process for the given annotation. May be used to render a
+     * specific appearance such as "hover".
      *
      * @param annotation The current annotation.
      * @return The stream to process.
@@ -473,11 +503,16 @@ public abstract class PDFStreamEngine
         PDRectangle bbox = contentStream.getBBox();
         clipToRect(bbox);
 
-        processStreamOperators(contentStream);
-
-        initialMatrix = parentMatrix;
-        restoreGraphicsStack(savedStack);
-        popResources(parent);
+        try
+        {
+            processStreamOperators(contentStream);
+        }
+        finally
+        {
+            initialMatrix = parentMatrix;
+            restoreGraphicsStack(savedStack);
+            popResources(parent);
+        }
     }
 
     /**
@@ -493,7 +528,7 @@ public abstract class PDFStreamEngine
             if (token instanceof Operator)
             {
                 processOperator((Operator) token, arguments);
-                arguments.clear();
+                arguments = new ArrayList<>();
             }
             else
             {
@@ -541,21 +576,22 @@ public abstract class PDFStreamEngine
     }
 
     /**
-     * Transforms the given rectangle using the CTM and then intersects it with the current clipping area.
+     * Transforms the given rectangle using the CTM and then intersects it with the current clipping
+     * area.
      */
     private void clipToRect(PDRectangle rectangle)
     {
         if (rectangle != null)
         {
-            GeneralPath clip = rectangle
-                    .transform(getGraphicsState().getCurrentTransformationMatrix());
-            getGraphicsState().intersectClippingPath(clip);
+            PDGraphicsState graphicsState = getGraphicsState();
+            GeneralPath clip = rectangle.transform(graphicsState.getCurrentTransformationMatrix());
+            graphicsState.intersectClippingPath(clip);
         }
     }
 
     /**
-     * Called when the BT operator is encountered. This method is for overriding in subclasses, the default
-     * implementation does nothing.
+     * Called when the BT operator is encountered. This method is for overriding in subclasses, the
+     * default implementation does nothing.
      *
      * @throws IOException if there was an error processing the text
      */
@@ -565,8 +601,8 @@ public abstract class PDFStreamEngine
     }
 
     /**
-     * Called when the ET operator is encountered. This method is for overriding in subclasses, the default
-     * implementation does nothing.
+     * Called when the ET operator is encountered. This method is for overriding in subclasses, the
+     * default implementation does nothing.
      *
      * @throws IOException if there was an error processing the text
      */
@@ -598,7 +634,7 @@ public abstract class PDFStreamEngine
         float fontSize = textState.getFontSize();
         float horizontalScaling = textState.getHorizontalScaling() / 100f;
         PDFont font = textState.getFont();
-        boolean isVertical = ofNullable(font).map(f -> f.isVertical()).orElse(false);
+        boolean isVertical = ofNullable(font).map(PDFont::isVertical).orElse(false);
 
         for (COSBase obj : array)
         {
@@ -626,9 +662,14 @@ public abstract class PDFStreamEngine
                 byte[] string = ((COSString) obj).getBytes();
                 showText(string);
             }
+            else if (obj instanceof COSArray)
+            {
+                LOG.error("Nested arrays are not allowed in an array for TJ operation: {}", obj);
+            }
             else
             {
-                throw new IOException("Unknown type in array for TJ operation:" + obj);
+                throw new IOException("Unknown type " + obj.getClass().getSimpleName()
+                        + " in array for TJ operation:" + obj);
             }
         }
     }
@@ -646,8 +687,8 @@ public abstract class PDFStreamEngine
     }
 
     /**
-     * Process text from the PDF Stream. You should override this method if you want to perform an action when encoded
-     * text is being processed.
+     * Process text from the PDF Stream. You should override this method if you want to perform an
+     * action when encoded text is being processed.
      *
      * @param string the encoded text
      * @throws IOException if there is an error processing the string
@@ -682,7 +723,6 @@ public abstract class PDFStreamEngine
             int before = in.available();
             int code = font.readCode(in);
             int codeLength = before - in.available();
-            String unicode = font.toUnicode(code);
 
             // Word spacing shall be applied to every occurrence of the single-byte character code
             // 32 in a string when using a simple font or a composite font that defines code 32 as
@@ -712,13 +752,7 @@ public abstract class PDFStreamEngine
             Vector w = font.getDisplacement(code);
 
             // process the decoded glyph
-            saveGraphicsState();
-            Matrix textMatrixOld = textMatrix;
-            Matrix textLineMatrixOld = textLineMatrix;
-            showGlyph(textRenderingMatrix, font, code, unicode, w);
-            textMatrix = textMatrixOld;
-            textLineMatrix = textLineMatrixOld;
-            restoreGraphicsState();
+            showGlyph(textRenderingMatrix, font, code, w);
 
             // calculate the combined displacements
             float tx, ty;
@@ -739,59 +773,57 @@ public abstract class PDFStreamEngine
     }
 
     /**
-     * Called when a glyph is to be processed.This method is intended for overriding in subclasses, the default
-     * implementation does nothing.
+     * Called when a glyph is to be processed. This method is intended for overriding in subclasses,
+     * Called when a glyph is to be processed. This method is intended for overriding in subclasses,
+     * the default the default implementation does nothing. implementation does nothing.
      *
      * @param textRenderingMatrix the current text rendering matrix, T<sub>rm</sub>
-     * @param font the current font
-     * @param code internal PDF character code for the glyph
-     * @param unicode the Unicode text for this glyph, or null if the PDF does provide it
-     * @param displacement the displacement (i.e. advance) of the glyph in text space
+     * @param font                the current font
+     * @param code                internal PDF character code for the glyph
+     * @param displacement        the displacement (i.e. advance) of the glyph in text space
      * @throws IOException if the glyph cannot be processed
      */
-    protected void showGlyph(Matrix textRenderingMatrix, PDFont font, int code, String unicode,
-            Vector displacement) throws IOException
+    protected void showGlyph(Matrix textRenderingMatrix, PDFont font, int code, Vector displacement)
+            throws IOException
     {
         if (font instanceof PDType3Font)
         {
-            showType3Glyph(textRenderingMatrix, (PDType3Font) font, code, unicode, displacement);
+            showType3Glyph(textRenderingMatrix, (PDType3Font) font, code, displacement);
         }
         else
         {
-            showFontGlyph(textRenderingMatrix, font, code, unicode, displacement);
+            showFontGlyph(textRenderingMatrix, font, code, displacement);
         }
     }
 
     /**
-     * Called when a glyph is to be processed.This method is intended for overriding in subclasses, the default
-     * implementation does nothing.
+     * Called when a glyph is to be processed.This method is intended for overriding in subclasses,
+     * the default implementation does nothing.
      *
      * @param textRenderingMatrix the current text rendering matrix, T<sub>rm</sub>
-     * @param font the current font
-     * @param code internal PDF character code for the glyph
-     * @param unicode the Unicode text for this glyph, or null if the PDF does provide it
-     * @param displacement the displacement (i.e. advance) of the glyph in text space
+     * @param font                the current font
+     * @param code                internal PDF character code for the glyph
+     * @param displacement        the displacement (i.e. advance) of the glyph in text space
      * @throws IOException if the glyph cannot be processed
      */
-    protected void showFontGlyph(Matrix textRenderingMatrix, PDFont font, int code, String unicode,
+    protected void showFontGlyph(Matrix textRenderingMatrix, PDFont font, int code,
             Vector displacement) throws IOException
     {
         // overridden in subclasses
     }
 
     /**
-     * Called when a glyph is to be processed.This method is intended for overriding in subclasses, the default
-     * implementation does nothing.
+     * Called when a glyph is to be processed.This method is intended for overriding in subclasses,
+     * the default implementation does nothing.
      *
      * @param textRenderingMatrix the current text rendering matrix, T<sub>rm</sub>
-     * @param font the current font
-     * @param code internal PDF character code for the glyph
-     * @param unicode the Unicode text for this glyph, or null if the PDF does provide it
-     * @param displacement the displacement (i.e. advance) of the glyph in text space
+     * @param font                the current font
+     * @param code                internal PDF character code for the glyph
+     * @param displacement        the displacement (i.e. advance) of the glyph in text space
      * @throws IOException if the glyph cannot be processed
      */
     protected void showType3Glyph(Matrix textRenderingMatrix, PDType3Font font, int code,
-            String unicode, Vector displacement) throws IOException
+            Vector displacement) throws IOException
     {
         PDType3CharProc charProc = font.getCharProc(code);
         if (charProc != null)
@@ -803,7 +835,7 @@ public abstract class PDFStreamEngine
     /**
      * Called when a marked content group begins
      *
-     * @param tag content tag
+     * @param tag        content tag
      * @param properties optional properties
      */
     public void beginMarkedContentSequence(COSName tag, COSDictionary properties)
@@ -821,7 +853,7 @@ public abstract class PDFStreamEngine
 
     /**
      * This is used to handle an operation.
-     * 
+     *
      * @param operation The operation to perform.
      * @param arguments The list of arguments.
      * @throws IOException If there is an error processing the operation.
@@ -834,7 +866,7 @@ public abstract class PDFStreamEngine
 
     /**
      * This is used to handle an operation.
-     * 
+     *
      * @param operator The operation to perform.
      * @param operands The list of arguments.
      * @throws IOException If there is an error processing the operation.
@@ -924,7 +956,7 @@ public abstract class PDFStreamEngine
     protected final Deque<PDGraphicsState> saveGraphicsStack()
     {
         Deque<PDGraphicsState> savedStack = graphicsStack;
-        graphicsStack = new ArrayDeque<>();
+        graphicsStack = new ArrayDeque<>(1);
         graphicsStack.add(savedStack.peek().clone());
         return savedStack;
     }
@@ -1001,7 +1033,8 @@ public abstract class PDFStreamEngine
     }
 
     /**
-     * @return the stream' resources. This is mainly to be used by the {@link OperatorProcessor} classes
+     * @return the stream' resources. This is mainly to be used by the {@link OperatorProcessor}
+     * classes
      */
     public PDResources getResources()
     {
@@ -1047,8 +1080,8 @@ public abstract class PDFStreamEngine
     }
 
     /**
-     * Get the current level. This can be used to decide whether a recursion has done too deep and an operation should
-     * be skipped to avoid a stack overflow.
+     * Get the current level. This can be used to decide whether a recursion has done too deep and
+     * an operation should be skipped to avoid a stack overflow.
      *
      * @return the current level.
      */
@@ -1066,9 +1099,9 @@ public abstract class PDFStreamEngine
     }
 
     /**
-     * Decrease the level. Call this after running a potentially recursive operation. A log message is shown if the
-     * level is below 0. This can happen if the level is not decreased after an operation is done, e.g. by using a
-     * "finally" block.
+     * Decrease the level. Call this after running a potentially recursive operation. A log message
+     * is shown if the level is below 0. This can happen if the level is not decreased after an
+     * operation is done, e.g. by using a "finally" block.
      */
     public void decreaseLevel()
     {
