@@ -16,11 +16,12 @@
  */
 package org.sejda.sambox.contentstream.operator.graphics;
 
-import java.awt.geom.GeneralPath;
+import java.awt.geom.Path2D;
 import java.io.IOException;
 import java.util.List;
 
 import org.sejda.sambox.contentstream.operator.Operator;
+import org.sejda.sambox.contentstream.operator.OperatorName;
 import org.sejda.sambox.cos.COSBase;
 
 /**
@@ -33,12 +34,12 @@ public class ClipNonZeroRule extends GraphicsOperatorProcessor
     @Override
     public void process(Operator operator, List<COSBase> operands) throws IOException
     {
-        getContext().clip(GeneralPath.WIND_NON_ZERO);
+        getContext().clip(Path2D.WIND_NON_ZERO);
     }
 
     @Override
     public String getName()
     {
-        return "W";
+        return OperatorName.CLIP_NON_ZERO;
     }
 }
