@@ -19,14 +19,15 @@ package org.sejda.sambox.filter;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.io.PushbackInputStream;
 
 import org.sejda.commons.util.IOUtils;
 import org.sejda.sambox.cos.COSDictionary;
 import org.sejda.sambox.cos.COSName;
 
 /**
- * Decodes image data that has been encoded using either Group 3 or Group 4 CCITT facsimile (fax) encoding, and encodes
- * image data to Group 4.
+ * Decodes image data that has been encoded using either Group 3 or Group 4 CCITT facsimile (fax)
+ * encoding, and encodes image data to Group 4.
  *
  * @author Ben Litchfield
  * @author Marcel Kammer
@@ -64,29 +65,46 @@ final class CCITTFaxFilter extends Filter
         byte[] decompressed = new byte[arraySize];
         CCITTFaxDecoderStream s;
         int type;
-        long tiffOptions;
+        long tiffOptions = 0;
         if (k == 0)
         {
-            tiffOptions = encodedByteAlign ? TIFFExtension.GROUP3OPT_BYTEALIGNED : 0;
-            type = TIFFExtension.COMPRESSION_CCITT_MODIFIED_HUFFMAN_RLE;
+            type = TIFFExtension.COMPRESSION_CCITT_T4; // Group 3 1D
+            byte[] streamData = new byte[20];
+            int bytesRead = encoded.read(streamData);
+            PushbackInputStream pushbackInputStream = new PushbackInputStream(encoded,
+                    streamData.length);
+            pushbackInputStream.unread(streamData, 0, bytesRead);
+            encoded = pushbackInputStream;
+            if (streamData[0] != 0 || (streamData[1] >> 4 != 1 && streamData[1] != 1))
+            {
+                // leading EOL (0b000000000001) not found, search further and try RLE if not
+                // found
+                type = TIFFExtension.COMPRESSION_CCITT_MODIFIED_HUFFMAN_RLE;
+                short b = (short) (((streamData[0] << 8) + streamData[1]) >> 4);
+                for (int i = 12; i < bytesRead * 8; i++)
+                {
+                    b = (short) ((b << 1) + ((streamData[(i / 8)] >> (7 - (i % 8))) & 0x01));
+                    if ((b & 0xFFF) == 1)
+                    {
+                        type = TIFFExtension.COMPRESSION_CCITT_T4;
+                        break;
+                    }
+                }
+            }
+        }
+        else if (k > 0)
+        {
+            // Group 3 2D
+            type = TIFFExtension.COMPRESSION_CCITT_T4;
+            tiffOptions = TIFFExtension.GROUP3OPT_2DENCODING;
         }
         else
         {
-            if (k > 0)
-            {
-                tiffOptions = encodedByteAlign ? TIFFExtension.GROUP3OPT_BYTEALIGNED : 0;
-                tiffOptions |= TIFFExtension.GROUP3OPT_2DENCODING;
-                type = TIFFExtension.COMPRESSION_CCITT_T4;
-            }
-            else
-            {
-                // k < 0
-                tiffOptions = encodedByteAlign ? TIFFExtension.GROUP4OPT_BYTEALIGNED : 0;
-                type = TIFFExtension.COMPRESSION_CCITT_T6;
-            }
+            // Group 4
+            type = TIFFExtension.COMPRESSION_CCITT_T6;
         }
-        s = new CCITTFaxDecoderStream(encoded, cols, type, TIFFExtension.FILL_LEFT_TO_RIGHT,
-                tiffOptions);
+
+        s = new CCITTFaxDecoderStream(encoded, cols, type, tiffOptions, encodedByteAlign);
         readFromDecoderStream(s, decompressed);
 
         // invert bitmap
@@ -136,6 +154,5 @@ final class CCITTFaxFilter extends Filter
         int rows = parameters.getInt(COSName.ROWS);
         IOUtils.copy(input,
                 new CCITTFaxEncoderStream(encoded, cols, rows, TIFFExtension.FILL_LEFT_TO_RIGHT));
-        input.close();
     }
 }
