@@ -17,17 +17,15 @@ package org.sejda.sambox.filter;
 
 import java.io.FilterOutputStream;
 import java.io.IOException;
-import java.io.InputStream;
 import java.io.OutputStream;
 import java.util.Arrays;
 
-import org.sejda.commons.util.IOUtils;
 import org.sejda.sambox.cos.COSDictionary;
 import org.sejda.sambox.cos.COSName;
 
 /**
- * Helper class to contain predictor decoding used by Flate and LZW filter. To see the history, look at the FlateFilter
- * class.
+ * Helper class to contain predictor decoding used by Flate and LZW filter. To see the history, look
+ * at the FlateFilter class.
  */
 public final class Predictor
 {
@@ -38,15 +36,16 @@ public final class Predictor
 
     /**
      * Decodes a single line of data in-place.
-     * 
-     * @param predictor Predictor value for the current line
-     * @param colors Number of color components, from decode parameters.
+     *
+     * @param predictor        Predictor value for the current line
+     * @param colors           Number of color components, from decode parameters.
      * @param bitsPerComponent Number of bits per components, from decode parameters.
-     * @param columns Number samples in a row, from decode parameters.
-     * @param actline Current (active) line to decode. Data will be decoded in-place, i.e. - the contents of this buffer
-     * will be modified.
-     * @param lastline The previous decoded line. When decoding the first line, this parameter should be an empty byte
-     * array of the same length as <code>actline</code>.
+     * @param columns          Number samples in a row, from decode parameters.
+     * @param actline          Current (active) line to decode. Data will be decoded in-place, i.e.
+     *                         - the contents of this buffer will be modified.
+     * @param lastline         The previous decoded line. When decoding the first line, this
+     *                         parameter should be an empty byte array of the same length as
+     *                         <code>actline</code>.
      */
     static void decodePredictorRow(int predictor, int colors, int bitsPerComponent, int columns,
             byte[] actline, byte[] lastline)
@@ -76,11 +75,11 @@ public final class Predictor
             }
             if (bitsPerComponent == 16)
             {
-                for (int p = bytesPerPixel; p < rowlength; p += 2)
+                for (int p = bytesPerPixel; p < rowlength - 1; p += 2)
                 {
                     int sub = ((actline[p] & 0xff) << 8) + (actline[p + 1] & 0xff);
-                    int left = (((actline[p - bytesPerPixel] & 0xff) << 8)
-                            + (actline[p - bytesPerPixel + 1] & 0xff));
+                    int left = (((actline[p - bytesPerPixel] & 0xff) << 8) + (
+                            actline[p - bytesPerPixel + 1] & 0xff));
                     actline[p] = (byte) (((sub + left) >> 8) & 0xff);
                     actline[p + 1] = (byte) ((sub + left) & 0xff);
                 }
@@ -115,12 +114,12 @@ public final class Predictor
                         if (((sub + left) & 1) == 0)
                         {
                             // reset bit
-                            actline[p] = (byte) (actline[p] & ~(1 << bit));
+                            actline[p] &= ~(1 << bit);
                         }
                         else
                         {
                             // set bit
-                            actline[p] = (byte) (actline[p] | (1 << bit));
+                            actline[p] |= 1 << bit;
                         }
                     }
                 }
@@ -205,54 +204,6 @@ public final class Predictor
         }
     }
 
-    static void decodePredictor(int predictor, int colors, int bitsPerComponent, int columns,
-            InputStream in, OutputStream out) throws IOException
-    {
-        if (predictor == 1)
-        {
-            IOUtils.copy(in, out);
-        }
-        else
-        {
-            // calculate sizes
-            final int rowlength = calculateRowLength(colors, bitsPerComponent, columns);
-            byte[] actline = new byte[rowlength];
-            byte[] lastline = new byte[rowlength];
-
-            int linepredictor = predictor;
-
-            while (in.available() > 0)
-            {
-                // test for PNG predictor; each value >= 10 (not only 15) indicates usage of PNG predictor
-                if (predictor >= 10)
-                {
-                    // PNG predictor; each row starts with predictor type (0, 1, 2, 3, 4)
-                    // read per line predictor
-                    linepredictor = in.read();
-                    if (linepredictor == -1)
-                    {
-                        return;
-                    }
-                    // add 10 to tread value 0 as 10, 1 as 11, ...
-                    linepredictor += 10;
-                }
-
-                // read line
-                int i, offset = 0;
-                while (offset < rowlength
-                        && ((i = in.read(actline, offset, rowlength - offset)) != -1))
-                {
-                    offset += i;
-                }
-
-                decodePredictorRow(linepredictor, colors, bitsPerComponent, columns, actline,
-                        lastline);
-                System.arraycopy(actline, 0, lastline, 0, rowlength);
-                out.write(actline);
-            }
-        }
-    }
-
     static int calculateRowLength(int colors, int bitsPerComponent, int columns)
     {
         final int bitsPerPixel = colors * bitsPerComponent;
@@ -276,13 +227,13 @@ public final class Predictor
     }
 
     /**
-     * Wraps and <code>OutputStream</code> in a predictor decoding stream as necessary. If no predictor is specified by
-     * the parameters, the original stream is returned as is.
+     * Wraps and <code>OutputStream</code> in a predictor decoding stream as necessary. If no
+     * predictor is specified by the parameters, the original stream is returned as is.
      *
-     * @param out The stream to which decoded data should be written
+     * @param out          The stream to which decoded data should be written
      * @param decodeParams Decode parameters for the stream
-     * @return An <code>OutputStream</code> is returned, which will write decoded data into the given stream. If no
-     * predictor is specified, the original stream is returned.
+     * @return An <code>OutputStream</code> is returned, which will write decoded data into the
+     * given stream. If no predictor is specified, the original stream is returned.
      */
     static OutputStream wrapPredictor(OutputStream out, COSDictionary decodeParams)
     {
@@ -299,8 +250,9 @@ public final class Predictor
     }
 
     /**
-     * Output stream that implements predictor decoding. Data is buffered until a complete row is available, which is
-     * then decoded and written to the underlying stream. The previous row is retained for decoding the next row.
+     * Output stream that implements predictor decoding. Data is buffered until a complete row is
+     * available, which is then decoded and written to the underlying stream. The previous row is
+     * retained for decoding the next row.
      */
     private static final class PredictorOutputStream extends FilterOutputStream
     {
@@ -316,7 +268,8 @@ public final class Predictor
         private final boolean predictorPerRow;
 
         // data buffers
-        private byte[] currentRow, lastRow;
+        private byte[] currentRow;
+        private byte[] lastRow;
         // amount of data in the current row
         private int currentRowData = 0;
         // was the per-row predictor value read for the current row being processed
@@ -382,7 +335,8 @@ public final class Predictor
         }
 
         /**
-         * Flips the row buffers (to avoid copying), and resets the current-row index and predictorRead flag
+         * Flips the row buffers (to avoid copying), and resets the current-row index and
+         * predictorRead flag
          */
         private void flipRows()
         {
