@@ -19,6 +19,7 @@ package org.sejda.sambox.input;
 import static java.util.Arrays.asList;
 import static org.sejda.commons.util.RequireUtils.requireIOCondition;
 import static org.sejda.commons.util.RequireUtils.requireNotNullArg;
+import static org.sejda.io.SeekableSources.asOffsettable;
 import static org.sejda.sambox.util.CharUtils.ASCII_BACKSPACE;
 import static org.sejda.sambox.util.CharUtils.ASCII_CARRIAGE_RETURN;
 import static org.sejda.sambox.util.CharUtils.ASCII_FORM_FEED;
@@ -43,6 +44,7 @@ import java.nio.charset.StandardCharsets;
 import org.sejda.commons.FastByteArrayOutputStream;
 import org.sejda.commons.Pool;
 import org.sejda.commons.util.IOUtils;
+import org.sejda.io.OffsettableSeekableSource;
 import org.sejda.io.SeekableSource;
 import org.sejda.sambox.SAMBox;
 import org.sejda.sambox.cos.COSObjectKey;
@@ -97,9 +99,26 @@ class SourceReader implements Closeable
         return source.position();
     }
 
+    /**
+     * adds an offset to the source
+     *
+     * @param offset
+     * @throws IOException
+     * @see {@link OffsettableSeekableSource#offset(long)}
+     */
+    public void offset(long offset) throws IOException
+    {
+        if (offset > 0)
+        {
+            OffsettableSeekableSource offsettable = asOffsettable(source);
+            offsettable.offset(offset);
+            this.source = offsettable;
+        }
+    }
+
     /**
      * seeks to the given offset
-     * 
+     *
      * @param offset the new offset
      * @throws IOException
      * @see {@link SeekableSource#position(long)}
@@ -121,7 +140,7 @@ class SourceReader implements Closeable
     /**
      * Skips the expected given String
      *
-     * @param expectedString the String value that is expected.
+     * @param expected the String value that is expected.
      * @throws IOException if the String char is not the expected value or if an I/O error occurs.
      */
     public final void skipExpected(String expected) throws IOException
@@ -264,7 +283,7 @@ class SourceReader implements Closeable
     }
 
     /**
-     * @param valid values for the next token.
+     * @param values values for the next token.
      * @return true if the next token is one of the given values. false otherwise.
      * @throws IOException if there is an error reading from the stream
      */
@@ -438,7 +457,8 @@ class SourceReader implements Closeable
     }
 
     /**
-     * Reads a a token conforming with a PDF Integer object defined in Numeric Objects chap 7.3.3 PDF 32000-1:2008.
+     * Reads a token conforming with a PDF Integer object defined in Numeric Objects chap 7.3.3 PDF
+     * 32000-1:2008.
      *
      * @return the token to parse as {@link Integer} or {@link Long}.
      * @throws IOException If there is an error reading from the stream.
