@@ -16,14 +16,6 @@
  */
 package org.sejda.sambox.pdmodel;
 
-import static java.util.Objects.nonNull;
-import static java.util.Optional.ofNullable;
-import static org.sejda.sambox.util.SpecVersionUtils.V1_5;
-
-import java.io.IOException;
-import java.util.ArrayList;
-import java.util.List;
-
 import org.sejda.sambox.cos.COSArray;
 import org.sejda.sambox.cos.COSArrayList;
 import org.sejda.sambox.cos.COSBase;
@@ -36,6 +28,9 @@ import org.sejda.sambox.pdmodel.common.PDMetadata;
 import org.sejda.sambox.pdmodel.common.PDPageLabels;
 import org.sejda.sambox.pdmodel.documentinterchange.logicalstructure.PDMarkInfo;
 import org.sejda.sambox.pdmodel.documentinterchange.logicalstructure.PDStructureTreeRoot;
+import org.sejda.sambox.pdmodel.fixup.AcroFormDefaultFixup;
+import org.sejda.sambox.pdmodel.fixup.AcroFormDefaultSamboxFixup;
+import org.sejda.sambox.pdmodel.fixup.PDDocumentFixup;
 import org.sejda.sambox.pdmodel.graphics.color.PDOutputIntent;
 import org.sejda.sambox.pdmodel.graphics.optionalcontent.PDOptionalContentProperties;
 import org.sejda.sambox.pdmodel.interactive.action.PDActionFactory;
@@ -51,6 +46,14 @@ import org.sejda.sambox.pdmodel.interactive.viewerpreferences.PDViewerPreference
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.List;
+
+import static java.util.Objects.nonNull;
+import static java.util.Optional.ofNullable;
+import static org.sejda.sambox.util.SpecVersionUtils.V1_5;
+
 /**
  * The Document Catalog of a PDF.
  *
@@ -62,6 +65,7 @@ public class PDDocumentCatalog implements COSObjectable
 
     private final COSDictionary root;
     private final PDDocument document;
+    private PDDocumentFixup acroFormFixupApplied;
     private PDAcroForm cachedAcroForm;
 
     /**
@@ -80,7 +84,7 @@ public class PDDocumentCatalog implements COSObjectable
     /**
      * Constructor.
      *
-     * @param doc The document that this catalog is part of.
+     * @param doc            The document that this catalog is part of.
      * @param rootDictionary The root dictionary that this object wraps.
      */
     public PDDocumentCatalog(PDDocument doc, COSDictionary rootDictionary)
@@ -107,6 +111,41 @@ public class PDDocumentCatalog implements COSObjectable
      */
     public PDAcroForm getAcroForm()
     {
+        // SAMBOX: by default we apply no fixups
+        //return getAcroForm(new AcroFormDefaultFixup(document));
+        return getAcroForm(new AcroFormDefaultSamboxFixup(document));
+    }
+    
+    public PDAcroForm getAcroFromWithFixups()
+    {
+        return getAcroForm(new AcroFormDefaultFixup(document));
+    }
+
+    /**
+     * Get the documents AcroForm. This will return null if no AcroForm is part of the document.
+     *
+     * Dependent on setting <code>acroFormFixup</code> some fixing/changes will be done to the AcroForm.
+     * If you need to ensure that there are no fixes applied call <code>getAcroForm</code> with <code>null</code>.
+     *
+     * Using <code>getAcroForm(PDDocumentFixup acroFormFixup)</code> might change the original content and
+     * subsequent calls with <code>getAcroForm(null)</code> will return the changed content.
+     *
+     * @param acroFormFixup the fix up action or null
+     * @return The document's AcroForm.
+     */
+    public PDAcroForm getAcroForm(PDDocumentFixup acroFormFixup)
+    {
+        if (acroFormFixup != null && acroFormFixup != acroFormFixupApplied)
+        {
+            acroFormFixup.apply();
+            cachedAcroForm = null;
+            acroFormFixupApplied =  acroFormFixup;
+        }
+        else if (acroFormFixupApplied != null)
+        {
+            LOG.debug("AcroForm content has already been retrieved with fixes applied - original content changed because of that");
+        }
+
         if (cachedAcroForm == null)
         {
             COSDictionary dict = root.getDictionaryObject(COSName.ACRO_FORM, COSDictionary.class);
@@ -116,9 +155,9 @@ public class PDDocumentCatalog implements COSObjectable
     }
 
     /**
-     * Sets the AcroForm for this catalog.
+     * sets the acroform for this catalog.
      *
-     * @param acroForm The new AcroForm.
+     * @param acroForm the new acroform.
      */
     public void setAcroForm(PDAcroForm acroForm)
     {
@@ -132,7 +171,8 @@ public class PDDocumentCatalog implements COSObjectable
     public PDPageTree getPages()
     {
         // TODO cache this since it's probably going to be called often and we want to reduce GC
-        return new PDPageTree((COSDictionary) root.getDictionaryObject(COSName.PAGES), document);
+        return new PDPageTree(root.getDictionaryObject(COSName.PAGES, COSDictionary.class),
+                document);
     }
 
     /**
@@ -142,8 +182,9 @@ public class PDDocumentCatalog implements COSObjectable
      */
     public PDViewerPreferences getViewerPreferences()
     {
-        return ofNullable(root.getDictionaryObject(COSName.VIEWER_PREFERENCES, COSDictionary.class))
-                .map(PDViewerPreferences::new).orElse(null);
+        return ofNullable(
+                root.getDictionaryObject(COSName.VIEWER_PREFERENCES, COSDictionary.class)).map(
+                PDViewerPreferences::new).orElse(null);
     }
 
     /**
@@ -163,8 +204,8 @@ public class PDDocumentCatalog implements COSObjectable
      */
     public PDDocumentOutline getDocumentOutline()
     {
-        return ofNullable(root.getDictionaryObject(COSName.OUTLINES, COSDictionary.class))
-                .map(PDDocumentOutline::new).orElse(null);
+        return ofNullable(root.getDictionaryObject(COSName.OUTLINES, COSDictionary.class)).map(
+                PDDocumentOutline::new).orElse(null);
     }
 
     /**
@@ -207,15 +248,15 @@ public class PDDocumentCatalog implements COSObjectable
     }
 
     /**
-     * Get the metadata that is part of the document catalog. This will return null if there is no meta data for this
-     * object.
+     * Get the metadata that is part of the document catalog. This will return null if there is no
+     * meta data for this object.
      *
      * @return The metadata for this object.
      */
     public PDMetadata getMetadata()
     {
-        return ofNullable(root.getDictionaryObject(COSName.METADATA, COSStream.class))
-                .map(PDMetadata::new).orElse(null);
+        return ofNullable(root.getDictionaryObject(COSName.METADATA, COSStream.class)).map(
+                PDMetadata::new).orElse(null);
     }
 
     /**
@@ -290,8 +331,8 @@ public class PDDocumentCatalog implements COSObjectable
      */
     public PDDocumentNameDictionary getNames()
     {
-        return ofNullable(root.getDictionaryObject(COSName.NAMES, COSDictionary.class))
-                .map(PDDocumentNameDictionary::new).orElse(null);
+        return ofNullable(root.getDictionaryObject(COSName.NAMES, COSDictionary.class)).map(
+                PDDocumentNameDictionary::new).orElse(null);
     }
 
     /**
@@ -299,8 +340,8 @@ public class PDDocumentCatalog implements COSObjectable
      */
     public PDDocumentNameDestinationDictionary getDests()
     {
-        return ofNullable(root.getDictionaryObject(COSName.DESTS, COSDictionary.class))
-                .map(PDDocumentNameDestinationDictionary::new).orElse(null);
+        return ofNullable(root.getDictionaryObject(COSName.DESTS, COSDictionary.class)).map(
+                PDDocumentNameDestinationDictionary::new).orElse(null);
     }
 
     /**
@@ -314,14 +355,15 @@ public class PDDocumentCatalog implements COSObjectable
     }
 
     /**
-     * Get info about doc's usage of tagged features. This will return null if there is no information.
+     * Get info about doc's usage of tagged features. This will return null if there is no
+     * information.
      *
      * @return The new mark info.
      */
     public PDMarkInfo getMarkInfo()
     {
-        return ofNullable(root.getDictionaryObject(COSName.MARK_INFO, COSDictionary.class))
-                .map(PDMarkInfo::new).orElse(null);
+        return ofNullable(root.getDictionaryObject(COSName.MARK_INFO, COSDictionary.class)).map(
+                PDMarkInfo::new).orElse(null);
     }
 
     /**
@@ -355,7 +397,8 @@ public class PDDocumentCatalog implements COSObjectable
     }
 
     /**
-     * Add an OutputIntent to the list. If there is not OutputIntent, the list is created and the first element added.
+     * Add an OutputIntent to the list. If there is not OutputIntent, the list is created and the
+     * first element added.
      *
      * @param outputIntent the OutputIntent to add.
      */
@@ -373,7 +416,8 @@ public class PDDocumentCatalog implements COSObjectable
     /**
      * Replace the list of OutputIntents of the document.
      *
-     * @param outputIntents the list of OutputIntents, if the list is empty all OutputIntents are removed.
+     * @param outputIntents the list of OutputIntents, if the list is empty all OutputIntents are
+     *                      removed.
      */
     public void setOutputIntents(List<PDOutputIntent> outputIntents)
     {
@@ -450,8 +494,8 @@ public class PDDocumentCatalog implements COSObjectable
      */
     public PDURIDictionary getURI()
     {
-        return ofNullable(root.getDictionaryObject(COSName.URI, COSDictionary.class))
-                .map(PDURIDictionary::new).orElse(null);
+        return ofNullable(root.getDictionaryObject(COSName.URI, COSDictionary.class)).map(
+                PDURIDictionary::new).orElse(null);
     }
 
     /**
@@ -469,8 +513,9 @@ public class PDDocumentCatalog implements COSObjectable
      */
     public PDStructureTreeRoot getStructureTreeRoot()
     {
-        return ofNullable(root.getDictionaryObject(COSName.STRUCT_TREE_ROOT, COSDictionary.class))
-                .map(PDStructureTreeRoot::new).orElse(null);
+        return ofNullable(
+                root.getDictionaryObject(COSName.STRUCT_TREE_ROOT, COSDictionary.class)).map(
+                PDStructureTreeRoot::new).orElse(null);
     }
 
     /**
@@ -529,8 +574,12 @@ public class PDDocumentCatalog implements COSObjectable
      */
     public PDPageLabels getPageLabels() throws IOException
     {
-        COSDictionary dict = (COSDictionary) root.getDictionaryObject(COSName.PAGE_LABELS);
-        return dict == null ? null : new PDPageLabels(dict);
+        COSDictionary labels = root.getDictionaryObject(COSName.PAGE_LABELS, COSDictionary.class);
+        if (nonNull(labels))
+        {
+            return new PDPageLabels(labels);
+        }
+        return null;
     }
 
     /**
@@ -550,8 +599,8 @@ public class PDDocumentCatalog implements COSObjectable
      */
     public PDOptionalContentProperties getOCProperties()
     {
-        return ofNullable(root.getDictionaryObject(COSName.OCPROPERTIES, COSDictionary.class))
-                .map(PDOptionalContentProperties::new).orElse(null);
+        return ofNullable(root.getDictionaryObject(COSName.OCPROPERTIES, COSDictionary.class)).map(
+                PDOptionalContentProperties::new).orElse(null);
     }
 
     /**
@@ -571,8 +620,9 @@ public class PDDocumentCatalog implements COSObjectable
     }
 
     /**
-     * Looks up for the {@link PDPageDestination} referenced by the given {@link PDNamedDestination}
-     * 
+     * Looks up for the {@link PDPageDestination} referenced by the given {@link
+     * PDNamedDestination}
+     *
      * @param namedDest
      * @return the destination or null if nothing is fond
      * @throws IOException
@@ -588,8 +638,8 @@ public class PDDocumentCatalog implements COSObjectable
             PDDocumentNameDestinationDictionary nameDestDict = getDests();
             if (nameDestDict != null)
             {
-                return (PDPageDestination) nameDestDict
-                        .getDestination(namedDest.getNamedDestination());
+                return (PDPageDestination) nameDestDict.getDestination(
+                        namedDest.getNamedDestination());
             }
         }
         return namesDest;
