@@ -20,9 +20,20 @@ import static java.util.Objects.isNull;
 import static java.util.Objects.nonNull;
 import static java.util.Optional.ofNullable;
 import static java.util.stream.Collectors.toList;
+import static org.sejda.commons.util.RequireUtils.require;
 import static org.sejda.commons.util.RequireUtils.requireNotNullArg;
 
-import java.util.*;
+import java.util.ArrayDeque;
+import java.util.ArrayList;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.NoSuchElementException;
+import java.util.Objects;
+import java.util.Queue;
+import java.util.Set;
+import java.util.Spliterator;
+import java.util.Spliterators;
 import java.util.stream.Stream;
 import java.util.stream.StreamSupport;
 
@@ -73,7 +84,7 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
     /**
      * Constructor for reading.
      *
-     * @param root A page tree root.
+     * @param root     A page tree root.
      * @param document The document which contains "root".
      */
     PDPageTree(COSDictionary root, PDDocument document)
@@ -97,19 +108,25 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
     }
 
     /**
-     * Similar to {@link #getInheritableAttribute(COSDictionary, COSName)} but also checks the returned element 
-     * matches the expected type class
+     * Similar to {@link #getInheritableAttribute(COSDictionary, COSName)} but also checks the
+     * returned element matches the expected type class
+     *
      * @param node
      * @param key
      * @param clazz expected type class
      * @param <T>
      * @return
      */
-    public static <T extends COSBase> COSBase getInheritableAttribute(COSDictionary node, COSName key, Class<T> clazz) {
+    public static <T extends COSBase> COSBase getInheritableAttribute(COSDictionary node,
+            COSName key, Class<T> clazz)
+    {
         COSBase result = getInheritableAttribute(node, key);
-        if(clazz.isInstance(result)) {
+        if (clazz.isInstance(result))
+        {
             return result;
-        } else {
+        }
+        else
+        {
             return null;
         }
     }
@@ -118,7 +135,7 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
      * Returns the given attribute, inheriting from parent tree nodes if necessary.
      *
      * @param node page object
-     * @param key the key to look up
+     * @param key  the key to look up
      * @return COS value for the given key
      */
     public static COSBase getInheritableAttribute(COSDictionary node, COSName key)
@@ -131,6 +148,13 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
 
         COSDictionary parent = node.getDictionaryObject(COSName.PARENT, COSName.P,
                 COSDictionary.class);
+        
+        // prevent infinite recursion
+        if (parent == node)
+        {
+            return null;
+        }
+        
         if (parent != null)
         {
             return getInheritableAttribute(parent, key);
@@ -145,7 +169,21 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
     @Override
     public Iterator<PDPage> iterator()
     {
-        return new PageIterator(root);
+        PageIterator iterator = new PageIterator(root);
+        // there's a problem when the expected number of pages is different than the actual number of pages loaded
+        // because iterating on PDPageTree.iterator() will silently skip broken pages that could not be loaded
+        if(iterator.size() != document.getNumberOfPages()) {
+            // try to throw a specific PageNotFound exception, identify which page is missing
+            for (int i = 0; i < document.getNumberOfPages(); i++)
+            {
+                get(i);
+            }
+            
+            // throw a generic "something's wrong" exception
+            throw new InvalidNumberOfPagesException(iterator.size(), document.getNumberOfPages());
+        }
+        
+        return iterator;
     }
 
     /**
@@ -168,7 +206,7 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
 
     /**
      * Helper to get kids from malformed PDFs.
-     * 
+     *
      * @param node page tree node
      * @return list of kids
      */
@@ -229,6 +267,11 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
             ResourceCache resourceCache = document != null ? document.getResourceCache() : null;
             return new PDPage(next, resourceCache);
         }
+        
+        public int size()
+        {
+            return queue.size();
+        }
 
         @Override
         public void remove()
@@ -288,7 +331,7 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
      */
     public PDPage get(int index)
     {
-        PageAndPageTreeParent res = get(index + 1, root, 0, null);
+        PageAndPageTreeParent res = get(index + 1, root, 0, null, new HashSet<>());
         COSDictionary dict = res.node;
 
         sanitizeType(dict);
@@ -315,20 +358,20 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
     /**
      * Returns the given COS page using a depth-first search.
      *
-     * @param pageNum 1-based page number
-     * @param node page tree node to search
-     * @param encountered number of pages encountered so far
+     * @param pageNum        1-based page number
+     * @param node           page tree node to search
+     * @param encountered    number of pages encountered so far
      * @param pageTreeParent the parent node, as determined traversing the page tree top -> down
      * @return COS dictionary of the Page object
      */
-    private PageAndPageTreeParent get(int pageNum, COSDictionary node, int encountered, COSDictionary pageTreeParent)
+    private PageAndPageTreeParent get(int pageNum, COSDictionary node, int encountered,
+            COSDictionary pageTreeParent, Set<COSDictionary> visited)
     {
-        if (pageNum < 0)
-        {
-            throw new PageNotFoundException(
-                    "Index out of bounds: " + pageNum + " in " + getSourcePath(), pageNum,
-                    getSourcePath());
-        }
+        require(pageNum >= 0, () -> new PageNotFoundException(
+                "Index out of bounds: " + pageNum + " in " + getSourcePath(), pageNum,
+                getSourcePath()));
+        require(visited.add(node), () -> new IllegalStateException(
+                "Possible recursion found when searching for page " + pageNum));
 
         if (isPageTreeNode(node))
         {
@@ -345,7 +388,7 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
                         if (pageNum <= encountered + kidCount)
                         {
                             // it's this kid
-                            return get(pageNum, kid, encountered, node);
+                            return get(pageNum, kid, encountered, node, visited);
                         }
                         encountered += kidCount;
                     }
@@ -356,11 +399,10 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
                         if (pageNum == encountered)
                         {
                             // it's this page
-                            return get(pageNum, kid, encountered, node);
+                            return get(pageNum, kid, encountered, node, visited);
                         }
                     }
                 }
-
                 throw new PageNotFoundException(
                         "Unable to find page " + pageNum + " in " + getSourcePath(), pageNum,
                         getSourcePath());
@@ -390,8 +432,8 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
     {
         // some files such as PDFBOX-2250-229205.pdf don't have Pages set as the Type, so we have
         // to check for the presence of Kids too
-        return nonNull(node) && (node.getCOSName(COSName.TYPE) == COSName.PAGES
-                || node.containsKey(COSName.KIDS));
+        return nonNull(node) && (node.getCOSName(COSName.TYPE) == COSName.PAGES || node.containsKey(
+                COSName.KIDS));
     }
 
     /**
@@ -449,7 +491,8 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
     }
 
     /**
-     * Returns the number of leaf nodes (page objects) that are descendants of this root within the page tree.
+     * Returns the number of leaf nodes (page objects) that are descendants of this root within the
+     * page tree.
      */
     public int getCount()
     {
@@ -464,12 +507,12 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
 
     /**
      * Removes the page with the given index from the page tree.
-     * 
+     *
      * @param index zero-based page index
      */
     public void remove(int index)
     {
-        PageAndPageTreeParent res = get(index + 1, root, 0, null);
+        PageAndPageTreeParent res = get(index + 1, root, 0, null, new HashSet<>());
         remove(res.node, res.parent);
     }
 
@@ -499,20 +542,20 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
         // remove from parent's kids
         COSDictionary parent = node.getDictionaryObject(COSName.PARENT, COSName.P,
                 COSDictionary.class);
-        
-        if(parent == null)
+
+        if (parent == null)
         {
             // broken node with missing PARENT, use the one known from traversing the page tree
             parent = knownParent;
         }
-        
+
         COSArray kids = parent.getDictionaryObject(COSName.KIDS, COSArray.class);
         if (kids.removeObject(node))
         {
             // update ancestor counts
             parent.setInt(COSName.COUNT, parent.getInt(COSName.COUNT) - 1);
             node = parent;
-            
+
             do
             {
                 node = node.getDictionaryObject(COSName.PARENT, COSName.P, COSDictionary.class);
@@ -526,7 +569,7 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
 
     /**
      * Adds the given page to this page tree.
-     * 
+     *
      * @param page The page to add.
      */
     public void add(PDPage page)
@@ -555,19 +598,21 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
     /**
      * Insert a page before another page within a page tree.
      *
-     * @param newPage the page to be inserted.
+     * @param newPage  the page to be inserted.
      * @param nextPage the page that is to be after the new page.
-     * @throws IllegalArgumentException if one attempts to insert a page that isn't part of a page tree.
+     * @throws IllegalArgumentException if one attempts to insert a page that isn't part of a page
+     *                                  tree.
      */
     public void insertBefore(PDPage newPage, PDPage nextPage)
     {
         COSDictionary nextPageDict = nextPage.getCOSObject();
-        COSDictionary parentDict = nextPageDict.getDictionaryObject(COSName.PARENT, COSDictionary.class);
-        if(nextPage.getPageTreeParent() != null)
+        COSDictionary parentDict = nextPageDict.getDictionaryObject(COSName.PARENT,
+                COSDictionary.class);
+        if (nextPage.getPageTreeParent() != null)
         {
             parentDict = nextPage.getPageTreeParent();
         }
-        
+
         COSArray kids = parentDict.getDictionaryObject(COSName.KIDS, COSArray.class);
         boolean found = false;
         for (int i = 0; i < kids.size(); ++i)
@@ -591,20 +636,21 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
     /**
      * Insert a page after another page within a page tree.
      *
-     * @param newPage the page to be inserted.
+     * @param newPage  the page to be inserted.
      * @param prevPage the page that is to be before the new page.
-     * @throws IllegalArgumentException if one attempts to insert a page that isn't part of a page tree.
+     * @throws IllegalArgumentException if one attempts to insert a page that isn't part of a page
+     *                                  tree.
      */
     public void insertAfter(PDPage newPage, PDPage prevPage)
     {
         COSDictionary prevPageDict = prevPage.getCOSObject();
         COSDictionary parentDict = prevPageDict.getDictionaryObject(COSName.PARENT,
                 COSDictionary.class);
-        if(prevPage.getPageTreeParent() != null)
+        if (prevPage.getPageTreeParent() != null)
         {
             parentDict = prevPage.getPageTreeParent();
         }
-        
+
         COSArray kids = parentDict.getDictionaryObject(COSName.KIDS, COSArray.class);
         boolean found = false;
         for (int i = 0; i < kids.size(); ++i)
@@ -636,15 +682,17 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
     }
 
     /**
-     * A tuple of a page and the parent found when traversing the page tree top down.
-     * NOTE: The page tree parent can be different from page.PARENT (for invalid documents)
-     * hence the need for this class when adding new pages and having to increment kids counters in the page tree
+     * A tuple of a page and the parent found when traversing the page tree top down. NOTE: The page
+     * tree parent can be different from page.PARENT (for invalid documents) hence the need for this
+     * class when adding new pages and having to increment kids counters in the page tree
      */
-    public static class PageAndPageTreeParent {
-        public final COSDictionary node; 
+    public static class PageAndPageTreeParent
+    {
+        public final COSDictionary node;
         public final COSDictionary parent;
 
-        public PageAndPageTreeParent(COSDictionary node, COSDictionary parent) {
+        public PageAndPageTreeParent(COSDictionary node, COSDictionary parent)
+        {
             this.node = node;
             this.parent = parent;
         }
