@@ -16,17 +16,6 @@
  */
 package org.sejda.sambox.pdmodel.common;
 
-import static java.util.Objects.isNull;
-import static java.util.Objects.nonNull;
-
-import java.io.IOException;
-import java.util.ArrayList;
-import java.util.Collections;
-import java.util.LinkedHashMap;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-
 import org.sejda.sambox.cos.COSArray;
 import org.sejda.sambox.cos.COSArrayList;
 import org.sejda.sambox.cos.COSBase;
@@ -37,6 +26,18 @@ import org.sejda.sambox.cos.COSString;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.LinkedHashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+
+import static java.util.Objects.isNull;
+import static java.util.Objects.nonNull;
+import static java.util.Optional.ofNullable;
+
 /**
  * This class represents a node in a name tree.
  *
@@ -80,7 +81,7 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
 
     /**
      * Returns the parent node.
-     * 
+     *
      * @return parent node
      */
     public PDNameTreeNode<T> getParent()
@@ -90,7 +91,7 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
 
     /**
      * Sets the parent to the given node.
-     * 
+     *
      * @param parentNode the node to be set as parent
      */
     public void setParent(PDNameTreeNode<T> parentNode)
@@ -101,7 +102,7 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
 
     /**
      * Determines if this is a root node or not.
-     * 
+     *
      * @return true if this is a root node
      */
     public boolean isRootNode()
@@ -118,7 +119,7 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
     {
         if (nonNull(node))
         {
-            COSArray kids = (COSArray) node.getDictionaryObject(COSName.KIDS);
+            COSArray kids = node.getDictionaryObject(COSName.KIDS, COSArray.class);
             if (kids != null)
             {
                 List<PDNameTreeNode<T>> pdObjects = new ArrayList<>();
@@ -139,7 +140,7 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
      */
     public void setKids(List<? extends PDNameTreeNode<T>> kids)
     {
-        if (kids != null && kids.size() > 0)
+        if (kids != null && !kids.isEmpty())
         {
             for (PDNameTreeNode<T> kidsNode : kids)
             {
@@ -169,7 +170,7 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
         else
         {
             List<PDNameTreeNode<T>> kids = getKids();
-            if (kids != null && kids.size() > 0)
+            if (kids != null && !kids.isEmpty())
             {
                 PDNameTreeNode<T> firstKid = kids.get(0);
                 PDNameTreeNode<T> lastKid = kids.get(kids.size() - 1);
@@ -269,33 +270,39 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
     }
 
     /**
-     * This will return a map of names. The key will be a string, and the value will depend on where this class is being
-     * used.
+     * This will return a map of names. The key will be a string, and the value will depend on where
+     * this class is being used.
      *
-     * @return ordered map of cos objects or <code>null</code> if dictionary contains no 'Names' entry
+     * @return ordered map of cos objects or <code>null</code> if dictionary contains no 'Names'
+     * entry
      * @throws IOException If there is an error while creating the sub types.
      */
     public Map<String, T> getNames() throws IOException
     {
-        COSArray namesArray = (COSArray) node.getDictionaryObject(COSName.NAMES);
+        COSArray namesArray = node.getDictionaryObject(COSName.NAMES, COSArray.class);
         if (namesArray != null)
         {
             Map<String, T> names = new LinkedHashMap<>();
+            if (namesArray.size() % 2 != 0)
+            {
+                LOG.warn("Names array has odd size: {}", namesArray.size());
+            }
             for (int i = 0; i < namesArray.size(); i += 2)
             {
-                COSString key = (COSString) namesArray.getObject(i);
-                if(i + 1 >= namesArray.size())
+                COSBase base = namesArray.getObject(i);
+                if (i + 1 >= namesArray.size())
                 {
                     // some invalid NAMES arrays have only the key without a value
-                    LOG.warn("Found key without value in NAMES array: " + key.getString() + ", at index: " + i);
+                    LOG.warn("Found key without value in NAMES array: {}, at index: {}", base, i);
                     continue;
                 }
+                COSString key = (COSString) base;
                 COSBase cosValue = namesArray.getObject(i + 1);
                 T pdValue = null;
-                try 
+                try
                 {
                     pdValue = convertCOSToPD(cosValue);
-                } 
+                }
                 catch (ClassCastException ex)
                 {
                     LOG.warn("Skipping, could not convert COS to PD: " + cosValue, ex);
@@ -309,8 +316,9 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
     }
 
     /**
-     * Method to convert the COS value in the name tree to the PD Model object. The default implementation will simply
-     * return the given COSBase object. Subclasses should do something specific.
+     * Method to convert the COS value in the name tree to the PD Model object. The default
+     * implementation will simply return the given COSBase object. Subclasses should do something
+     * specific.
      *
      * @param base The COS object to convert.
      * @return The converted PD Model object.
@@ -327,8 +335,9 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
     protected abstract PDNameTreeNode<T> createChildNode(COSDictionary dic);
 
     /**
-     * Set the names of for this node. The keys should be java.lang.String and the values must be a COSObjectable. This
-     * method will set the appropriate upper and lower limits based on the keys in the map.
+     * Set the names of for this node. The keys should be java.lang.String and the values must be a
+     * COSObjectable. This method will set the appropriate upper and lower limits based on the keys
+     * in the map.
      *
      * @param names map of names to objects, or <code>null</code>
      */
@@ -361,13 +370,8 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
      */
     public String getUpperLimit()
     {
-        String retval = null;
-        COSArray arr = (COSArray) node.getDictionaryObject(COSName.LIMITS);
-        if (arr != null)
-        {
-            retval = arr.getString(1);
-        }
-        return retval;
+        return ofNullable(node.getDictionaryObject(COSName.LIMITS, COSArray.class)).map(
+                a -> a.getString(1)).orElse(null);
     }
 
     /**
@@ -377,12 +381,10 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
      */
     private void setUpperLimit(String upper)
     {
-        COSArray arr = (COSArray) node.getDictionaryObject(COSName.LIMITS);
+        COSArray arr = node.getDictionaryObject(COSName.LIMITS, COSArray.class);
         if (arr == null)
         {
-            arr = new COSArray();
-            arr.add(null);
-            arr.add(null);
+            arr = new COSArray(null, null);
             node.setItem(COSName.LIMITS, arr);
         }
         arr.setString(1, upper);
@@ -395,12 +397,8 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
      */
     public String getLowerLimit()
     {
-        COSArray arr = (COSArray) node.getDictionaryObject(COSName.LIMITS);
-        if (nonNull(arr))
-        {
-            return arr.getString(0);
-        }
-        return null;
+        return ofNullable(node.getDictionaryObject(COSName.LIMITS, COSArray.class)).map(
+                a -> a.getString(0)).orElse(null);
     }
 
     /**
@@ -410,12 +408,10 @@ public abstract class PDNameTreeNode<T extends COSObjectable> implements COSObje
      */
     private void setLowerLimit(String lower)
     {
-        COSArray arr = (COSArray) node.getDictionaryObject(COSName.LIMITS);
+        COSArray arr = node.getDictionaryObject(COSName.LIMITS, COSArray.class);
         if (arr == null)
         {
-            arr = new COSArray();
-            arr.add(null);
-            arr.add(null);
+            arr = new COSArray(null, null);
             node.setItem(COSName.LIMITS, arr);
         }
         arr.setString(0, lower);
