@@ -16,16 +16,6 @@
  */
 package org.sejda.sambox.pdmodel.common;
 
-import static java.util.Objects.nonNull;
-import static org.sejda.commons.util.RequireUtils.requireArg;
-
-import java.io.IOException;
-import java.util.Collections;
-import java.util.List;
-import java.util.Map;
-import java.util.Map.Entry;
-import java.util.TreeMap;
-
 import org.sejda.sambox.cos.COSArray;
 import org.sejda.sambox.cos.COSBase;
 import org.sejda.sambox.cos.COSDictionary;
@@ -34,9 +24,19 @@ import org.sejda.sambox.cos.COSName;
 import org.sejda.sambox.cos.COSObjectable;
 import org.sejda.sambox.pdmodel.PDDocumentCatalog;
 
+import java.io.IOException;
+import java.util.Collections;
+import java.util.List;
+import java.util.Map;
+import java.util.Map.Entry;
+import java.util.TreeMap;
+
+import static java.util.Objects.nonNull;
+import static org.sejda.commons.util.RequireUtils.requireArg;
+
 /**
  * Represents the page label dictionary of a document.
- * 
+ *
  * @author Igor Podolskiy
  */
 public class PDPageLabels implements COSObjectable
@@ -46,12 +46,12 @@ public class PDPageLabels implements COSObjectable
 
     /**
      * Creates an empty page label dictionary for the given document.
-     * 
+     *
      * <p>
-     * Note that the page label dictionary won't be automatically added to the document; you will still need to do it
-     * manually (see {@link PDDocumentCatalog#setPageLabels(PDPageLabels)}.
+     * Note that the page label dictionary won't be automatically added to the document; you will
+     * still need to do it manually (see {@link PDDocumentCatalog#setPageLabels(PDPageLabels)}.
      * </p>
-     * 
+     *
      * @see PDDocumentCatalog#setPageLabels(PDPageLabels)
      */
     public PDPageLabels()
@@ -62,43 +62,47 @@ public class PDPageLabels implements COSObjectable
     }
 
     /**
-     * Creates an page label dictionary for a document using the information in the given COS dictionary.
-     * 
+     * Creates an page label dictionary for a document using the information in the given COS
+     * dictionary.
+     *
      * <p>
-     * Note that the page label dictionary won't be automatically added to the document; you will still need to do it
-     * manually (see {@link PDDocumentCatalog#setPageLabels(PDPageLabels)}.
+     * Note that the page label dictionary won't be automatically added to the document; you will
+     * still need to do it manually (see {@link PDDocumentCatalog#setPageLabels(PDPageLabels)}.
      * </p>
-     * 
+     *
      * @param dict an existing page label dictionary
-     * @see PDDocumentCatalog#setPageLabels(PDPageLabels)
      * @throws IOException If something goes wrong during the number tree conversion.
+     * @see PDDocumentCatalog#setPageLabels(PDPageLabels)
      */
     public PDPageLabels(COSDictionary dict) throws IOException
     {
         if (nonNull(dict))
         {
-            findLabels(new PDNumberTreeNode(dict, COSDictionary.class));
+            findLabels(new PDNumberTreeNode(dict, PDPageLabelRange.class));
         }
     }
 
     private void findLabels(PDNumberTreeNode node) throws IOException
     {
-        if (node.getKids() != null)
+        List<PDNumberTreeNode> kids = node.getKids();
+        if (nonNull(kids))
         {
-            List<PDNumberTreeNode> kids = node.getKids();
             for (PDNumberTreeNode kid : kids)
             {
                 findLabels(kid);
             }
         }
-        else if (node.getNumbers() != null)
+        else
         {
             Map<Integer, COSObjectable> numbers = node.getNumbers();
-            for (Entry<Integer, COSObjectable> i : numbers.entrySet())
+            if (numbers != null)
             {
-                if (i.getKey() >= 0)
+                for (Entry<Integer, COSObjectable> i : numbers.entrySet())
                 {
-                    labels.put(i.getKey(), new PDPageLabelRange((COSDictionary) i.getValue()));
+                    if (i.getKey() >= 0)
+                    {
+                        labels.put(i.getKey(), (PDPageLabelRange) i.getValue());
+                    }
                 }
             }
         }
@@ -106,12 +110,12 @@ public class PDPageLabels implements COSObjectable
 
     /**
      * Returns the number of page label ranges.
-     * 
+     *
      * <p>
-     * This will be always &gt;= 1, as the required default entry for the page range starting at the first page is added
-     * automatically by this implementation (see PDF32000-1:2008, p. 375).
+     * This will be always &gt;= 1, as the required default entry for the page range starting at the
+     * first page is added automatically by this implementation (see PDF32000-1:2008, p. 375).
      * </p>
-     * 
+     *
      * @return the number of page label ranges.
      */
     public int getPageRangeCount()
@@ -120,10 +124,13 @@ public class PDPageLabels implements COSObjectable
     }
 
     /**
-     * Returns the page label range starting at the given page, or {@code null} if no such range is defined.
-     * 
-     * @param startPage the 0-based page index representing the start page of the page range the item is defined for.
-     * @return the page label range or {@code null} if no label range is defined for the given start page.
+     * Returns the page label range starting at the given page, or {@code null} if no such range is
+     * defined.
+     *
+     * @param startPage the 0-based page index representing the start page of the page range the
+     *                  item is defined for.
+     * @return the page label range or {@code null} if no label range is defined for the given start
+     * page.
      */
     public PDPageLabelRange getPageLabelRange(int startPage)
     {
@@ -132,9 +139,10 @@ public class PDPageLabels implements COSObjectable
 
     /**
      * Sets the page label range beginning at the specified start page.
-     * 
-     * @param startPage the 0-based index of the page representing the start of the page label range.
-     * @param item the page label item to set.
+     *
+     * @param startPage the 0-based index of the page representing the start of the page label
+     *                  range.
+     * @param item      the page label item to set.
      * @throws IllegalArgumentException if the startPage parameter is &lt; 0.
      */
     public void setLabelItem(int startPage, PDPageLabelRange item)
@@ -143,7 +151,8 @@ public class PDPageLabels implements COSObjectable
         labels.put(startPage, item);
     }
 
-    public Map<Integer, PDPageLabelRange> getLabels() {
+    public Map<Integer, PDPageLabelRange> getLabels()
+    {
         return Collections.unmodifiableMap(labels);
     }
 
