@@ -17,14 +17,6 @@
 
 package org.sejda.sambox.pdmodel.encryption;
 
-import java.io.IOException;
-import java.math.BigInteger;
-import java.security.KeyStoreException;
-import java.security.PrivateKey;
-import java.security.cert.CertificateEncodingException;
-import java.security.cert.X509Certificate;
-import java.util.Collection;
-
 import org.bouncycastle.cert.X509CertificateHolder;
 import org.bouncycastle.cms.CMSEnvelopedData;
 import org.bouncycastle.cms.CMSException;
@@ -37,15 +29,26 @@ import org.sejda.sambox.cos.COSName;
 import org.sejda.sambox.cos.COSString;
 import org.sejda.sambox.pdmodel.PDDocument;
 
+import java.io.IOException;
+import java.math.BigInteger;
+import java.security.KeyStoreException;
+import java.security.PrivateKey;
+import java.security.cert.CertificateEncodingException;
+import java.security.cert.X509Certificate;
+import java.util.Arrays;
+import java.util.Collection;
+
 /**
  * This class implements the public key security handler described in the PDF specification.
  *
- * @see PublicKeyProtectionPolicy to see how to protect document with this security handler.
  * @author Benoit Guillon
+ * @see PublicKeyProtectionPolicy to see how to protect document with this security handler.
  */
 public final class PublicKeySecurityHandler extends SecurityHandler
 {
-    /** The filter name. */
+    /**
+     * The filter name.
+     */
     public static final String FILTER = "Adobe.PubSec";
 
     private PublicKeyProtectionPolicy policy = null;
@@ -71,13 +74,14 @@ public final class PublicKeySecurityHandler extends SecurityHandler
     /**
      * Prepares everything to decrypt the document.
      *
-     * @param encryption encryption dictionary, can be retrieved via {@link PDDocument#getEncryption()}
-     * @param documentIDArray document id which is returned via
-     * {@link org.apache.pdfbox.cos.COSDocument#getDocumentID()} (not used by this handler)
+     * @param encryption         encryption dictionary, can be retrieved via {@link
+     *                           PDDocument#getEncryption()}
+     * @param documentIDArray    document id which is returned via {@link org.sejda.sambox.cos.COSDocument#getDocumentID()}
+     *                           (not used by this handler)
      * @param decryptionMaterial Information used to decrypt the document.
-     *
-     * @throws IOException If there is an error accessing data. If verbose mode is enabled, the exception message will
-     * provide more details why the match wasn't successful.
+     * @throws IOException If there is an error accessing data. If verbose mode is enabled, the
+     *                     exception message will provide more details why the match wasn't
+     *                     successful.
      */
     @Override
     public void prepareForDecryption(PDEncryption encryption, COSArray documentIDArray,
@@ -89,10 +93,16 @@ public final class PublicKeySecurityHandler extends SecurityHandler
                     "Provided decryption material is not compatible with the document");
         }
 
-        setDecryptMetadata(encryption.isEncryptMetaData());
-        if (encryption.getLength() != 0)
+        PDCryptFilterDictionary defaultCryptFilterDictionary = encryption.getDefaultCryptFilterDictionary();
+        if (defaultCryptFilterDictionary != null && defaultCryptFilterDictionary.getLength() != 0)
         {
-            this.keyLength = encryption.getLength();
+            setKeyLength(defaultCryptFilterDictionary.getLength());
+            setDecryptMetadata(defaultCryptFilterDictionary.isEncryptMetaData());
+        }
+        else if (encryption.getLength() != 0)
+        {
+            setKeyLength(encryption.getLength());
+            setDecryptMetadata(encryption.isEncryptMetaData());
         }
 
         PublicKeyDecryptionMaterial material = (PublicKeyDecryptionMaterial) decryptionMaterial;
@@ -113,12 +123,14 @@ public final class PublicKeySecurityHandler extends SecurityHandler
             byte[] envelopedData = null;
 
             // the bytes of each recipient in the recipients array
-            COSArray array = encryption.getCOSObject().getDictionaryObject(COSName.RECIPIENTS,
-                    COSArray.class);
+            COSArray array = encryption.getCOSObject().getCOSArray(COSName.RECIPIENTS);
+            if (array == null && defaultCryptFilterDictionary != null)
+            {
+                array = defaultCryptFilterDictionary.getCOSObject().getCOSArray(COSName.RECIPIENTS);
+            }
             if (array == null)
             {
-                array = encryption.getDefaultCryptFilterDictionary().getCOSObject()
-                        .getDictionaryObject(COSName.RECIPIENTS, COSArray.class);
+                throw new IOException("/Recipients entry is missing in encryption dictionary");
             }
             byte[][] recipientFieldsBytes = new byte[array.size()][];
             // TODO encryption.getRecipientsLength() and getRecipientStringAt() should be deprecated
@@ -144,8 +156,8 @@ public final class PublicKeySecurityHandler extends SecurityHandler
                         PrivateKey privateKey = (PrivateKey) material.getPrivateKey();
                         // might need to call setContentProvider() if we use PKI token, see
                         // http://bouncy-castle.1462172.n4.nabble.com/CMSException-exception-unwrapping-key-key-invalid-unknown-key-type-passed-to-RSA-td4658109.html
-                        envelopedData = ri
-                                .getContent(new JceKeyTransEnvelopedRecipient(privateKey));
+                        envelopedData = ri.getContent(
+                                new JceKeyTransEnvelopedRecipient(privateKey));
                         break;
                     }
                     j++;
@@ -166,8 +178,9 @@ public final class PublicKeySecurityHandler extends SecurityHandler
             }
             if (!foundRecipient || envelopedData == null)
             {
-                throw new IOException("The certificate matches none of " + array.size()
-                        + " recipient entries" + extraInfo.toString());
+                throw new IOException(
+                        "The certificate matches none of " + array.size() + " recipient entries"
+                                + extraInfo.toString());
             }
             if (envelopedData.length != 24)
             {
@@ -202,18 +215,32 @@ public final class PublicKeySecurityHandler extends SecurityHandler
             byte[] mdResult;
             if (encryption.getVersion() == 4 || encryption.getVersion() == 5)
             {
-                mdResult = MessageDigests.getSHA256().digest(sha1Input);
+                if (!isDecryptMetadata())
+                {
+                    // "4 bytes with the value 0xFF if the key being generated is intended for use in
+                    // document-level encryption and the document metadata is being left as plaintext"
+                    sha1Input = Arrays.copyOf(sha1Input, sha1Input.length + 4);
+                    System.arraycopy(
+                            new byte[] { (byte) 0xff, (byte) 0xff, (byte) 0xff, (byte) 0xff }, 0,
+                            sha1Input, sha1Input.length - 4, 4);
+                }
+                if (encryption.getVersion() == 4)
+                {
+                    mdResult = MessageDigests.getSHA1().digest(sha1Input);
+                }
+                else
+                {
+                    mdResult = MessageDigests.getSHA256().digest(sha1Input);
+                }
 
                 // detect whether AES encryption is used. This assumes that the encryption algo is
                 // stored in the PDCryptFilterDictionary
                 // However, crypt filters are used only when V is 4 or 5.
-                PDCryptFilterDictionary defaultCryptFilterDictionary = encryption
-                        .getDefaultCryptFilterDictionary();
                 if (defaultCryptFilterDictionary != null)
                 {
                     COSName cryptFilterMethod = defaultCryptFilterDictionary.getCryptFilterMethod();
-                    setAES(COSName.AESV2.equals(cryptFilterMethod)
-                            || COSName.AESV3.equals(cryptFilterMethod));
+                    setAES(COSName.AESV2.equals(cryptFilterMethod) || COSName.AESV3.equals(
+                            cryptFilterMethod));
                 }
             }
             else
@@ -225,15 +252,7 @@ public final class PublicKeySecurityHandler extends SecurityHandler
             setEncryptionKey(new byte[this.keyLength / 8]);
             System.arraycopy(mdResult, 0, getEncryptionKey(), 0, this.keyLength / 8);
         }
-        catch (CMSException e)
-        {
-            throw new IOException(e);
-        }
-        catch (KeyStoreException e)
-        {
-            throw new IOException(e);
-        }
-        catch (CertificateEncodingException e)
+        catch (CMSException | KeyStoreException | CertificateEncodingException e)
         {
             throw new IOException(e);
         }
@@ -262,13 +281,4 @@ public final class PublicKeySecurityHandler extends SecurityHandler
             extraInfo.append("\' ");
         }
     }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public boolean hasProtectionPolicy()
-    {
-        return policy != null;
-    }
 }
