@@ -16,7 +16,9 @@
  */
 package org.sejda.sambox.pdmodel.encryption;
 
+import static java.util.Objects.nonNull;
 import static org.bouncycastle.util.Arrays.copyOf;
+import static org.sejda.commons.util.RequireUtils.requireIOCondition;
 
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
@@ -26,13 +28,8 @@ import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import java.security.GeneralSecurityException;
 import java.security.MessageDigest;
-import java.security.NoSuchAlgorithmException;
 import java.util.Arrays;
 
-import javax.crypto.Cipher;
-import javax.crypto.spec.IvParameterSpec;
-import javax.crypto.spec.SecretKeySpec;
-
 import org.bouncycastle.crypto.BufferedBlockCipher;
 import org.bouncycastle.crypto.DataLengthException;
 import org.bouncycastle.crypto.InvalidCipherTextException;
@@ -44,29 +41,35 @@ import org.sejda.sambox.cos.COSName;
 import org.sejda.sambox.cos.COSString;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import javax.crypto.Cipher;
+import javax.crypto.spec.IvParameterSpec;
+import javax.crypto.spec.SecretKeySpec;
 
 /**
  * The standard security handler. This security handler protects document with password.
- * 
- * @see StandardProtectionPolicy to see how to protect document with this security handler.
+ *
  * @author Ben Litchfield
  * @author Benoit Guillon
  * @author Manuel Kasper
+ * @see StandardProtectionPolicy to see how to protect document with this security handler.
  */
 public final class StandardSecurityHandler extends SecurityHandler
 {
-    /**
-     * Log instance.
-     */
     private static final Logger LOG = LoggerFactory.getLogger(StandardSecurityHandler.class);
 
-    /** Type of security handler. */
+    /**
+     * Type of security handler.
+     */
     public static final String FILTER = "Standard";
 
-    /** Protection policy class for this handler. */
+    /**
+     * Protection policy class for this handler.
+     */
     public static final Class<?> PROTECTION_POLICY_CLASS = StandardProtectionPolicy.class;
 
-    /** Standard padding for encryption. */
+    /**
+     * Standard padding for encryption.
+     */
     private static final byte[] ENCRYPT_PADDING = { (byte) 0x28, (byte) 0xBF, (byte) 0x4E,
             (byte) 0x5E, (byte) 0x4E, (byte) 0x75, (byte) 0x8A, (byte) 0x41, (byte) 0x64,
             (byte) 0x00, (byte) 0x4E, (byte) 0x56, (byte) 0xFF, (byte) 0xFA, (byte) 0x01,
@@ -77,8 +80,6 @@ public final class StandardSecurityHandler extends SecurityHandler
     // hashes used for Algorithm 2.B, depending on remainder from E modulo 3
     private static final String[] HASHES_2B = new String[] { "SHA-256", "SHA-384", "SHA-512" };
 
-    private StandardProtectionPolicy policy;
-
     /**
      * Constructor.
      */
@@ -86,28 +87,16 @@ public final class StandardSecurityHandler extends SecurityHandler
     {
     }
 
-    /**
-     * Constructor used for encryption.
-     *
-     * @param p The protection policy.
-     */
-    public StandardSecurityHandler(StandardProtectionPolicy p)
-    {
-        policy = p;
-        keyLength = policy.getEncryptionKeyLength();
-    }
-
     /**
      * Prepares everything to decrypt the document.
-     *
+     * <p>
      * Only if decryption of single objects is needed this should be called.
      *
-     * @param encryption encryption dictionary
-     * @param documentIDArray document id
+     * @param encryption         encryption dictionary
+     * @param documentIDArray    document id
      * @param decryptionMaterial Information used to decrypt the document.
-     *
      * @throws InvalidPasswordException If the password is incorrect.
-     * @throws IOException If there is an error accessing data.
+     * @throws IOException              If there is an error accessing data.
      */
     @Override
     public void prepareForDecryption(PDEncryption encryption, COSArray documentIDArray,
@@ -122,7 +111,7 @@ public final class StandardSecurityHandler extends SecurityHandler
         if (encryption.getVersion() >= 4)
         {
             setStreamFilterName(encryption.getStreamFilterName());
-            setStringFilterName(encryption.getStreamFilterName());
+            setStringFilterName(encryption.getStringFilterName());
         }
 
         setDecryptMetadata(encryption.isEncryptMetaData());
@@ -138,6 +127,50 @@ public final class StandardSecurityHandler extends SecurityHandler
         int dicRevision = encryption.getRevision();
         int dicLength = encryption.getVersion() == 1 ? 5 : encryption.getLength() / 8;
 
+        if (encryption.getVersion() == 4 || encryption.getVersion() == 5)
+        {
+            // detect whether AES encryption is used. This assumes that the encryption algo is
+            // stored in the PDCryptFilterDictionary
+            // However, crypt filters are used only when V is 4 or 5.
+            PDCryptFilterDictionary stdCryptFilterDictionary = encryption.getStdCryptFilterDictionary();
+            if (stdCryptFilterDictionary != null)
+            {
+                COSName cryptFilterMethod = stdCryptFilterDictionary.getCryptFilterMethod();
+                if (COSName.AESV2.equals(cryptFilterMethod))
+                {
+                    dicLength = 128 / 8;
+                    setAES(true);
+                    if (encryption.getCOSObject().containsKey(COSName.LENGTH))
+                    {
+                        // PDFBOX-5345
+                        int newLength = encryption.getLength() / 8;
+                        if (newLength < dicLength)
+                        {
+                            LOG.warn("Using {} bytes key length instead of {} in AESV2 encryption",
+                                    newLength, dicLength);
+                            dicLength = newLength;
+                        }
+                    }
+                }
+                if (COSName.AESV3.equals(cryptFilterMethod))
+                {
+                    dicLength = 256 / 8;
+                    setAES(true);
+                    if (encryption.getCOSObject().containsKey(COSName.LENGTH))
+                    {
+                        // PDFBOX-5345
+                        int newLength = encryption.getLength() / 8;
+                        if (newLength < dicLength)
+                        {
+                            LOG.warn("Using {} bytes key length instead of {} in AESV3 encryption",
+                                    newLength, dicLength);
+                            dicLength = newLength;
+                        }
+                    }
+                }
+            }
+        }
+
         byte[] documentIDBytes = getDocumentIDBytes(documentIDArray);
 
         // we need to know whether the meta data was encrypted for password calculation
@@ -189,9 +222,10 @@ public final class StandardSecurityHandler extends SecurityHandler
             currentAccessPermission = new AccessPermission(dicPermissions);
             setCurrentAccessPermission(currentAccessPermission);
 
-            setEncryptionKey(computeEncryptedKey(password.getBytes(passwordCharset), ownerKey,
-                    userKey, oe, ue, dicPermissions, documentIDBytes, dicRevision, dicLength,
-                    encryptMetadata, false));
+            setEncryptionKey(
+                    computeEncryptedKey(password.getBytes(passwordCharset), ownerKey, userKey, oe,
+                            ue, dicPermissions, documentIDBytes, dicRevision, dicLength,
+                            encryptMetadata, false));
         }
         else
         {
@@ -202,25 +236,6 @@ public final class StandardSecurityHandler extends SecurityHandler
         {
             validatePerms(encryption, dicPermissions, encryptMetadata);
         }
-
-        if (encryption.getVersion() == 4 || encryption.getVersion() == 5)
-        {
-            // detect whether AES encryption is used. This assumes that the encryption algo is
-            // stored in the PDCryptFilterDictionary
-            // However, crypt filters are used only when V is 4 or 5.
-            PDCryptFilterDictionary stdCryptFilterDictionary = encryption
-                    .getStdCryptFilterDictionary();
-
-            if (stdCryptFilterDictionary != null)
-            {
-                COSName cryptFilterMethod = stdCryptFilterDictionary.getCryptFilterMethod();
-                if (cryptFilterMethod != null)
-                {
-                    setAES("AESV2".equalsIgnoreCase(cryptFilterMethod.getName())
-                            || "AESV3".equalsIgnoreCase(cryptFilterMethod.getName()));
-                }
-            }
-        }
     }
 
     private byte[] getDocumentIDBytes(COSArray documentIDArray)
@@ -283,17 +298,15 @@ public final class StandardSecurityHandler extends SecurityHandler
     /**
      * Check for owner password.
      *
-     * @param ownerPassword The owner password.
-     * @param user The u entry of the encryption dictionary.
-     * @param owner The o entry of the encryption dictionary.
-     * @param permissions The set of permissions on the document.
-     * @param id The document id.
-     * @param encRevision The encryption algorithm revision.
-     * @param length The encryption key length.
+     * @param ownerPassword   The owner password.
+     * @param user            The u entry of the encryption dictionary.
+     * @param owner           The o entry of the encryption dictionary.
+     * @param permissions     The set of permissions on the document.
+     * @param id              The document id.
+     * @param encRevision     The encryption algorithm revision.
+     * @param length          The encryption key length.
      * @param encryptMetadata The encryption metadata
-     *
      * @return True If the ownerPassword param is the owner password.
-     *
      * @throws IOException If there is an error accessing data.
      */
     public boolean isOwnerPassword(byte[] ownerPassword, byte[] user, byte[] owner, int permissions,
@@ -305,6 +318,7 @@ public final class StandardSecurityHandler extends SecurityHandler
 
             byte[] oHash = new byte[32];
             byte[] oValidationSalt = new byte[8];
+            requireIOCondition(owner.length >= 40, "Owner password is too short");
             System.arraycopy(owner, 0, oHash, 0, 32);
             System.arraycopy(owner, 32, oValidationSalt, 0, 8);
 
@@ -332,12 +346,10 @@ public final class StandardSecurityHandler extends SecurityHandler
      * Get the user password based on the owner password.
      *
      * @param ownerPassword The plaintext owner password.
-     * @param owner The o entry of the encryption dictionary.
-     * @param encRevision The encryption revision number.
-     * @param length The key length.
-     *
+     * @param owner         The o entry of the encryption dictionary.
+     * @param encRevision   The encryption revision number.
+     * @param length        The key length.
      * @return The u entry of the encryption dictionary.
-     *
      * @throws IOException If there is an error accessing data while generating the user password.
      */
     public byte[] getUserPassword(byte[] ownerPassword, byte[] owner, int encRevision, int length)
@@ -374,20 +386,18 @@ public final class StandardSecurityHandler extends SecurityHandler
     /**
      * Compute the encryption key.
      *
-     * @param password The password to compute the encrypted key.
-     * @param o The O entry of the encryption dictionary.
-     * @param u The U entry of the encryption dictionary.
-     * @param oe The OE entry of the encryption dictionary.
-     * @param ue The UE entry of the encryption dictionary.
-     * @param permissions The permissions for the document.
-     * @param id The document id.
-     * @param encRevision The revision of the encryption algorithm.
-     * @param length The length of the encryption key.
+     * @param password        The password to compute the encrypted key.
+     * @param o               The O entry of the encryption dictionary.
+     * @param u               The U entry of the encryption dictionary.
+     * @param oe              The OE entry of the encryption dictionary.
+     * @param ue              The UE entry of the encryption dictionary.
+     * @param permissions     The permissions for the document.
+     * @param id              The document id.
+     * @param encRevision     The revision of the encryption algorithm.
+     * @param length          The length of the encryption key.
      * @param encryptMetadata The encryption metadata
      * @param isOwnerPassword whether the password given is the owner password (for revision 6)
-     *
      * @return The encrypted key bytes.
-     *
      * @throws IOException If there is an error with encryption.
      */
     public byte[] computeEncryptedKey(byte[] password, byte[] o, byte[] u, byte[] oe, byte[] ue,
@@ -455,6 +465,7 @@ public final class StandardSecurityHandler extends SecurityHandler
 
         if (isOwnerPassword)
         {
+            requireIOCondition(nonNull(oe), "/Encrypt/OE entry is missing");
             byte[] oKeySalt = new byte[8];
             System.arraycopy(o, 40, oKeySalt, 0, 8);
 
@@ -471,6 +482,7 @@ public final class StandardSecurityHandler extends SecurityHandler
         }
         else
         {
+            requireIOCondition(nonNull(ue), "/Encrypt/UE entry is missing");
             byte[] uKeySalt = new byte[8];
             System.arraycopy(u, 40, uKeySalt, 0, 8);
 
@@ -504,16 +516,14 @@ public final class StandardSecurityHandler extends SecurityHandler
     /**
      * This will compute the user password hash.
      *
-     * @param password The plain text password.
-     * @param owner The owner password hash.
-     * @param permissions The document permissions.
-     * @param id The document id.
-     * @param encRevision The revision of the encryption.
-     * @param length The length of the encryption key.
+     * @param password        The plain text password.
+     * @param owner           The owner password hash.
+     * @param permissions     The document permissions.
+     * @param id              The document id.
+     * @param encRevision     The revision of the encryption.
+     * @param length          The length of the encryption key.
      * @param encryptMetadata The encryption metadata
-     *
      * @return The user password.
-     *
      * @throws IOException if the password could not be computed
      */
     public byte[] computeUserPassword(byte[] password, byte[] owner, int permissions, byte[] id,
@@ -561,12 +571,10 @@ public final class StandardSecurityHandler extends SecurityHandler
      * Compute the owner entry in the encryption dictionary.
      *
      * @param ownerPassword The plaintext owner password.
-     * @param userPassword The plaintext user password.
-     * @param encRevision The revision number of the encryption algorithm.
-     * @param length The length of the encryption key.
-     *
+     * @param userPassword  The plaintext user password.
+     * @param encRevision   The revision number of the encryption algorithm.
+     * @param length        The length of the encryption key.
      * @return The o entry of the encryption dictionary.
-     *
      * @throws IOException if the owner password could not be computed
      */
     public byte[] computeOwnerPassword(byte[] ownerPassword, byte[] userPassword, int encRevision,
@@ -627,7 +635,6 @@ public final class StandardSecurityHandler extends SecurityHandler
      * This will take the password and truncate or pad it as necessary.
      *
      * @param password The password to pad or truncate.
-     *
      * @return The padded or truncated password.
      */
     private byte[] truncateOrPad(byte[] password)
@@ -643,17 +650,15 @@ public final class StandardSecurityHandler extends SecurityHandler
     /**
      * Check if a plaintext password is the user password.
      *
-     * @param password The plaintext password.
-     * @param user The u entry of the encryption dictionary.
-     * @param owner The o entry of the encryption dictionary.
-     * @param permissions The permissions set in the PDF.
-     * @param id The document id used for encryption.
-     * @param encRevision The revision of the encryption algorithm.
-     * @param length The length of the encryption key.
+     * @param password        The plaintext password.
+     * @param user            The u entry of the encryption dictionary.
+     * @param owner           The o entry of the encryption dictionary.
+     * @param permissions     The permissions set in the PDF.
+     * @param id              The document id used for encryption.
+     * @param encRevision     The revision of the encryption algorithm.
+     * @param length          The length of the encryption key.
      * @param encryptMetadata The encryption metadata
-     *
      * @return true If the plaintext password is the user password.
-     *
      * @throws IOException If there is an error accessing data.
      */
     public boolean isUserPassword(byte[] password, byte[] user, byte[] owner, int permissions,
@@ -702,17 +707,15 @@ public final class StandardSecurityHandler extends SecurityHandler
     /**
      * Check if a plaintext password is the user password.
      *
-     * @param password The plaintext password.
-     * @param user The u entry of the encryption dictionary.
-     * @param owner The o entry of the encryption dictionary.
-     * @param permissions The permissions set in the PDF.
-     * @param id The document id used for encryption.
-     * @param encRevision The revision of the encryption algorithm.
-     * @param length The length of the encryption key.
+     * @param password        The plaintext password.
+     * @param user            The u entry of the encryption dictionary.
+     * @param owner           The o entry of the encryption dictionary.
+     * @param permissions     The permissions set in the PDF.
+     * @param id              The document id used for encryption.
+     * @param encRevision     The revision of the encryption algorithm.
+     * @param length          The length of the encryption key.
      * @param encryptMetadata The encryption metadata
-     *
      * @return true If the plaintext password is the user password.
-     *
      * @throws IOException If there is an error accessing data.
      */
     public boolean isUserPassword(String password, byte[] user, byte[] owner, int permissions,
@@ -733,17 +736,15 @@ public final class StandardSecurityHandler extends SecurityHandler
     /**
      * Check for owner password.
      *
-     * @param password The owner password.
-     * @param user The u entry of the encryption dictionary.
-     * @param owner The o entry of the encryption dictionary.
-     * @param permissions The set of permissions on the document.
-     * @param id The document id.
-     * @param encRevision The encryption algorithm revision.
-     * @param length The encryption key length.
+     * @param password        The owner password.
+     * @param user            The u entry of the encryption dictionary.
+     * @param owner           The o entry of the encryption dictionary.
+     * @param permissions     The set of permissions on the document.
+     * @param id              The document id.
+     * @param encRevision     The encryption algorithm revision.
+     * @param length          The encryption key length.
      * @param encryptMetadata The encryption metadata
-     *
      * @return True If the ownerPassword param is the owner password.
-     *
      * @throws IOException If there is an error accessing data.
      */
     public boolean isOwnerPassword(String password, byte[] user, byte[] owner, int permissions,
@@ -787,7 +788,7 @@ public final class StandardSecurityHandler extends SecurityHandler
     {
         try
         {
-            MessageDigest md = MessageDigest.getInstance("SHA-256");
+            MessageDigest md = MessageDigests.getSHA256();
             byte[] k = md.digest(input);
 
             byte[] e = null;
@@ -853,19 +854,11 @@ public final class StandardSecurityHandler extends SecurityHandler
     }
 
     private static byte[] computeSHA256(byte[] input, byte[] password, byte[] userKey)
-            throws IOException
     {
-        try
-        {
-            MessageDigest md = MessageDigest.getInstance("SHA-256");
-            md.update(input);
-            md.update(password);
-            return userKey == null ? md.digest() : md.digest(userKey);
-        }
-        catch (NoSuchAlgorithmException e)
-        {
-            throw new IOException(e);
-        }
+        MessageDigest md = MessageDigests.getSHA256();
+        md.update(input);
+        md.update(password);
+        return userKey == null ? md.digest() : md.digest(userKey);
     }
 
     private static byte[] concat(byte[] a, byte[] b, byte[] c)
@@ -888,10 +881,4 @@ public final class StandardSecurityHandler extends SecurityHandler
         return trunc;
     }
 
-    @Override
-    public boolean hasProtectionPolicy()
-    {
-        return policy != null;
-    }
-
 }
