@@ -16,6 +16,8 @@
  */
 package org.sejda.sambox.pdmodel.font;
 
+import static java.util.Objects.nonNull;
+
 import java.io.BufferedReader;
 import java.io.BufferedWriter;
 import java.io.File;
@@ -36,6 +38,7 @@ import org.apache.fontbox.FontBoxFont;
 import org.apache.fontbox.cff.CFFCIDFont;
 import org.apache.fontbox.cff.CFFFont;
 import org.apache.fontbox.ttf.NamingTable;
+import org.apache.fontbox.ttf.OS2WindowsMetricsTable;
 import org.apache.fontbox.ttf.OTFParser;
 import org.apache.fontbox.ttf.OpenTypeFont;
 import org.apache.fontbox.ttf.TTFParser;
@@ -91,7 +94,9 @@ final class FileSystemFontProvider extends FontProvider
             this.ulCodePageRange1 = ulCodePageRange1;
             this.ulCodePageRange2 = ulCodePageRange2;
             this.macStyle = macStyle;
-            this.panose = panose != null ? new PDPanoseClassification(panose) : null;
+            this.panose = panose != null
+                    && panose.length >= PDPanoseClassification.LENGTH ? new PDPanoseClassification(
+                    panose) : null;
             this.parent = parent;
         }
 
@@ -114,7 +119,7 @@ final class FileSystemFontProvider extends FontProvider
         }
 
         @Override
-        public FontBoxFont getFont()
+        public synchronized FontBoxFont getFont()
         {
             FontBoxFont cached = parent.cache.getFont(this);
             if (cached != null)
@@ -196,7 +201,7 @@ final class FileSystemFontProvider extends FontProvider
             }
             catch (NullPointerException | IOException e) // TTF parser is buggy
             {
-                LOG.error("Could not load font file: " + file, e);
+                LOG.warn("Could not load font file: " + file, e);
             }
             return null;
         }
@@ -209,7 +214,16 @@ final class FileSystemFontProvider extends FontProvider
                 // ttc not closed here because it is needed later when ttf is accessed,
                 // e.g. rendering PDF with non-embedded font which is in ttc file in our font directory
                 TrueTypeCollection ttc = new TrueTypeCollection(file);
-                TrueTypeFont ttf = ttc.getFontByName(postScriptName);
+                TrueTypeFont ttf = null;
+                try
+                {
+                    ttf = ttc.getFontByName(postScriptName);
+                }
+                catch (IOException ex)
+                {
+                    ttc.close();
+                    throw ex;
+                }
                 if (ttf == null)
                 {
                     ttc.close();
@@ -225,7 +239,30 @@ final class FileSystemFontProvider extends FontProvider
         {
             try
             {
-                // todo JH: we don't yet support loading CFF fonts from OTC collections
+                if (file.getName().toLowerCase().endsWith(".ttc"))
+                {
+                    @SuppressWarnings("squid:S2095")
+                    // ttc not closed here because it is needed later when ttf is accessed,
+                    // e.g. rendering PDF with non-embedded font which is in ttc file in our font directory
+                    TrueTypeCollection ttc = new TrueTypeCollection(file);
+                    TrueTypeFont ttf = null;
+                    try
+                    {
+                        ttf = ttc.getFontByName(postScriptName);
+                    }
+                    catch (IOException ex)
+                    {
+                        LOG.error(ex.getMessage(), ex);
+                        ttc.close();
+                        return null;
+                    }
+                    if (ttf == null)
+                    {
+                        ttc.close();
+                        throw new IOException("Font " + postScriptName + " not found in " + file);
+                    }
+                    return (OpenTypeFont) ttf;
+                }
                 OTFParser parser = new OTFParser(false, true);
                 OpenTypeFont otf = parser.parse(file);
                 LOG.debug("Loaded {} from {}", postScriptName, file);
@@ -233,7 +270,7 @@ final class FileSystemFontProvider extends FontProvider
             }
             catch (IOException e)
             {
-                LOG.error("Could not load font file: " + file, e);
+                LOG.warn("Could not load font file: " + file, e);
             }
             return null;
         }
@@ -248,7 +285,7 @@ final class FileSystemFontProvider extends FontProvider
             }
             catch (IOException e)
             {
-                LOG.error("Could not load font file " + file, e);
+                LOG.warn("Could not load font file " + file, e);
             }
             return null;
         }
@@ -300,21 +337,24 @@ final class FileSystemFontProvider extends FontProvider
                 files.add(new File(font));
             }
 
-            LOG.trace("Found " + files.size() + " fonts on the local system");
+            LOG.trace("Found {} fonts on the local system", files.size());
 
-            // load cached FontInfo objects
-            List<FSFontInfo> cachedInfos = loadDiskCache(files);
-            if (cachedInfos != null && !cachedInfos.isEmpty())
+            if (!files.isEmpty())
             {
-                fontInfoList.addAll(cachedInfos);
-            }
-            else
-            {
-                LOG.warn("Building on-disk font cache, this may take a while");
-                scanFonts(files);
-                saveDiskCache();
-                LOG.warn("Finished building on-disk font cache, found " + fontInfoList.size()
-                        + " fonts");
+                // load cached FontInfo objects
+                List<FSFontInfo> cachedInfos = loadDiskCache(files);
+                if (cachedInfos != null && !cachedInfos.isEmpty())
+                {
+                    fontInfoList.addAll(cachedInfos);
+                }
+                else
+                {
+                    LOG.warn("Building on-disk font cache, this may take a while");
+                    scanFonts(files);
+                    saveDiskCache();
+                    LOG.warn("Finished building on-disk font cache, found {} fonts",
+                            fontInfoList.size());
+                }
             }
         }
         catch (AccessControlException e)
@@ -329,24 +369,24 @@ final class FileSystemFontProvider extends FontProvider
         {
             try
             {
-                if (file.getPath().toLowerCase().endsWith(".ttf")
-                        || file.getPath().toLowerCase().endsWith(".otf"))
+                String filePath = file.getPath().toLowerCase();
+                if (filePath.endsWith(".ttf") || filePath.endsWith(".otf"))
                 {
                     addTrueTypeFont(file);
                 }
-                else if (file.getPath().toLowerCase().endsWith(".ttc")
-                        || file.getPath().toLowerCase().endsWith(".otc"))
+                else if (filePath.toLowerCase().endsWith(".ttc") || filePath.toLowerCase()
+                        .endsWith(".otc"))
                 {
                     addTrueTypeCollection(file);
                 }
-                else if (file.getPath().toLowerCase().endsWith(".pfb"))
+                else if (filePath.toLowerCase().endsWith(".pfb"))
                 {
                     addType1Font(file);
                 }
             }
             catch (Exception e) // TTF parser is buggy
             {
-                LOG.error("Unable to load font file: " + file, e);
+                LOG.warn("Unable to load font file: " + file, e);
             }
         }
     }
@@ -465,7 +505,7 @@ final class FileSystemFontProvider extends FontProvider
                     String[] parts = line.split("(?<!\\\\)\\|", 10);
                     if (parts.length < 10)
                     {
-                        LOG.error("Incorrect line '" + line + "' in font disk cache is skipped");
+                        LOG.warn("Incorrect line '" + line + "' in font disk cache is skipped");
                         continue;
                     }
 
@@ -530,7 +570,7 @@ final class FileSystemFontProvider extends FontProvider
             }
             catch (IOException e)
             {
-                LOG.error("Error loading font cache, will be re-built", e);
+                LOG.warn("Error loading font cache, will be re-built", e);
                 return null;
             }
         }
@@ -547,7 +587,7 @@ final class FileSystemFontProvider extends FontProvider
 
     /**
      * Adds a TTC or OTC to the file cache. To reduce memory, the parsed font is not cached.
-     * 
+     *
      * @throws IOException
      */
     private void addTrueTypeCollection(final File ttcFile) throws IOException
@@ -567,12 +607,12 @@ final class FileSystemFontProvider extends FontProvider
 
     /**
      * Adds an OTF or TTF font to the file cache. To reduce memory, the parsed font is not cached.
-     * 
+     *
      * @throws IOException
      */
     private void addTrueTypeFont(File ttfFile) throws IOException
     {
-        if (ttfFile.getPath().endsWith(".otf"))
+        if (ttfFile.getPath().toLowerCase().endsWith(".otf"))
         {
             OTFParser parser = new OTFParser(false, true);
             OpenTypeFont otf = parser.parse(ttfFile);
@@ -610,14 +650,16 @@ final class FileSystemFontProvider extends FontProvider
                 int ulCodePageRange2 = 0;
                 byte[] panose = null;
 
+                OS2WindowsMetricsTable os2WindowsMetricsTable = ttf.getOS2Windows();
+
                 // Apple's AAT fonts don't have an OS/2 table
-                if (ttf.getOS2Windows() != null)
+                if (nonNull(os2WindowsMetricsTable))
                 {
-                    sFamilyClass = ttf.getOS2Windows().getFamilyClass();
-                    usWeightClass = ttf.getOS2Windows().getWeightClass();
-                    ulCodePageRange1 = (int) ttf.getOS2Windows().getCodePageRange1();
-                    ulCodePageRange2 = (int) ttf.getOS2Windows().getCodePageRange2();
-                    panose = ttf.getOS2Windows().getPanose();
+                    sFamilyClass = os2WindowsMetricsTable.getFamilyClass();
+                    usWeightClass = os2WindowsMetricsTable.getWeightClass();
+                    ulCodePageRange1 = (int) os2WindowsMetricsTable.getCodePageRange1();
+                    ulCodePageRange2 = (int) os2WindowsMetricsTable.getCodePageRange2();
+                    panose = os2WindowsMetricsTable.getPanose();
                 }
 
                 String format;
@@ -634,9 +676,10 @@ final class FileSystemFontProvider extends FontProvider
                         int supplement = cidFont.getSupplement();
                         ros = new CIDSystemInfo(registry, ordering, supplement);
                     }
-                    fontInfoList.add(new FSFontInfo(file, FontFormat.OTF, ttf.getName(), ros,
-                            usWeightClass, sFamilyClass, ulCodePageRange1, ulCodePageRange2,
-                            macStyle, panose, this));
+                    fontInfoList.add(
+                            new FSFontInfo(file, FontFormat.OTF, ttf.getName(), ros, usWeightClass,
+                                    sFamilyClass, ulCodePageRange1, ulCodePageRange2, macStyle,
+                                    panose, this));
                 }
                 else
                 {
@@ -654,9 +697,10 @@ final class FileSystemFontProvider extends FontProvider
                     }
 
                     format = "TTF";
-                    fontInfoList.add(new FSFontInfo(file, FontFormat.TTF, ttf.getName(), ros,
-                            usWeightClass, sFamilyClass, ulCodePageRange1, ulCodePageRange2,
-                            macStyle, panose, this));
+                    fontInfoList.add(
+                            new FSFontInfo(file, FontFormat.TTF, ttf.getName(), ros, usWeightClass,
+                                    sFamilyClass, ulCodePageRange1, ulCodePageRange2, macStyle,
+                                    panose, this));
                 }
 
                 if (LOG.isTraceEnabled())
@@ -678,7 +722,7 @@ final class FileSystemFontProvider extends FontProvider
         catch (IOException e)
         {
             fontInfoList.add(new FSIgnored(file, FontFormat.TTF, "*skipexception*"));
-            LOG.error("Could not load font file: " + file, e);
+            LOG.warn("Could not load font file: " + file, e);
         }
         finally
         {
@@ -688,7 +732,7 @@ final class FileSystemFontProvider extends FontProvider
 
     /**
      * Adds a Type 1 font to the file cache. To reduce memory, the parsed font is not cached.
-     * 
+     *
      * @throws IOException
      */
     private void addType1Font(File pfbFile) throws IOException
@@ -696,8 +740,9 @@ final class FileSystemFontProvider extends FontProvider
         try (InputStream input = new FileInputStream(pfbFile))
         {
             Type1Font type1 = Type1Font.createWithPFB(input);
-            fontInfoList.add(new FSFontInfo(pfbFile, FontFormat.PFB, type1.getName(), null, -1, -1,
-                    0, 0, -1, null, this));
+            fontInfoList.add(
+                    new FSFontInfo(pfbFile, FontFormat.PFB, type1.getName(), null, -1, -1, 0, 0, -1,
+                            null, this));
 
             if (LOG.isTraceEnabled())
             {
