@@ -16,9 +16,14 @@
  */
 package org.sejda.sambox.pdmodel.font;
 
-import static java.util.Optional.ofNullable;
-import static org.sejda.commons.util.RequireUtils.requireNotNullArg;
+import org.apache.fontbox.FontBoxFont;
+import org.apache.fontbox.ttf.OpenTypeFont;
+import org.apache.fontbox.ttf.TTFParser;
+import org.apache.fontbox.ttf.TrueTypeFont;
+import org.apache.fontbox.type1.Type1Font;
+import org.sejda.sambox.SAMBox;
 
+import java.io.BufferedInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.ArrayList;
@@ -32,12 +37,8 @@ import java.util.Map;
 import java.util.PriorityQueue;
 import java.util.Set;
 
-import org.apache.fontbox.FontBoxFont;
-import org.apache.fontbox.ttf.OpenTypeFont;
-import org.apache.fontbox.ttf.TTFParser;
-import org.apache.fontbox.ttf.TrueTypeFont;
-import org.apache.fontbox.type1.Type1Font;
-import org.sejda.sambox.SAMBox;
+import static java.util.Optional.ofNullable;
+import static org.sejda.commons.util.RequireUtils.requireNotNullArg;
 
 /**
  * Font mapper, locates non-embedded fonts via a pluggable FontProvider.
@@ -51,43 +52,56 @@ final class FontMapperImpl implements FontMapper
     private Map<String, FontInfo> fontInfoByName;
     private final TrueTypeFont lastResortFont;
 
-    /** Map of PostScript name substitutes, in priority order. */
+    /**
+     * Map of PostScript name substitutes, in priority order.
+     */
     private final Map<String, List<String>> substitutes = new HashMap<>();
 
     FontMapperImpl()
     {
         // substitutes for standard 14 fonts
-        substitutes.put("Courier",
-                Arrays.asList("CourierNew", "CourierNewPSMT", "LiberationMono", "NimbusMonL-Regu"));
-        substitutes.put("Courier-Bold", Arrays.asList("CourierNewPS-BoldMT", "CourierNew-Bold",
-                "LiberationMono-Bold", "NimbusMonL-Bold"));
-        substitutes.put("Courier-Oblique", Arrays.asList("CourierNewPS-ItalicMT",
-                "CourierNew-Italic", "LiberationMono-Italic", "NimbusMonL-ReguObli"));
-        substitutes.put("Courier-BoldOblique", Arrays.asList("CourierNewPS-BoldItalicMT",
-                "CourierNew-BoldItalic", "LiberationMono-BoldItalic", "NimbusMonL-BoldObli"));
-        substitutes.put("Helvetica",
-                Arrays.asList("ArialMT", "Arial", "LiberationSans", "NimbusSanL-Regu"));
-        substitutes.put("Helvetica-Bold", Arrays.asList("Arial-BoldMT", "Arial-Bold",
-                "LiberationSans-Bold", "NimbusSanL-Bold"));
-        substitutes.put("Helvetica-Oblique", Arrays.asList("Arial-ItalicMT", "Arial-Italic",
-                "Helvetica-Italic", "LiberationSans-Italic", "NimbusSanL-ReguItal"));
-        substitutes.put("Helvetica-BoldOblique", Arrays.asList("Arial-BoldItalicMT",
-                "Helvetica-BoldItalic", "LiberationSans-BoldItalic", "NimbusSanL-BoldItal"));
-        substitutes.put("Times-Roman", Arrays.asList("TimesNewRomanPSMT", "TimesNewRoman",
-                "TimesNewRomanPS", "LiberationSerif", "NimbusRomNo9L-Regu"));
-        substitutes.put("Times-Bold",
+        substitutes.put("Courier", new ArrayList<>(
+                Arrays.asList("CourierNew", "CourierNewPSMT", "LiberationMono",
+                        "NimbusMonL-Regu")));
+        substitutes.put("Courier-Bold", new ArrayList<>(
+                Arrays.asList("CourierNewPS-BoldMT", "CourierNew-Bold", "LiberationMono-Bold",
+                        "NimbusMonL-Bold")));
+        substitutes.put("Courier-Oblique", new ArrayList<>(
+                Arrays.asList("CourierNewPS-ItalicMT", "CourierNew-Italic", "LiberationMono-Italic",
+                        "NimbusMonL-ReguObli")));
+        substitutes.put("Courier-BoldOblique", new ArrayList<>(
+                Arrays.asList("CourierNewPS-BoldItalicMT", "CourierNew-BoldItalic",
+                        "LiberationMono-BoldItalic", "NimbusMonL-BoldObli")));
+        substitutes.put("Helvetica", new ArrayList<>(
+                Arrays.asList("ArialMT", "Arial", "LiberationSans", "NimbusSanL-Regu")));
+        substitutes.put("Helvetica-Bold", new ArrayList<>(
+                Arrays.asList("Arial-BoldMT", "Arial-Bold", "LiberationSans-Bold",
+                        "NimbusSanL-Bold")));
+        substitutes.put("Helvetica-Oblique", new ArrayList<>(
+                Arrays.asList("Arial-ItalicMT", "Arial-Italic", "Helvetica-Italic",
+                        "LiberationSans-Italic", "NimbusSanL-ReguItal")));
+        substitutes.put("Helvetica-BoldOblique", new ArrayList<>(
+                Arrays.asList("Arial-BoldItalicMT", "Helvetica-BoldItalic",
+                        "LiberationSans-BoldItalic", "NimbusSanL-BoldItal")));
+        substitutes.put("Times-Roman", new ArrayList<>(
+                Arrays.asList("TimesNewRomanPSMT", "TimesNewRoman", "TimesNewRomanPS",
+                        "LiberationSerif", "NimbusRomNo9L-Regu")));
+        substitutes.put("Times-Bold", new ArrayList<>(
                 Arrays.asList("TimesNewRomanPS-BoldMT", "TimesNewRomanPS-Bold",
-                        "TimesNewRoman-Bold", "LiberationSerif-Bold", "NimbusRomNo9L-Medi"));
-        substitutes.put("Times-Italic",
+                        "TimesNewRoman-Bold", "LiberationSerif-Bold", "NimbusRomNo9L-Medi")));
+        substitutes.put("Times-Italic", new ArrayList<>(
                 Arrays.asList("TimesNewRomanPS-ItalicMT", "TimesNewRomanPS-Italic",
                         "TimesNewRoman-Italic", "LiberationSerif-Italic",
-                        "NimbusRomNo9L-ReguItal"));
-        substitutes.put("Times-BoldItalic",
+                        "NimbusRomNo9L-ReguItal")));
+        substitutes.put("Times-BoldItalic", new ArrayList<>(
                 Arrays.asList("TimesNewRomanPS-BoldItalicMT", "TimesNewRomanPS-BoldItalic",
                         "TimesNewRoman-BoldItalic", "LiberationSerif-BoldItalic",
-                        "NimbusRomNo9L-MediItal"));
-        substitutes.put("Symbol", Arrays.asList("Symbol", "SymbolMT", "StandardSymL"));
-        substitutes.put("ZapfDingbats", Arrays.asList("ZapfDingbatsITC", "Dingbats", "MS-Gothic"));
+                        "NimbusRomNo9L-MediItal")));
+        substitutes.put("Symbol",
+                new ArrayList<>(Arrays.asList("Symbol", "SymbolMT", "StandardSymL")));
+        substitutes.put("ZapfDingbats", new ArrayList<>(
+                Arrays.asList("ZapfDingbatsITCbyBT-Regular", "ZapfDingbatsITC", "Dingbats",
+                        "MS-Gothic")));
 
         // Acrobat also uses alternative names for Standard 14 fonts, which we map to those above
         // these include names such as "Arial" and "TimesNewRoman"
@@ -108,7 +122,7 @@ final class FontMapperImpl implements FontMapper
                     "/org/sejda/sambox/resources/ttf/LiberationSans-Regular.ttf");
             requireNotNullArg(stream,
                     "Unable to load org/sejda/sambox/resources/ttf/LiberationSans-Regular.ttf");
-            lastResortFont = new TTFParser().parse(stream);
+            lastResortFont = new TTFParser().parse(new BufferedInputStream(stream));
         }
         catch (IOException e)
         {
@@ -116,7 +130,8 @@ final class FontMapperImpl implements FontMapper
         }
     }
 
-    public TrueTypeFont getLastResortFont() {
+    public TrueTypeFont getLastResortFont()
+    {
         return lastResortFont;
     }
 
@@ -124,11 +139,15 @@ final class FontMapperImpl implements FontMapper
     private static class DefaultFontProvider
     {
         private static final FontProvider INSTANCE = initializeFontProvider();
-        
-        private static FontProvider initializeFontProvider() {
-            if("noop".equalsIgnoreCase(System.getProperty(SAMBox.FONT_PROVIDER_PROPERTY))) {
+
+        private static FontProvider initializeFontProvider()
+        {
+            if ("noop".equalsIgnoreCase(System.getProperty(SAMBox.FONT_PROVIDER_PROPERTY)))
+            {
                 return new NoopFontProvider();
-            } else {
+            }
+            else
+            {
                 return new FileSystemFontProvider(fontCache);
             }
         }
@@ -156,7 +175,8 @@ final class FontMapperImpl implements FontMapper
     }
 
     /**
-     * Returns the font cache associated with this FontMapper. This method is needed by FontProvider subclasses.
+     * Returns the font cache associated with this FontMapper. This method is needed by FontProvider
+     * subclasses.
      */
     public FontCache getFontCache()
     {
@@ -181,7 +201,7 @@ final class FontMapperImpl implements FontMapper
      */
     private Set<String> getPostScriptNames(String postScriptName)
     {
-        Set<String> names = new HashSet<>();
+        Set<String> names = new HashSet<>(2);
 
         // built-in PostScript name
         names.add(postScriptName);
@@ -203,7 +223,7 @@ final class FontMapperImpl implements FontMapper
     /**
      * Adds a top-priority substitute for the given font.
      *
-     * @param match PostScript name of the font to match
+     * @param match   PostScript name of the font to match
      * @param replace PostScript name of the font to use as a replacement
      */
     public void addSubstitute(String match, String replace)
@@ -242,8 +262,8 @@ final class FontMapperImpl implements FontMapper
             if (name != null)
             {
                 String lower = fontDescriptor.getFontName().toLowerCase();
-                isBold = lower.contains("bold") || lower.contains("black")
-                        || lower.contains("heavy");
+                isBold = lower.contains("bold") || lower.contains("black") || lower.contains(
+                        "heavy");
             }
 
             // font descriptor flags should describe the style
@@ -334,8 +354,8 @@ final class FontMapperImpl implements FontMapper
     }
 
     /**
-     * Finds a font with the given PostScript name, or a suitable substitute, or null. This allows any font to be
-     * substituted with a PFB, TTF or OTF.
+     * Finds a font with the given PostScript name, or a suitable substitute, or null. This allows
+     * any font to be substituted with a PFB, TTF or OTF.
      *
      * @param fontDescriptor the FontDescriptor of the font to find
      */
@@ -468,11 +488,11 @@ final class FontMapperImpl implements FontMapper
     }
 
     /**
-     * Finds a CFF CID-Keyed font with the given PostScript name, or a suitable substitute, or null. This method can
-     * also map CJK fonts via their CIDSystemInfo (ROS).
-     * 
+     * Finds a CFF CID-Keyed font with the given PostScript name, or a suitable substitute, or null.
+     * This method can also map CJK fonts via their CIDSystemInfo (ROS).
+     *
      * @param fontDescriptor FontDescriptor
-     * @param cidSystemInfo the CID system info, e.g. "Adobe-Japan1", if any.
+     * @param cidSystemInfo  the CID system info, e.g. "Adobe-Japan1", if any.
      */
     @Override
     public CIDFontMapping getCIDFont(String baseFont, PDFontDescriptor fontDescriptor,
@@ -526,11 +546,11 @@ final class FontMapperImpl implements FontMapper
     }
 
     /**
-     * Returns a list of matching fonts, scored by suitability. Positive scores indicate matches for certain attributes,
-     * while negative scores indicate mismatches. Zero scores are neutral.
-     * 
+     * Returns a list of matching fonts, scored by suitability. Positive scores indicate matches for
+     * certain attributes, while negative scores indicate mismatches. Zero scores are neutral.
+     *
      * @param fontDescriptor FontDescriptor, always present.
-     * @param cidSystemInfo Font's CIDSystemInfo, may be null.
+     * @param cidSystemInfo  Font's CIDSystemInfo, may be null.
      */
     private PriorityQueue<FontMatch> getFontMatches(PDFontDescriptor fontDescriptor,
             PDCIDSystemInfo cidSystemInfo)
@@ -553,8 +573,8 @@ final class FontMapperImpl implements FontMapper
                 PDPanoseClassification panose = fontDescriptor.getPanose().getPanose();
                 if (panose.getFamilyKind() == info.getPanose().getFamilyKind())
                 {
-                    if (panose.getFamilyKind() == 0
-                            && (info.getPostScriptName().toLowerCase().contains("barcode")
+                    if (panose.getFamilyKind() == 0 && (
+                            info.getPostScriptName().toLowerCase().contains("barcode")
                                     || info.getPostScriptName().startsWith("Code"))
                             && !probablyBarcodeFont(fontDescriptor))
                     {
@@ -629,13 +649,13 @@ final class FontMapperImpl implements FontMapper
     {
         String ff = ofNullable(fontDescriptor.getFontFamily()).orElse("");
         String fn = ofNullable(fontDescriptor.getFontName()).orElse("");
-        return ff.startsWith("Code") || ff.toLowerCase().contains("barcode")
-                || fn.startsWith("Code") || fn.toLowerCase().contains("barcode");
+        return ff.startsWith("Code") || ff.toLowerCase().contains("barcode") || fn.startsWith(
+                "Code") || fn.toLowerCase().contains("barcode");
     }
 
     /**
-     * Returns true if the character set described by CIDSystemInfo is present in the given font. Only applies to
-     * Adobe-GB1, Adobe-CNS1, Adobe-Japan1, Adobe-Korea1, as per the PDF spec.
+     * Returns true if the character set described by CIDSystemInfo is present in the given font.
+     * Only applies to Adobe-GB1, Adobe-CNS1, Adobe-Japan1, Adobe-Korea1, as per the PDF spec.
      */
     private boolean isCharSetMatch(PDCIDSystemInfo cidSystemInfo, FontInfo info)
     {
@@ -652,6 +672,11 @@ final class FontMapperImpl implements FontMapper
         long CHINESE_TRADITIONAL = 1 << 20;
         long KOREAN_JOHAB = 1 << 21;
 
+        if ("MalgunGothic-Semilight".equals(info.getPostScriptName()))
+        {
+            // PDFBOX-4793 and PDF.js 10699: This font has only Korean, but has bits 17-21 set.
+            codePageRange &= ~(JIS_JAPAN | CHINESE_SIMPLIFIED | CHINESE_TRADITIONAL);
+        }
         if (cidSystemInfo.getOrdering().equals("GB1")
                 && (codePageRange & CHINESE_SIMPLIFIED) == CHINESE_SIMPLIFIED)
         {
@@ -669,9 +694,9 @@ final class FontMapperImpl implements FontMapper
         }
         else
         {
-            return cidSystemInfo.getOrdering().equals("Korea1")
-                    && (codePageRange & KOREAN_WANSUNG) == KOREAN_WANSUNG
-                    || (codePageRange & KOREAN_JOHAB) == KOREAN_JOHAB;
+            return cidSystemInfo.getOrdering().equals("Korea1") && (
+                    (codePageRange & KOREAN_WANSUNG) == KOREAN_WANSUNG
+                            || (codePageRange & KOREAN_JOHAB) == KOREAN_JOHAB);
         }
     }
 
