@@ -421,9 +421,9 @@ final class PDCIDFontType2Embedder extends TrueTypeEmbedder
 
     private COSArray getWidths(int[] widths) throws IOException
     {
-        if (widths.length == 0)
+        if (widths.length < 2)
         {
-            throw new IllegalArgumentException("length of widths must be > 0");
+            throw new IllegalArgumentException("length of widths must be >= 2");
         }
 
         float scaling = 1000f / ttf.getHeader().getUnitsPerEm();
@@ -437,7 +437,7 @@ final class PDCIDFontType2Embedder extends TrueTypeEmbedder
 
         State state = State.FIRST;
 
-        for (int i = 2; i < widths.length; i += 2)
+        for (int i = 2; i < widths.length - 1; i += 2)
         {
             long cid = widths[i];
             long value = Math.round(widths[i + 1] * scaling);
@@ -549,9 +549,9 @@ final class PDCIDFontType2Embedder extends TrueTypeEmbedder
 
     private COSArray getVerticalMetrics(int[] values) throws IOException
     {
-        if (values.length == 0)
+        if (values.length < 4)
         {
-            throw new IllegalArgumentException("length of values must be > 0");
+            throw new IllegalArgumentException("length of values must be >= 4");
         }
 
         float scaling = 1000f / ttf.getHeader().getUnitsPerEm();
@@ -567,7 +567,7 @@ final class PDCIDFontType2Embedder extends TrueTypeEmbedder
 
         State state = State.FIRST;
 
-        for (int i = 4; i < values.length; i += 4)
+        for (int i = 4; i < values.length - 3; i += 4)
         {
             long cid = values[i];
             if (cid == Integer.MIN_VALUE)
