@@ -16,6 +16,7 @@
  */
 package org.sejda.sambox.pdmodel.font;
 
+import static java.util.Objects.isNull;
 import static java.util.Objects.nonNull;
 
 import java.awt.geom.GeneralPath;
@@ -23,9 +24,11 @@ import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Set;
 
 import org.apache.fontbox.cmap.CMap;
+import org.apache.fontbox.ttf.CmapLookup;
 import org.apache.fontbox.ttf.TTFParser;
 import org.apache.fontbox.ttf.TrueTypeFont;
 import org.apache.fontbox.util.BoundingBox;
@@ -57,10 +60,10 @@ public class PDType0Font extends PDFont implements PDVectorFont
     private TrueTypeFont ttf;
 
     /**
-     * Loads a TTF to be embedded and subset into a document as a Type 0 font. If you are loading a font for AcroForm,
-     * then use the 3-parameter constructor instead.
+     * Loads a TTF to be embedded and subset into a document as a Type 0 font. If you are loading a
+     * font for AcroForm, then use the 3-parameter constructor instead.
      *
-     * @param doc The PDF document that will hold the embedded font.
+     * @param doc  The PDF document that will hold the embedded font.
      * @param file A TrueType font.
      * @return A Type0 font with a CIDFontType2 descendant.
      * @throws IOException If there is an error reading the font file.
@@ -73,10 +76,10 @@ public class PDType0Font extends PDFont implements PDVectorFont
     }
 
     /**
-     * Loads a TTF to be embedded and subset into a document as a Type 0 font. If you are loading a font for AcroForm,
-     * then use the 3-parameter constructor instead.
+     * Loads a TTF to be embedded and subset into a document as a Type 0 font. If you are loading a
+     * font for AcroForm, then use the 3-parameter constructor instead.
      *
-     * @param doc The PDF document that will hold the embedded font.
+     * @param doc   The PDF document that will hold the embedded font.
      * @param input An input stream of a TrueType font. It will be closed before returning.
      * @return A Type0 font with a CIDFontType2 descendant.
      * @throws IOException If there is an error reading the font stream.
@@ -91,8 +94,8 @@ public class PDType0Font extends PDFont implements PDVectorFont
     /**
      * Loads a TTF to be embedded into a document as a Type 0 font.
      *
-     * @param doc The PDF document that will hold the embedded font.
-     * @param input A TrueType font.
+     * @param doc         The PDF document that will hold the embedded font.
+     * @param input       A TrueType font.
      * @param embedSubset True if the font will be subset before embedding
      * @return A Type0 font with a CIDFontType2 descendant.
      * @throws IOException If there is an error reading the font stream.
@@ -106,8 +109,8 @@ public class PDType0Font extends PDFont implements PDVectorFont
     /**
      * Loads a TTF to be embedded into a document as a Type 0 font.
      *
-     * @param doc The PDF document that will hold the embedded font.
-     * @param ttf A TrueType font.
+     * @param doc         The PDF document that will hold the embedded font.
+     * @param ttf         A TrueType font.
      * @param embedSubset True if the font will be subset before embedding
      * @return A Type0 font with a CIDFontType2 descendant.
      * @throws IOException If there is an error reading the font stream.
@@ -121,7 +124,7 @@ public class PDType0Font extends PDFont implements PDVectorFont
     /**
      * Loads a TTF to be embedded into a document as a vertical Type 0 font.
      *
-     * @param doc The PDF document that will hold the embedded font.
+     * @param doc  The PDF document that will hold the embedded font.
      * @param file A TrueType font.
      * @return A Type0 font with a CIDFontType2 descendant.
      * @throws IOException If there is an error reading the font file.
@@ -134,7 +137,7 @@ public class PDType0Font extends PDFont implements PDVectorFont
     /**
      * Loads a TTF to be embedded into a document as a vertical Type 0 font.
      *
-     * @param doc The PDF document that will hold the embedded font.
+     * @param doc   The PDF document that will hold the embedded font.
      * @param input A TrueType font.
      * @return A Type0 font with a CIDFontType2 descendant.
      * @throws IOException If there is an error reading the font stream.
@@ -147,8 +150,8 @@ public class PDType0Font extends PDFont implements PDVectorFont
     /**
      * Loads a TTF to be embedded into a document as a vertical Type 0 font.
      *
-     * @param doc The PDF document that will hold the embedded font.
-     * @param input A TrueType font.
+     * @param doc         The PDF document that will hold the embedded font.
+     * @param input       A TrueType font.
      * @param embedSubset True if the font will be subset before embedding
      * @return A Type0 font with a CIDFontType2 descendant.
      * @throws IOException If there is an error reading the font stream.
@@ -162,8 +165,8 @@ public class PDType0Font extends PDFont implements PDVectorFont
     /**
      * Loads a TTF to be embedded into a document as a vertical Type 0 font.
      *
-     * @param doc The PDF document that will hold the embedded font.
-     * @param ttf A TrueType font.
+     * @param doc         The PDF document that will hold the embedded font.
+     * @param ttf         A TrueType font.
      * @param embedSubset True if the font will be subset before embedding
      * @return A Type0 font with a CIDFontType2 descendant.
      * @throws IOException If there is an error reading the font stream.
@@ -176,40 +179,44 @@ public class PDType0Font extends PDFont implements PDVectorFont
 
     /**
      * Constructor for reading a Type0 font from a PDF file.
-     * 
+     *
      * @param fontDictionary The font dictionary according to the PDF specification.
      */
     public PDType0Font(COSDictionary fontDictionary) throws IOException
     {
         super(fontDictionary);
-        COSBase base = dict.getDictionaryObject(COSName.DESCENDANT_FONTS);
-        if (!(base instanceof COSArray))
+        COSArray descendantFonts = dict.getDictionaryObject(COSName.DESCENDANT_FONTS,
+                COSArray.class);
+        if (isNull(descendantFonts))
         {
             throw new IOException("Missing descendant font array");
         }
-        COSArray descendantFonts = (COSArray) base;
         if (descendantFonts.size() == 0)
         {
             throw new IOException("Descendant font array is empty");
         }
-        COSBase descendantFontDictBase = descendantFonts.getObject(0);
-        if (!(descendantFontDictBase instanceof COSDictionary))
+        COSDictionary descendantFontDict = descendantFonts.getObject(0, COSDictionary.class);
+        if (isNull(descendantFontDict))
         {
             throw new IOException("Missing descendant font dictionary");
         }
-        descendantFont = PDFontFactory.createDescendantFont((COSDictionary) descendantFontDictBase,
-                this);
+        if (!COSName.FONT.equals(
+                descendantFontDict.getDictionaryObject(COSName.TYPE, COSName.FONT, COSName.class)))
+        {
+            throw new IOException("Missing or wrong type in descendant font dictionary");
+        }
+        descendantFont = PDFontFactory.createDescendantFont(descendantFontDict, this);
         readEncoding();
         fetchCMapUCS2();
     }
 
     /**
-     *
      * @param document
      * @param ttf
      * @param embedSubset
-     * @param closeTTF whether to close the ttf parameter after embedding. Must be true when the ttf parameter was
-     * created in the load() method, false when the ttf parameter was passed to the load() method.
+     * @param closeTTF    whether to close the ttf parameter after embedding. Must be true when the
+     *                    ttf parameter was created in the load() method, false when the ttf
+     *                    parameter was passed to the load() method.
      * @param vertical
      * @throws IOException
      */
@@ -281,14 +288,7 @@ public class PDType0Font extends PDFont implements PDVectorFont
             // predefined CMap
             COSName encodingName = (COSName) encoding;
             cMap = CMapManager.getPredefinedCMap(encodingName.getName());
-            if (cMap != null)
-            {
-                isCMapPredefined = true;
-            }
-            else
-            {
-                throw new IOException("Missing required CMap");
-            }
+            isCMapPredefined = true;
         }
         else if (encoding != null)
         {
@@ -297,7 +297,7 @@ public class PDType0Font extends PDFont implements PDVectorFont
             {
                 throw new IOException("Missing required CMap");
             }
-            else if (!cMap.hasCIDMappings())
+            if (!cMap.hasCIDMappings())
             {
                 LOG.warn("Invalid Encoding CMap in font " + getName());
             }
@@ -347,9 +347,16 @@ public class PDType0Font extends PDFont implements PDVectorFont
             // try to find the corresponding Unicode (UC2) CMap
             if (strName != null)
             {
-                CMap prdCMap = CMapManager.getPredefinedCMap(strName);
-                String ucs2Name = prdCMap.getRegistry() + "-" + prdCMap.getOrdering() + "-UCS2";
-                cMapUCS2 = CMapManager.getPredefinedCMap(ucs2Name);
+                try
+                {
+                    CMap prdCMap = CMapManager.getPredefinedCMap(strName);
+                    String ucs2Name = prdCMap.getRegistry() + "-" + prdCMap.getOrdering() + "-UCS2";
+                    cMapUCS2 = CMapManager.getPredefinedCMap(ucs2Name);
+                }
+                catch (IOException ex)
+                {
+                    LOG.warn("Could not get {} UC2 map for font {}", strName, getName(), ex);
+                }
             }
         }
     }
@@ -482,7 +489,7 @@ public class PDType0Font extends PDFont implements PDVectorFont
         if ((isCMapPredefined || isDescendantCJK) && cMapUCS2 != null)
         {
             // if the font is composite and uses a predefined cmap (excluding Identity-H/V) then
-            // or if its decendant font uses Adobe-GB1/CNS1/Japan1/Korea1
+            // or if its descendant font uses Adobe-GB1/CNS1/Japan1/Korea1
 
             // a) Map the character code to a character identifier (CID) according to the font?s CMap
             int cid = codeToCID(code);
@@ -490,6 +497,44 @@ public class PDType0Font extends PDFont implements PDVectorFont
             // e) Map the CID according to the CMap from step d), producing a Unicode value
             return cMapUCS2.toUnicode(cid);
         }
+
+        // PDFBOX-5324: try to get unicode from font cmap
+        if (descendantFont instanceof PDCIDFontType2)
+        {
+            TrueTypeFont font = ((PDCIDFontType2) descendantFont).getTrueTypeFont();
+            if (font != null)
+            {
+                try
+                {
+                    CmapLookup cmap = font.getUnicodeCmapLookup(false);
+                    if (cmap != null)
+                    {
+                        int gid;
+                        if (descendantFont.isEmbedded())
+                        {
+                            // original PDFBOX-5324 supported only embedded fonts
+                            gid = descendantFont.codeToGID(code);
+                        }
+                        else
+                        {
+                            // PDFBOX-5331: this bypasses the fallback attempt in
+                            // PDCIDFontType2.codeToGID() which would bring a stackoverflow
+                            gid = descendantFont.codeToCID(code);
+                        }
+                        List<Integer> codes = cmap.getCharCodes(gid);
+                        if (codes != null && !codes.isEmpty())
+                        {
+                            return Character.toString((char) (int) codes.get(0));
+                        }
+                    }
+                }
+                catch (IOException | NullPointerException e)
+                {
+                    LOG.warn("get unicode from font cmap fail", e);
+                }
+            }
+        }
+
         if (LOG.isWarnEnabled() && !noUnicode.contains(code))
         {
             // if no value has been produced, there is no way to obtain Unicode for the character.
@@ -555,12 +600,14 @@ public class PDType0Font extends PDFont implements PDVectorFont
     }
 
     @Override
-    public boolean isOriginalEmbeddedMissing() {
+    public boolean isOriginalEmbeddedMissing()
+    {
         return descendantFont.isOriginalEmbeddedMissing();
     }
 
     @Override
-    public boolean isMappingFallbackUsed() {
+    public boolean isMappingFallbackUsed()
+    {
         return descendantFont.isMappingFallbackUsed();
     }
 
@@ -588,5 +635,4 @@ public class PDType0Font extends PDFont implements PDVectorFont
         return descendantFont.hasGlyph(code);
     }
 
-    
 }
