@@ -31,6 +31,7 @@ import java.util.Map;
 
 import org.apache.fontbox.EncodedFont;
 import org.apache.fontbox.FontBoxFont;
+import org.apache.fontbox.cff.CFFFont;
 import org.apache.fontbox.cff.CFFParser;
 import org.apache.fontbox.cff.CFFType1Font;
 import org.apache.fontbox.util.BoundingBox;
@@ -70,7 +71,7 @@ public class PDType1CFont extends PDSimpleFont
 
     /**
      * Constructor.
-     * 
+     *
      * @param fontDictionary the corresponding dictionary
      * @throws IOException it something went wrong
      */
@@ -102,7 +103,17 @@ public class PDType1CFont extends PDSimpleFont
             {
                 // note: this could be an OpenType file, fortunately CFFParser can handle that
                 CFFParser cffParser = new CFFParser();
-                cffEmbedded = (CFFType1Font) cffParser.parse(bytes, new FF3ByteSource()).get(0);
+                CFFFont parsedCffFont = cffParser.parse(bytes, new FF3ByteSource()).get(0);
+                if (parsedCffFont instanceof CFFType1Font)
+                {
+                    cffEmbedded = (CFFType1Font) parsedCffFont;
+                }
+                else
+                {
+                    LOG.error("Expected CFFType1Font, got {}",
+                            parsedCffFont.getClass().getSimpleName());
+                    fontIsDamaged = true;
+                }
             }
         }
         catch (IOException e)
@@ -121,8 +132,8 @@ public class PDType1CFont extends PDSimpleFont
         else
         {
             this.isOriginalEmbeddedMissing = true;
-            FontMapping<FontBoxFont> mapping = FontMappers.instance().getFontBoxFont(getBaseFont(),
-                    fd);
+            FontMapping<FontBoxFont> mapping = FontMappers.instance()
+                    .getFontBoxFont(getBaseFont(), fd);
             genericFont = mapping.getFont();
 
             if (mapping.isFallback())
@@ -168,6 +179,18 @@ public class PDType1CFont extends PDSimpleFont
         {
             return new GeneralPath();
         }
+        if ("sfthyphen".equals(name))
+        {
+            return genericFont.getPath("hyphen");
+        }
+        if ("nbspace".equals(name))
+        {
+            if (!hasGlyph("space"))
+            {
+                return new GeneralPath();
+            }
+            return genericFont.getPath("space");
+        }
         return genericFont.getPath(name);
     }
 
@@ -198,8 +221,8 @@ public class PDType1CFont extends PDSimpleFont
         if (getFontDescriptor() != null)
         {
             PDRectangle bbox = getFontDescriptor().getFontBoundingBox();
-            if (nonNull(bbox) && bbox.getLowerLeftX() != 0 || bbox.getLowerLeftY() != 0
-                    || bbox.getUpperRightX() != 0 || bbox.getUpperRightY() != 0)
+            if (nonNull(bbox) && (bbox.getLowerLeftX() != 0 || bbox.getLowerLeftY() != 0
+                    || bbox.getUpperRightX() != 0 || bbox.getUpperRightY() != 0))
             {
                 return new BoundingBox(bbox.getLowerLeftX(), bbox.getLowerLeftY(),
                         bbox.getUpperRightX(), bbox.getUpperRightY());
@@ -297,7 +320,12 @@ public class PDType1CFont extends PDSimpleFont
         float height;
         if (!glyphHeights.containsKey(name))
         {
-            height = (float) cffFont.getType1CharString(name).getBounds().getHeight(); // todo: cffFont could be null
+            if (cffFont == null)
+            {
+                LOG.warn("No embedded CFF font, returning height 0");
+                return 0;
+            }
+            height = (float) cffFont.getType1CharString(name).getBounds().getHeight();
             glyphHeights.put(name, height);
         }
         else
@@ -334,6 +362,11 @@ public class PDType1CFont extends PDSimpleFont
     @Override
     public float getStringWidth(String string) throws IOException
     {
+        if (cffFont == null)
+        {
+            LOG.warn("No embedded CFF font, returning width 0");
+            return 0;
+        }
         float width = 0;
         for (int i = 0; i < string.length(); i++)
         {
@@ -356,7 +389,7 @@ public class PDType1CFont extends PDSimpleFont
 
     /**
      * Returns the embedded Type 1-equivalent CFF font.
-     * 
+     *
      * @return the cffFont
      */
     public CFFType1Font getCFFType1Font()
@@ -372,8 +405,8 @@ public class PDType1CFont extends PDSimpleFont
     }
 
     /**
-     * Maps a PostScript glyph name to the name in the underlying font, for example when using a TTF font we might map
-     * "W" to "uni0057".
+     * Maps a PostScript glyph name to the name in the underlying font, for example when using a TTF
+     * font we might map "W" to "uni0057".
      */
     private String getNameInFont(String name) throws IOException
     {
@@ -395,12 +428,14 @@ public class PDType1CFont extends PDSimpleFont
     }
 
     @Override
-    public boolean isOriginalEmbeddedMissing() {
+    public boolean isOriginalEmbeddedMissing()
+    {
         return isOriginalEmbeddedMissing;
     }
 
     @Override
-    public boolean isMappingFallbackUsed() {
+    public boolean isMappingFallbackUsed()
+    {
         return isMappingFallbackUsed;
     }
 }
