@@ -16,14 +16,10 @@
  */
 package org.sejda.sambox.pdmodel.font;
 
-import java.io.IOException;
-import java.io.InputStream;
-import java.util.ArrayList;
-import java.util.List;
-
 import org.apache.fontbox.afm.FontMetrics;
 import org.apache.fontbox.pfb.PfbParser;
 import org.apache.fontbox.type1.Type1Font;
+import org.apache.fontbox.util.BoundingBox;
 import org.sejda.commons.util.IOUtils;
 import org.sejda.sambox.cos.COSArrayList;
 import org.sejda.sambox.cos.COSDictionary;
@@ -35,6 +31,11 @@ import org.sejda.sambox.pdmodel.font.encoding.Encoding;
 import org.sejda.sambox.pdmodel.font.encoding.GlyphList;
 import org.sejda.sambox.pdmodel.font.encoding.Type1Encoding;
 
+import java.io.IOException;
+import java.io.InputStream;
+import java.util.ArrayList;
+import java.util.List;
+
 /**
  * Embedded PDType1Font builder. Helper class to populate a PDType1Font from a PFB and AFM.
  *
@@ -48,8 +49,8 @@ class PDType1FontEmbedder
     /**
      * This will load a PFB to be embedded into a document.
      *
-     * @param doc The PDF document that will hold the embedded font.
-     * @param dict The Font dictionary to write to.
+     * @param doc       The PDF document that will hold the embedded font.
+     * @param dict      The Font dictionary to write to.
      * @param pfbStream The pfb input.
      * @throws IOException If there is an error loading the data.
      */
@@ -107,18 +108,18 @@ class PDType1FontEmbedder
      */
     static PDFontDescriptor buildFontDescriptor(Type1Font type1)
     {
-        boolean isSymbolic = type1
-                .getEncoding() instanceof org.apache.fontbox.encoding.BuiltInEncoding;
+        boolean isSymbolic = type1.getEncoding() instanceof org.apache.fontbox.encoding.BuiltInEncoding;
 
         PDFontDescriptor fd = new PDFontDescriptor();
         fd.setFontName(type1.getName());
         fd.setFontFamily(type1.getFamilyName());
         fd.setNonSymbolic(!isSymbolic);
         fd.setSymbolic(isSymbolic);
-        fd.setFontBoundingBox(new PDRectangle(type1.getFontBBox()));
+        BoundingBox fontBBox = type1.getFontBBox();
+        fd.setFontBoundingBox(new PDRectangle(fontBBox));
         fd.setItalicAngle(type1.getItalicAngle());
-        fd.setAscent(type1.getFontBBox().getUpperRightY());
-        fd.setDescent(type1.getFontBBox().getLowerLeftY());
+        fd.setAscent(fontBBox.getUpperRightY());
+        fd.setDescent(fontBBox.getLowerLeftY());
         fd.setCapHeight(type1.getBlueValues().get(2).floatValue());
         fd.setStemV(0); // for PDF/A
         return fd;
