@@ -17,6 +17,8 @@
 
 package org.sejda.sambox.pdmodel.font;
 
+import static java.util.Objects.nonNull;
+import static org.sejda.commons.util.RequireUtils.requireIOCondition;
 import static org.sejda.sambox.pdmodel.font.FontUtils.getTag;
 import static org.sejda.sambox.pdmodel.font.FontUtils.isEmbeddingPermitted;
 import static org.sejda.sambox.pdmodel.font.FontUtils.isSubsettingPermitted;
@@ -32,7 +34,6 @@ import java.util.Map;
 import java.util.Set;
 
 import org.apache.fontbox.ttf.CmapLookup;
-import org.apache.fontbox.ttf.CmapSubtable;
 import org.apache.fontbox.ttf.HeaderTable;
 import org.apache.fontbox.ttf.HorizontalHeaderTable;
 import org.apache.fontbox.ttf.OS2WindowsMetricsTable;
@@ -58,9 +59,6 @@ abstract class TrueTypeEmbedder implements Subsetter
 
     protected TrueTypeFont ttf;
     protected PDFontDescriptor fontDescriptor;
-    @Deprecated
-    protected final CmapSubtable cmap;
-
     protected final CmapLookup cmapLookup;
     private final Set<Integer> subsetCodePoints = new HashSet<>();
     private final boolean embedSubset;
@@ -82,15 +80,32 @@ abstract class TrueTypeEmbedder implements Subsetter
         if (!embedSubset)
         {
             // full embedding
-            PDStream stream = new PDStream(ttf.getOriginalData(), COSName.FLATE_DECODE);
+
+            // TrueType collections are not supported
+            InputStream is = ttf.getOriginalData();
+            byte[] b = new byte[4];
+            is.mark(b.length);
+            if (is.read(b) == b.length && new String(b).equals("ttcf"))
+            {
+                is.close();
+                throw new IOException("Full embedding of TrueType font collections not supported");
+            }
+            if (is.markSupported())
+            {
+                is.reset();
+            }
+            else
+            {
+                is.close();
+                is = ttf.getOriginalData();
+            }
+            PDStream stream = new PDStream(is, COSName.FLATE_DECODE);
             stream.getCOSObject().setLong(COSName.LENGTH1, ttf.getOriginalDataSize());
             fontDescriptor.setFontFile2(stream);
         }
 
         dict.setName(COSName.BASE_FONT, ttf.getName());
 
-        // choose a Unicode "cmap"
-        cmap = ttf.getUnicodeCmap();
         cmapLookup = ttf.getUnicodeCmapLookup();
     }
 
@@ -131,7 +146,9 @@ abstract class TrueTypeEmbedder implements Subsetter
         fd.setFontName(ttf.getName());
 
         OS2WindowsMetricsTable os2 = ttf.getOS2Windows();
+        requireIOCondition(nonNull(os2), "os2 table is missing in font " + ttf.getName());
         PostScriptTable post = ttf.getPostScript();
+        requireIOCondition(nonNull(post), "post table is missing in font " + ttf.getName());
 
         // Flags
         fd.setFixedPitch(
