@@ -16,9 +16,19 @@
  */
 package org.sejda.sambox.pdmodel.graphics.color;
 
-import static org.sejda.commons.util.RequireUtils.requireIOCondition;
+import org.sejda.commons.util.IOUtils;
+import org.sejda.sambox.cos.COSArray;
+import org.sejda.sambox.cos.COSArrayList;
+import org.sejda.sambox.cos.COSBase;
+import org.sejda.sambox.cos.COSFloat;
+import org.sejda.sambox.cos.COSName;
+import org.sejda.sambox.cos.COSStream;
+import org.sejda.sambox.pdmodel.PDResources;
+import org.sejda.sambox.pdmodel.common.PDRange;
+import org.sejda.sambox.pdmodel.common.PDStream;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-import java.awt.Color;
 import java.awt.Transparency;
 import java.awt.color.CMMException;
 import java.awt.color.ColorSpace;
@@ -36,22 +46,11 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.StringTokenizer;
 
-import org.sejda.commons.util.IOUtils;
-import org.sejda.sambox.cos.COSArray;
-import org.sejda.sambox.cos.COSArrayList;
-import org.sejda.sambox.cos.COSBase;
-import org.sejda.sambox.cos.COSFloat;
-import org.sejda.sambox.cos.COSName;
-import org.sejda.sambox.cos.COSStream;
-import org.sejda.sambox.pdmodel.PDResources;
-import org.sejda.sambox.pdmodel.common.PDRange;
-import org.sejda.sambox.pdmodel.common.PDStream;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
+import static org.sejda.commons.util.RequireUtils.requireIOCondition;
 
 /**
- * ICCBased colour spaces are based on a cross-platform colour profile as defined by the International Color Consortium
- * (ICC).
+ * ICCBased colour spaces are based on a cross-platform colour profile as defined by the
+ * International Color Consortium (ICC).
  *
  * @author Ben Litchfield
  * @author John Hewson
@@ -70,8 +69,8 @@ public final class PDICCBased extends PDCIEBasedColorSpace
     // allows to force using alternate color space instead of ICC color space for performance
     // reasons with LittleCMS (LCMS), see PDFBOX-4309
     // WARNING: do not activate this in a conforming reader
-    private boolean useOnlyAlternateColorSpace = Boolean
-            .getBoolean("org.sejda.sambox.rendering.UseAlternateInsteadOfICCColorSpace");
+    private boolean useOnlyAlternateColorSpace = Boolean.getBoolean(
+            "org.sejda.sambox.rendering.UseAlternateInsteadOfICCColorSpace");
 
     private static final boolean IS_KCMS;
 
@@ -115,10 +114,10 @@ public final class PDICCBased extends PDCIEBasedColorSpace
      * Creates a new ICC color space using the PDF array.
      *
      * @param iccArray the ICC stream object
-     * @throws IOException if there is an error reading the ICC profile or if the parameter is invalid
-     * @deprecated This will be private in 3.0. Please use
-     * {@link PDICCBased#create(org.apache.pdfbox.cos.COSArray, org.apache.pdfbox.pdmodel.PDResources)} instead, which
-     * supports caching.
+     * @throws IOException if there is an error reading the ICC profile or if the parameter is
+     *                     invalid
+     * @deprecated This will be private in 3.0. Please use {@link PDICCBased#create(org.sejda.sambox.cos.COSArray,
+     * org.sejda.sambox.pdmodel.PDResources)} instead, which supports caching.
      */
     @Deprecated
     public PDICCBased(COSArray iccArray) throws IOException
@@ -135,10 +134,11 @@ public final class PDICCBased extends PDCIEBasedColorSpace
     /**
      * Creates a new ICC color space using the PDF array, optionally using a resource cache.
      *
-     * @param iccArray the ICC stream object.
+     * @param iccArray  the ICC stream object.
      * @param resources resources to use as cache, or null for no caching.
      * @return an ICC color space.
-     * @throws IOException if there is an error reading the ICC profile or if the parameter is invalid.
+     * @throws IOException if there is an error reading the ICC profile or if the parameter is
+     *                     invalid.
      */
     public static PDICCBased create(COSArray iccArray, PDResources resources) throws IOException
     {
@@ -148,14 +148,14 @@ public final class PDICCBased extends PDCIEBasedColorSpace
                 "ICCBased colorspace array must have a stream as second element");
 
         COSBase base = iccArray.get(1);
-        boolean canCache = base.hasId() && resources != null
-                && resources.getResourceCache() != null;
+        boolean canCache =
+                base.hasId() && resources != null && resources.getResourceCache() != null;
 
         if (canCache)
         {
             PDColorSpace space = resources.getResourceCache()
                     .getColorSpace(base.id().objectIdentifier);
-            if (space != null && space instanceof PDICCBased)
+            if (space instanceof PDICCBased)
             {
                 return (PDICCBased) space;
             }
@@ -176,7 +176,7 @@ public final class PDICCBased extends PDCIEBasedColorSpace
 
     /**
      * Get the underlying ICC profile stream.
-     * 
+     *
      * @return the underlying ICC profile stream
      */
     public PDStream getPDStream()
@@ -227,7 +227,7 @@ public final class PDICCBased extends PDCIEBasedColorSpace
 
                 // set initial colour
                 float[] initial = new float[getNumberOfComponents()];
-                for (int c = 0; c < getNumberOfComponents(); c++)
+                for (int c = 0; c < initial.length; c++)
                 {
                     initial[c] = Math.max(0, getRangeForComponent(c).getMin());
                 }
@@ -239,7 +239,7 @@ public final class PDICCBased extends PDCIEBasedColorSpace
                     // or CMMException due to invalid profiles, see PDFBOX-1295 and PDFBOX-1740 (ü-file)
                     // or ArrayIndexOutOfBoundsException, see PDFBOX-3610
                     // also triggers a ProfileDataException for PDFBOX-3549 with KCMS
-                    new Color(awtColorSpace, new float[getNumberOfComponents()], 1f);
+                    awtColorSpace.toRGB(new float[getNumberOfComponents()]);
                 }
                 else
                 {
@@ -249,23 +249,7 @@ public final class PDICCBased extends PDCIEBasedColorSpace
                 }
             }
         }
-        catch (ProfileDataException e)
-        {
-            fallbackToAlternateColorSpace(e);
-        }
-        catch (CMMException e)
-        {
-            fallbackToAlternateColorSpace(e);
-        }
-        catch (IllegalArgumentException e)
-        {
-            fallbackToAlternateColorSpace(e);
-        }
-        catch (ArrayIndexOutOfBoundsException e)
-        {
-            fallbackToAlternateColorSpace(e);
-        }
-        catch (IOException e)
+        catch (ProfileDataException | IOException | ArrayIndexOutOfBoundsException | IllegalArgumentException | CMMException e)
         {
             fallbackToAlternateColorSpace(e);
         }
@@ -352,8 +336,8 @@ public final class PDICCBased extends PDCIEBasedColorSpace
         {
             float minValue = cs.getMinValue(i);
             float maxValue = cs.getMaxValue(i);
-            result[i] = value[i] < minValue ? minValue
-                    : (value[i] > maxValue ? maxValue : value[i]);
+            result[i] =
+                    value[i] < minValue ? minValue : (value[i] > maxValue ? maxValue : value[i]);
         }
         return result;
     }
@@ -377,12 +361,34 @@ public final class PDICCBased extends PDCIEBasedColorSpace
         return alternateColorSpace.toRGBImage(raster);
     }
 
+    @Override
+    public BufferedImage toRawImage(WritableRaster raster) throws IOException
+    {
+        if (awtColorSpace == null)
+        {
+            return alternateColorSpace.toRawImage(raster);
+        }
+        return toRawImage(raster, awtColorSpace);
+    }
+
     @Override
     public int getNumberOfComponents()
     {
         if (numberOfComponents < 0)
         {
             numberOfComponents = stream.getCOSObject().getInt(COSName.N);
+            // PDFBOX-4801 correct wrong /N values
+            if (iccProfile != null)
+            {
+                int numIccComponents = iccProfile.getNumComponents();
+                if (numIccComponents != numberOfComponents)
+                {
+                    LOG.warn(
+                            "Using {} components from ICC profile info instead of {} components from /N entry",
+                            numIccComponents, numberOfComponents);
+                    numberOfComponents = numIccComponents;
+                }
+            }
         }
         return numberOfComponents;
     }
@@ -414,9 +420,9 @@ public final class PDICCBased extends PDCIEBasedColorSpace
     }
 
     /**
-     * Returns a list of alternate color spaces for non-conforming readers. WARNING: Do not use the information in a
-     * conforming reader.
-     * 
+     * Returns a list of alternate color spaces for non-conforming readers. WARNING: Do not use the
+     * information in a conforming reader.
+     *
      * @return A list of alternateColorSpace color spaces.
      * @throws IOException If there is an error getting the alternateColorSpace color spaces.
      */
@@ -458,17 +464,18 @@ public final class PDICCBased extends PDCIEBasedColorSpace
             }
             else
             {
-                throw new IOException("Error: expected COSArray or COSName and not "
-                        + alternate.getClass().getName());
+                throw new IOException(
+                        "Error: expected COSArray or COSName and not " + alternate.getClass()
+                                .getName());
             }
         }
         return PDColorSpace.create(alternateArray);
     }
 
     /**
-     * Returns the range for a certain component number. This will never return null. If it is not present then the
-     * range 0..1 will be returned.
-     * 
+     * Returns the range for a certain component number. This will never return null. If it is not
+     * present then the range 0..1 will be returned.
+     *
      * @param n the component number to get the range for
      * @return the range for this component
      */
@@ -484,18 +491,18 @@ public final class PDICCBased extends PDCIEBasedColorSpace
 
     /**
      * Returns the metadata stream for this object, or null if there is no metadata stream.
-     * 
+     *
      * @return the metadata stream, or null if there is none
      */
     public COSStream getMetadata()
     {
-        return (COSStream) stream.getCOSObject().getDictionaryObject(COSName.METADATA);
+        return stream.getCOSObject().getDictionaryObject(COSName.METADATA, COSStream.class);
     }
 
     /**
-     * Returns the type of the color space in the ICC profile. Will be one of {@code TYPE_GRAY}, {@code TYPE_RGB}, or
-     * {@code TYPE_CMYK}.
-     * 
+     * Returns the type of the color space in the ICC profile. Will be one of {@code TYPE_GRAY},
+     * {@code TYPE_RGB}, or {@code TYPE_CMYK}.
+     *
      * @return an ICC color space type
      */
     public int getColorSpaceType()
@@ -522,7 +529,7 @@ public final class PDICCBased extends PDCIEBasedColorSpace
 
     /**
      * Sets the number of color components.
-     * 
+     *
      * @param n the number of color components
      */
     // TODO it's probably not safe to use this
@@ -550,9 +557,9 @@ public final class PDICCBased extends PDCIEBasedColorSpace
 
     /**
      * Sets the range for this color space.
-     * 
+     *
      * @param range the new range for the a component
-     * @param n the component to set the range for
+     * @param n     the component to set the range for
      */
     public void setRangeForComponent(PDRange range, int n)
     {
@@ -574,7 +581,7 @@ public final class PDICCBased extends PDCIEBasedColorSpace
 
     /**
      * Sets the metadata stream that is associated with this color space.
-     * 
+     *
      * @param metadata the new metadata stream
      */
     public void setMetadata(COSStream metadata)
@@ -582,6 +589,16 @@ public final class PDICCBased extends PDCIEBasedColorSpace
         stream.getCOSObject().setItem(COSName.METADATA, metadata);
     }
 
+    /**
+     * Internal accessor to support indexed raw images.
+     *
+     * @return true if this colorspace is sRGB.
+     */
+    boolean isSRGB()
+    {
+        return isRGB;
+    }
+
     @Override
     public String toString()
     {
