@@ -16,13 +16,6 @@
  */
 package org.sejda.sambox.pdmodel.graphics.color;
 
-import java.awt.Point;
-import java.awt.image.BufferedImage;
-import java.awt.image.DataBuffer;
-import java.awt.image.Raster;
-import java.awt.image.WritableRaster;
-import java.io.IOException;
-
 import org.sejda.sambox.cos.COSArray;
 import org.sejda.sambox.cos.COSBase;
 import org.sejda.sambox.cos.COSInteger;
@@ -34,10 +27,19 @@ import org.sejda.sambox.cos.COSString;
 import org.sejda.sambox.pdmodel.PDResources;
 import org.sejda.sambox.pdmodel.common.PDStream;
 
+import java.awt.Point;
+import java.awt.image.BufferedImage;
+import java.awt.image.ColorModel;
+import java.awt.image.DataBuffer;
+import java.awt.image.IndexColorModel;
+import java.awt.image.Raster;
+import java.awt.image.WritableRaster;
+import java.io.IOException;
+
 /**
- * An Indexed colour space specifies that an area is to be painted using a colour table of arbitrary colours from
- * another color space.
- * 
+ * An Indexed colour space specifies that an area is to be painted using a colour table of arbitrary
+ * colours from another color space.
+ *
  * @author John Hewson
  * @author Ben Litchfield
  */
@@ -67,7 +69,7 @@ public final class PDIndexed extends PDSpecialColorSpace
 
     /**
      * Creates a new indexed color space from the given PDF array.
-     * 
+     *
      * @param indexedArray the array containing the indexed parameters
      * @throws java.io.IOException
      */
@@ -78,9 +80,9 @@ public final class PDIndexed extends PDSpecialColorSpace
 
     /**
      * Creates a new indexed color space from the given PDF array.
-     * 
+     *
      * @param indexedArray the array containing the indexed parameters
-     * @param resources the resources, can be null. Allows to use its cache for the colorspace.
+     * @param resources    the resources, can be null. Allows to use its cache for the colorspace.
      * @throws java.io.IOException
      */
     public PDIndexed(COSArray indexedArray, PDResources resources) throws IOException
@@ -212,9 +214,32 @@ public final class PDIndexed extends PDSpecialColorSpace
         return rgbImage;
     }
 
+    @Override
+    public BufferedImage toRawImage(WritableRaster raster)
+    {
+        // We can only convert sRGB index colorspaces, depending on the base colorspace
+        if (baseColorSpace instanceof PDICCBased && ((PDICCBased) baseColorSpace).isSRGB())
+        {
+            byte[] r = new byte[colorTable.length];
+            byte[] g = new byte[colorTable.length];
+            byte[] b = new byte[colorTable.length];
+            for (int i = 0; i < colorTable.length; i++)
+            {
+                r[i] = (byte) ((int) (colorTable[i][0] * 255) & 0xFF);
+                g[i] = (byte) ((int) (colorTable[i][1] * 255) & 0xFF);
+                b[i] = (byte) ((int) (colorTable[i][2] * 255) & 0xFF);
+            }
+            ColorModel colorModel = new IndexColorModel(8, colorTable.length, r, g, b);
+            return new BufferedImage(colorModel, raster, false, null);
+        }
+
+        // We can't handle all other cases at the moment.
+        return null;
+    }
+
     /**
      * Returns the base color space.
-     * 
+     *
      * @return the base color space.
      */
     public PDColorSpace getBaseColorSpace()
@@ -229,7 +254,7 @@ public final class PDIndexed extends PDSpecialColorSpace
     }
 
     // reads the lookup table data from the array
-    private byte[] getLookupData() throws IOException
+    private void readLookupData() throws IOException
     {
         if (lookupData == null)
         {
@@ -251,7 +276,6 @@ public final class PDIndexed extends PDSpecialColorSpace
                 throw new IOException("Error: Unknown type for lookup table " + lookupTable);
             }
         }
-        return lookupData;
     }
 
     //
@@ -259,7 +283,8 @@ public final class PDIndexed extends PDSpecialColorSpace
     //
     private void readColorTable() throws IOException
     {
-        byte[] lookupData = getLookupData();
+        readLookupData();
+        
         int maxIndex = Math.min(getHival(), 255);
         int numComponents = baseColorSpace.getNumberOfComponents();
 
@@ -283,7 +308,7 @@ public final class PDIndexed extends PDSpecialColorSpace
 
     /**
      * Sets the base color space.
-     * 
+     *
      * @param base the base color space
      */
     public void setBaseColorSpace(PDColorSpace base)
@@ -294,7 +319,7 @@ public final class PDIndexed extends PDSpecialColorSpace
 
     /**
      * Sets the highest value that is allowed. This cannot be higher than 255.
-     * 
+     *
      * @param high the highest value for the lookup table
      */
     public void setHighValue(int high)
