@@ -17,6 +17,7 @@
 package org.sejda.sambox.pdmodel.graphics.color;
 
 import java.awt.Point;
+import java.awt.color.ColorSpace;
 import java.awt.image.BufferedImage;
 import java.awt.image.DataBuffer;
 import java.awt.image.Raster;
@@ -32,10 +33,10 @@ import org.sejda.sambox.cos.COSNull;
 import org.sejda.sambox.pdmodel.common.function.PDFunction;
 
 /**
- * A Separation color space used to specify either additional colorants or for isolating the control of individual
- * colour components of a device colour space for a subtractive device. When such a space is the current colour space,
- * the current colour shall be a single-component value, called a tint, that controls the given colorant or colour
- * components only.
+ * A Separation color space used to specify either additional colorants or for isolating the control
+ * of individual colour components of a device colour space for a subtractive device. When such a
+ * space is the current colour space, the current colour shall be a single-component value, called a
+ * tint, that controls the given colorant or colour components only.
  *
  * @author Ben Litchfield
  * @author John Hewson
@@ -54,10 +55,11 @@ public class PDSeparation extends PDSpecialColorSpace
     private PDFunction tintTransform = null;
 
     /**
-     * Map used to speed up {@link #toRGB(float[])}. Note that this class contains three maps (this and the two in
-     * {@link #toRGBImage(java.awt.image.WritableRaster) } and {@link #toRGBImage2(java.awt.image.WritableRaster) }. The
-     * maps use different key intervals. This map here is needed for shading, which produce more than 256 different
-     * float values, which we cast to int so that the map can work.
+     * Map used to speed up {@link #toRGB(float[])}. Note that this class contains three maps (this
+     * and the two in {@link #toRGBImage(java.awt.image.WritableRaster) } and {@link
+     * #toRGBImage2(java.awt.image.WritableRaster) }. The maps use different key intervals. This map
+     * here is needed for shading, which produce more than 256 different float values, which we cast
+     * to int so that the map can work.
      */
     private Map<Integer, float[]> toRGBMap = null;
 
@@ -76,7 +78,7 @@ public class PDSeparation extends PDSpecialColorSpace
 
     /**
      * Creates a new Separation color space from a PDF color space array.
-     * 
+     *
      * @param separation an array containing all separation information.
      * @throws IOException if the color space or the function could not be created.
      */
@@ -85,6 +87,15 @@ public class PDSeparation extends PDSpecialColorSpace
         array = separation;
         alternateColorSpace = PDColorSpace.create(array.getObject(ALTERNATE_CS));
         tintTransform = PDFunction.create(array.getObject(TINT_TRANSFORM));
+        int numberOfOutputParameters = tintTransform.getNumberOfOutputParameters();
+        if (numberOfOutputParameters > 0
+                && numberOfOutputParameters < alternateColorSpace.getNumberOfComponents())
+        {
+            throw new IOException("The tint transform function has less output parameters ("
+                    + tintTransform.getNumberOfOutputParameters()
+                    + ") than the alternate colorspace " + alternateColorSpace + " ("
+                    + alternateColorSpace.getNumberOfComponents() + ")");
+        }
     }
 
     @Override
@@ -192,7 +203,8 @@ public class PDSeparation extends PDSpecialColorSpace
             for (int x = 0; x < width; x++)
             {
                 raster.getPixel(x, y, samples);
-                int[] rgb = calculatedValues.get(hash = Float.floatToIntBits(samples[0]));
+                hash = Float.floatToIntBits(samples[0]);
+                int[] rgb = calculatedValues.get(hash);
                 if (rgb == null)
                 {
                     samples[0] /= 255;
@@ -221,9 +233,15 @@ public class PDSeparation extends PDSpecialColorSpace
         }
     }
 
+    @Override
+    public BufferedImage toRawImage(WritableRaster raster)
+    {
+        return toRawImage(raster, ColorSpace.getInstance(ColorSpace.CS_GRAY));
+    }
+
     /**
      * Returns the colorant name.
-     * 
+     *
      * @return the name of the colorant
      */
     public PDColorSpace getAlternateColorSpace()
@@ -233,7 +251,7 @@ public class PDSeparation extends PDSpecialColorSpace
 
     /**
      * Returns the colorant name.
-     * 
+     *
      * @return the name of the colorant
      */
     public String getColorantName()
@@ -244,7 +262,7 @@ public class PDSeparation extends PDSpecialColorSpace
 
     /**
      * Sets the colorant name.
-     * 
+     *
      * @param name the name of the colorant
      */
     public void setColorantName(String name)
@@ -254,7 +272,7 @@ public class PDSeparation extends PDSpecialColorSpace
 
     /**
      * Sets the alternate color space.
-     * 
+     *
      * @param colorSpace The alternate color space.
      */
     public void setAlternateColorSpace(PDColorSpace colorSpace)
@@ -270,7 +288,7 @@ public class PDSeparation extends PDSpecialColorSpace
 
     /**
      * Sets the tint transform function.
-     * 
+     *
      * @param tint the tint transform function
      */
     public void setTintTransform(PDFunction tint)
