@@ -152,12 +152,7 @@ public class PDBorderStyleDictionary implements COSObjectable
      */
     public void setDashStyle(COSArray dashArray)
     {
-        COSArray array = null;
-        if (dashArray != null)
-        {
-            array = dashArray;
-        }
-        getCOSObject().setItem(COSName.D, array);
+        getCOSObject().setItem(COSName.D, dashArray);
     }
 
     /**
@@ -167,7 +162,7 @@ public class PDBorderStyleDictionary implements COSObjectable
      */
     public PDLineDashPattern getDashStyle()
     {
-        COSArray d = (COSArray) getCOSObject().getDictionaryObject(COSName.D);
+        COSArray d = getCOSObject().getDictionaryObject(COSName.D, COSArray.class);
         if (d == null)
         {
             d = new COSArray();
