@@ -18,11 +18,13 @@ package org.sejda.sambox.pdmodel.interactive.form;
 
 import static java.util.Arrays.asList;
 import static java.util.Objects.isNull;
+import static java.util.Objects.nonNull;
 import static java.util.Optional.ofNullable;
 import static org.sejda.commons.util.RequireUtils.requireNotNullArg;
 import static org.sejda.io.CountingWritableByteChannel.from;
 
 import java.awt.geom.AffineTransform;
+import java.awt.geom.GeneralPath;
 import java.awt.geom.Point2D;
 import java.io.IOException;
 import java.util.ArrayList;
@@ -30,7 +32,10 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import org.apache.fontbox.util.BoundingBox;
 import org.sejda.sambox.contentstream.operator.Operator;
+import org.sejda.sambox.cos.COSBase;
+import org.sejda.sambox.cos.COSDictionary;
 import org.sejda.sambox.cos.COSName;
 import org.sejda.sambox.cos.COSString;
 import org.sejda.sambox.input.ContentStreamParser;
@@ -39,7 +44,13 @@ import org.sejda.sambox.pdmodel.PDPageContentStream;
 import org.sejda.sambox.pdmodel.PDResources;
 import org.sejda.sambox.pdmodel.common.PDRectangle;
 import org.sejda.sambox.pdmodel.font.PDFont;
+import org.sejda.sambox.pdmodel.font.PDSimpleFont;
+import org.sejda.sambox.pdmodel.font.PDType3CharProc;
+import org.sejda.sambox.pdmodel.font.PDType3Font;
+import org.sejda.sambox.pdmodel.font.PDVectorFont;
 import org.sejda.sambox.pdmodel.graphics.color.PDColor;
+import org.sejda.sambox.pdmodel.interactive.action.PDAction;
+import org.sejda.sambox.pdmodel.interactive.action.PDActionJavaScript;
 import org.sejda.sambox.pdmodel.interactive.action.PDFormFieldAdditionalActions;
 import org.sejda.sambox.pdmodel.interactive.annotation.PDAnnotationWidget;
 import org.sejda.sambox.pdmodel.interactive.annotation.PDAppearanceCharacteristicsDictionary;
@@ -53,7 +64,7 @@ import org.slf4j.LoggerFactory;
 
 /**
  * Create the AcroForms field appearance helper.
- * 
+ *
  * @author Stephan Gerhard
  * @author Ben Litchfield
  */
@@ -71,10 +82,12 @@ public class AppearanceGeneratorHelper
 
     /**
      * The highlight color
-     *
-     * The color setting is used by Adobe to display the highlight box for selected entries in a list box.
-     *
-     * Regardless of other settings in an existing appearance stream Adobe will always use this value.
+     * <p>
+     * The color setting is used by Adobe to display the highlight box for selected entries in a
+     * list box.
+     * <p>
+     * Regardless of other settings in an existing appearance stream Adobe will always use this
+     * value.
      */
     private static final float[] HIGHLIGHT_COLOR = { 153 / 255f, 193 / 255f, 215 / 255f };
 
@@ -111,8 +124,7 @@ public class AppearanceGeneratorHelper
         {
             throw new IOException(
                     "Could not process default appearance string '" + field.getDefaultAppearance()
-                            + "' for field '" + field.getFullyQualifiedName() + "'",
-                    ex);
+                            + "' for field '" + field.getFullyQualifiedName() + "'", ex);
         }
     }
 
@@ -133,33 +145,40 @@ public class AppearanceGeneratorHelper
 
         for (PDAnnotationWidget widget : field.getWidgets())
         {
-            if (widget.getNormalAppearanceStream() != null
-                    && widget.getNormalAppearanceStream().getResources() != null)
+            PDAppearanceStream normalAppearanceStream = widget.getNormalAppearanceStream();
+            if (nonNull(normalAppearanceStream))
             {
-                PDResources widgetResources = widget.getNormalAppearanceStream().getResources();
-                Map<COSName, PDFont> missingFonts = new HashMap<>();
-                for (COSName fontResourceName : widgetResources.getFontNames())
+                PDResources widgetResources = normalAppearanceStream.getResources();
+                if (nonNull(widgetResources))
                 {
-                    try
+                    COSDictionary widgetFontDict = widgetResources.getCOSObject()
+                            .getDictionaryObject(COSName.FONT, COSDictionary.class);
+                    Map<COSName, COSBase> missingFonts = new HashMap<>();
+                    for (COSName fontResourceName : widgetResources.getFontNames())
                     {
-                        if (acroFormResources.getFont(fontResourceName) == null)
+                        try
                         {
-                            LOG.debug("Adding font resource " + fontResourceName
-                                    + " from widget to AcroForm");
-                            missingFonts.put(fontResourceName,
-                                    widgetResources.getFont(fontResourceName));
+                            if (acroFormResources.getFont(fontResourceName) == null)
+                            {
+                                LOG.debug("Adding font resource " + fontResourceName
+                                        + " from widget to AcroForm");
+                                missingFonts.put(fontResourceName,
+                                        widgetFontDict.getItem(fontResourceName));
+                            }
+                        }
+                        catch (IOException e)
+                        {
+                            LOG.warn("Unable to match field level font with AcroForm font");
                         }
                     }
-                    catch (IOException e)
-                    {
-                        LOG.warn("Unable to match field level font with AcroForm font");
-                    }
-                }
 
-                // add all missing font resources from widget to AcroForm
-                for (COSName key : missingFonts.keySet())
-                {
-                    acroFormResources.put(key, missingFonts.get(key));
+                    COSDictionary acroFormFontDict = acroFormResources.getCOSObject()
+                            .getDictionaryObject(COSName.FONT, COSDictionary.class);
+                    // add all missing font resources from widget to AcroForm
+                    for (COSName key : missingFonts.keySet())
+                    {
+                        acroFormFontDict.setItem(key, missingFonts.get(key));
+                    }
                 }
             }
         }
@@ -173,9 +192,9 @@ public class AppearanceGeneratorHelper
      */
     public void setAppearanceValue(String apValue) throws IOException
     {
-        value = apValue;
-        
-        if(field instanceof PDTextField)
+        value = getFormattedValue(apValue);
+
+        if (field instanceof PDTextField)
         {
             // avoid java.lang.IllegalArgumentException: U+00A0 ('nbspace') is not available in this font XYZ
             // TODO: generic way of handling any character which is not supported by the font?
@@ -196,6 +215,12 @@ public class AppearanceGeneratorHelper
 
         for (PDAnnotationWidget widget : field.getWidgets())
         {
+            if (widget.getCOSObject().containsKey("PMD"))
+            {
+                LOG.warn("Widget of field {} is a PaperMetaData, no appearance stream created",
+                        field.getFullyQualifiedName());
+                continue;
+            }
             // some fields have the /Da at the widget level if the
             // widgets differ in layout.
             PDDefaultAppearanceString acroFormAppearance = defaultAppearance;
@@ -214,51 +239,73 @@ public class AppearanceGeneratorHelper
                 continue;
             }
 
-            PDFormFieldAdditionalActions actions = field.getActions();
-
-            // in case all tests fail the field will be formatted by acrobat
-            // when it is opened. See FreedomExpressions.pdf for an example of this.
-            if (actions == null || actions.getF() == null
-                    || widget.getCOSObject().getDictionaryObject(COSName.AP) != null)
+            PDAppearanceDictionary appearanceDict = widget.getAppearance();
+            if (appearanceDict == null)
             {
-                PDAppearanceDictionary appearanceDict = widget.getAppearance();
-                if (appearanceDict == null)
-                {
-                    appearanceDict = new PDAppearanceDictionary();
-                    widget.setAppearance(appearanceDict);
-                }
-
-                PDAppearanceEntry appearance = appearanceDict.getNormalAppearance();
-                // TODO support appearances other than "normal"
-
-                PDAppearanceStream appearanceStream;
-                if (isValidAppearanceStream(appearance))
-                {
-                    appearanceStream = appearance.getAppearanceStream();
-                }
-                else
-                {
-                    appearanceStream = prepareNormalAppearanceStream(widget);
-                    appearanceDict.setNormalAppearance(appearanceStream);
-                    // TODO support appearances other than "normal"
-                }
-                /*
-                 * Adobe Acrobat always recreates the complete appearance stream if there is an appearance
-                 * characteristics entry (the widget dictionaries MK entry). In addition if there is no content yet also
-                 * create the apperance stream from the entries.
-                 */
-                if (widget.getAppearanceCharacteristics() != null
-                        || appearanceStream.getContentStream().getLength() == 0)
-                {
-                    initializeAppearanceContent(widget, appearanceStream);
-                }
-                setAppearanceContent(widget, appearanceStream);
+                appearanceDict = new PDAppearanceDictionary();
+                widget.setAppearance(appearanceDict);
             }
-            // restore the field level appearance;
+
+            PDAppearanceEntry appearance = appearanceDict.getNormalAppearance();
+            // TODO support appearances other than "normal"
+
+            PDAppearanceStream appearanceStream;
+            if (isValidAppearanceStream(appearance))
+            {
+                appearanceStream = appearance.getAppearanceStream();
+            }
+            else
+            {
+                appearanceStream = prepareNormalAppearanceStream(widget);
+                appearanceDict.setNormalAppearance(appearanceStream);
+                // TODO support appearances other than "normal"
+            }
+            PDAppearanceCharacteristicsDictionary appearanceCharacteristics = widget.getAppearanceCharacteristics();
+
+            /*
+             * Adobe Acrobat always recreates the complete appearance stream if there is an
+             * appearance characteristics entry (the widget dictionaries MK entry). In
+             * addition if there is no content yet also create the appearance stream from
+             * the entries.
+             *
+             */
+            if (appearanceCharacteristics != null
+                    || appearanceStream.getContentStream().getLength() == 0)
+            {
+                initializeAppearanceContent(widget, appearanceCharacteristics, appearanceStream);
+            }
+
+            setAppearanceContent(widget, appearanceStream);
+
+            // restore the field level appearance
             defaultAppearance = acroFormAppearance;
         }
     }
 
+    private String getFormattedValue(String apValue)
+    {
+        // format the field value for the appearance if there is scripting support and
+        // the field
+        // has a format event
+        PDFormFieldAdditionalActions actions = field.getActions();
+        if (actions == null)
+        {
+            return apValue;
+        }
+        PDAction actionF = actions.getF();
+        if (actionF != null)
+        {
+            if (field.getAcroForm().getScriptingHandler() != null)
+            {
+                ScriptingHandler scriptingHandler = field.getAcroForm().getScriptingHandler();
+                return scriptingHandler.format((PDActionJavaScript) actionF, apValue);
+            }
+            LOG.info(
+                    "Field contains a formatting action but no ScriptingHandler has been supplied - formatted value might be incorrect");
+        }
+        return apValue;
+    }
+
     private static boolean isValidAppearanceStream(PDAppearanceEntry appearance)
     {
         if (isNull(appearance) || !appearance.isStream())
@@ -301,7 +348,8 @@ public class AppearanceGeneratorHelper
     private PDDefaultAppearanceString getWidgetDefaultAppearanceString(PDAnnotationWidget widget)
             throws IOException
     {
-        COSString da = DefaultAppearanceHelper.getDefaultAppearance(widget.getCOSObject().getDictionaryObject(COSName.DA));
+        COSString da = DefaultAppearanceHelper.getDefaultAppearance(
+                widget.getCOSObject().getDictionaryObject(COSName.DA));
         PDResources dr = field.getAcroForm().getDefaultResources();
         try
         {
@@ -318,8 +366,7 @@ public class AppearanceGeneratorHelper
 
     private int resolveRotation(PDAnnotationWidget widget)
     {
-        PDAppearanceCharacteristicsDictionary characteristicsDictionary = widget
-                .getAppearanceCharacteristics();
+        PDAppearanceCharacteristicsDictionary characteristicsDictionary = widget.getAppearanceCharacteristics();
         if (characteristicsDictionary != null)
         {
             // 0 is the default value if the R key doesn't exist
@@ -330,23 +377,21 @@ public class AppearanceGeneratorHelper
 
     /**
      * Initialize the content of the appearance stream.
-     * 
-     * Get settings like border style, border width and colors to be used to draw a rectangle and background color
-     * around the widget
-     * 
-     * @param widget the field widget
+     * <p>
+     * Get settings like border style, border width and colors to be used to draw a rectangle and
+     * background color around the widget
+     *
+     * @param widget           the field widget
      * @param appearanceStream the appearance stream to be used
      * @throws IOException in case we can't write to the appearance stream
      */
     private void initializeAppearanceContent(PDAnnotationWidget widget,
+            PDAppearanceCharacteristicsDictionary appearanceCharacteristics,
             PDAppearanceStream appearanceStream) throws IOException
     {
         try (PDPageContentStream contents = new PDPageContentStream(
                 field.getAcroForm().getDocument(), appearanceStream))
         {
-            PDAppearanceCharacteristicsDictionary appearanceCharacteristics = widget
-                    .getAppearanceCharacteristics();
-
             // TODO: support more entries like patterns, etc.
             if (appearanceCharacteristics != null)
             {
@@ -408,13 +453,17 @@ public class AppearanceGeneratorHelper
         // the stream’s Resources dictionary
         defaultAppearance.copyNeededResourcesTo(appearanceStream);
 
+        // IMPORTANT: tokenize before opening stream for writing
+        // as .createUnfilteredStream() wipes all unsaved changes done to the stream in current session,
+        // for example by initializeAppearanceContent(), so we end up without borders for the widget
+        List<Object> tokens = tokenize(appearanceStream);
+
         // then replace the existing contents of the appearance stream from /Tx BMC
         // to the matching EMC
         try (ContentStreamWriter writer = new ContentStreamWriter(
                 from(appearanceStream.getCOSObject().createUnfilteredStream())))
         {
 
-            List<Object> tokens = tokenize(appearanceStream);
             int bmcIndex = tokens.indexOf(BMC);
             if (bmcIndex == -1)
             {
@@ -479,8 +528,8 @@ public class AppearanceGeneratorHelper
         // callers might have determined that the default font does not support rendering the field's value
         // so the font was substituted to another one, which has better unicode support
         // see PDVariableText.setAppearanceOverrideFont()
-        PDFont font = ofNullable(field.getAppearanceFont())
-                .orElseGet(() -> defaultAppearance.getFont());
+        PDFont font = ofNullable(field.getAppearanceFont()).orElseGet(
+                () -> defaultAppearance.getFont());
 
         requireNotNullArg(font, "font is null, check whether /DA entry is incomplete or incorrect");
         if (font.getName().contains("+"))
@@ -513,7 +562,8 @@ public class AppearanceGeneratorHelper
         {
             float newFontSize = calculateFontSize(font, contentRect);
             // avoid increasing the font size, if one was already specified
-            if(newFontSize < fontSize || fontSize == 0) 
+            // newFontSize can be infinite when contentRect width/height or string width are 0
+            if (Float.isFinite(newFontSize) && (newFontSize < fontSize || fontSize == 0))
             {
                 fontSize = newFontSize;
             }
@@ -537,8 +587,23 @@ public class AppearanceGeneratorHelper
         // calculate font metrics at font size
         float fontScaleY = fontSize / FONTSCALE;
         float fontBoundingBoxAtSize = font.getBoundingBox().getHeight() * fontScaleY;
-        float fontCapAtSize = font.getFontDescriptor().getCapHeight() * fontScaleY;
-        float fontDescentAtSize = font.getFontDescriptor().getDescent() * fontScaleY;
+        float fontCapAtSize;
+        float fontDescentAtSize;
+
+        if (font.getFontDescriptor() != null)
+        {
+            fontCapAtSize = font.getFontDescriptor().getCapHeight() * fontScaleY;
+            fontDescentAtSize = font.getFontDescriptor().getDescent() * fontScaleY;
+        }
+        else
+        {
+            float fontCapHeight = resolveCapHeight(font);
+            float fontDescent = resolveDescent(font);
+            LOG.debug("missing font descriptor - resolved Cap/Descent to " + fontCapHeight + "/"
+                    + fontDescent);
+            fontCapAtSize = fontCapHeight * fontScaleY;
+            fontDescentAtSize = fontDescent * fontScaleY;
+        }
 
         if (field instanceof PDTextField && ((PDTextField) field).isMultiline())
         {
@@ -546,7 +611,7 @@ public class AppearanceGeneratorHelper
         }
         else
         {
-            // Adobe shows the text 'shiftet up' in case the caps don't fit into the clipping area
+            // Adobe shows the text 'shifted up' in case the caps don't fit into the clipping area
             if (fontCapAtSize > clipRect.getHeight())
             {
                 y = clipRect.getLowerLeftY() + -fontDescentAtSize;
@@ -564,8 +629,8 @@ public class AppearanceGeneratorHelper
                 {
 
                     float fontDescentBased = -fontDescentAtSize + contentRect.getLowerLeftY();
-                    float fontCapBased = contentRect.getHeight() - contentRect.getLowerLeftY()
-                            - fontCapAtSize;
+                    float fontCapBased =
+                            contentRect.getHeight() - contentRect.getLowerLeftY() - fontCapAtSize;
 
                     y = Math.min(fontDescentBased, Math.max(y, fontCapBased));
                 }
@@ -596,9 +661,10 @@ public class AppearanceGeneratorHelper
             // Adobe Acrobat uses the font's bounding box for the leading between the lines
             appearanceStyle.setLeading(calculateLineHeight(font, fontScaleY));
 
-            PlainTextFormatter formatter = new PlainTextFormatter.Builder(contents)
-                    .style(appearanceStyle).text(textContent).width(contentRect.getWidth())
-                    .wrapLines(isMultiLine()).initialOffset(x, y).textAlign(field.getQ()).build();
+            PlainTextFormatter formatter = new PlainTextFormatter.Builder(contents).style(
+                            appearanceStyle).text(textContent).width(contentRect.getWidth())
+                    .wrapLines(isMultiLine()).initialOffset(x, y).textAlign(getTextAlign(widget))
+                    .build();
             formatter.format();
         }
 
@@ -606,6 +672,17 @@ public class AppearanceGeneratorHelper
         contents.restoreGraphicsState();
     }
 
+    /**
+     * PDFBox handles a widget with a joined in field dictionary and without an individual name as a
+     * widget only. As a result - as a widget can't have a quadding /Q entry we need to do a low
+     * level access to the dictionary and otherwise get the quadding from the field.
+     */
+    private int getTextAlign(PDAnnotationWidget widget)
+    {
+        // Use quadding value from joined field/widget if set, else use from field.
+        return widget.getCOSObject().getInt(COSName.Q, field.getQ());
+    }
+
     private AffineTransform calculateMatrix(PDRectangle bbox, int rotation)
     {
         if (rotation == 0)
@@ -640,13 +717,14 @@ public class AppearanceGeneratorHelper
 
     /**
      * Determine if the appearance shall provide a comb output.
-     * 
+     *
      * <p>
-     * May be set only if the MaxLen entry is present in the text field dictionary and if the Multiline, Password, and
-     * FileSelect flags are clear. If set, the field shall be automatically divided into as many equally spaced
-     * positions, or combs, as the value of MaxLen, and the text is laid out into those combs.
+     * May be set only if the MaxLen entry is present in the text field dictionary and if the
+     * Multiline, Password, and FileSelect flags are clear. If set, the field shall be automatically
+     * divided into as many equally spaced positions, or combs, as the value of MaxLen, and the text
+     * is laid out into those combs.
      * </p>
-     * 
+     *
      * @return the comb state
      */
     private boolean shallComb()
@@ -658,10 +736,10 @@ public class AppearanceGeneratorHelper
 
     /**
      * Generate the appearance for comb fields.
-     * 
+     *
      * @param contents the content stream to write to
-     * @param bbox the bbox used
-     * @param font the font to be used
+     * @param bbox     the bbox used
+     * @param font     the font to be used
      * @param fontSize the font size to be used
      * @throws IOException
      */
@@ -669,18 +747,16 @@ public class AppearanceGeneratorHelper
             PDFont font, float fontSize) throws IOException
     {
 
-        // TODO: Currently the quadding is not taken into account
-        // so the comb is always filled from left to right.
-
         int maxLen = ((PDTextField) field).getMaxLen();
+        int quadding = field.getQ();
         int numChars = Math.min(value.length(), maxLen);
 
         PDRectangle paddingEdge = applyPadding(bbox, 1);
 
         float combWidth = bbox.getWidth() / maxLen;
         float ascentAtFontSize = font.getFontDescriptor().getAscent() / FONTSCALE * fontSize;
-        float baselineOffset = paddingEdge.getLowerLeftY()
-                + (bbox.getHeight() - ascentAtFontSize) / 2;
+        float baselineOffset =
+                paddingEdge.getLowerLeftY() + (bbox.getHeight() - ascentAtFontSize) / 2;
 
         float prevCharWidth = 0f;
 
@@ -689,6 +765,16 @@ public class AppearanceGeneratorHelper
         contents.saveGraphicsState();
         contents.setFont(font, fontSize);
 
+        // add to initial offset if right aligned or centered
+        if (quadding == 2)
+        {
+            xOffset = xOffset + (maxLen - numChars) * combWidth;
+        }
+        else if (quadding == 1)
+        {
+            xOffset = xOffset + (maxLen - numChars) / 2 * combWidth;
+        }
+
         for (int i = 0; i < numChars; i++)
         {
             String combString = value.substring(i, i + 1);
@@ -710,14 +796,15 @@ public class AppearanceGeneratorHelper
     private void insertGeneratedListboxSelectionHighlight(PDPageContentStream contents,
             PDAppearanceStream appearanceStream, PDFont font, float fontSize) throws IOException
     {
-        List<Integer> indexEntries = ((PDListBox) field).getSelectedOptionsIndex();
-        List<String> values = ((PDListBox) field).getValue();
-        List<String> options = ((PDListBox) field).getOptionsExportValues();
+        PDListBox listBox = (PDListBox) field;
+        List<Integer> indexEntries = listBox.getSelectedOptionsIndex();
+        List<String> values = listBox.getValue();
+        List<String> options = listBox.getOptionsExportValues();
 
         if (!values.isEmpty() && !options.isEmpty() && indexEntries.isEmpty())
         {
             // create indexEntries from options
-            indexEntries = new ArrayList<>();
+            indexEntries = new ArrayList<Integer>(values.size());
             for (String v : values)
             {
                 indexEntries.add(options.indexOf(v));
@@ -725,9 +812,9 @@ public class AppearanceGeneratorHelper
         }
 
         // The first entry which shall be presented might be adjusted by the optional TI key
-        // If this entry is present the first entry to be displayed is the keys value otherwise
-        // display starts with the first entry in Opt.
-        int topIndex = ((PDListBox) field).getTopIndex();
+        // If this entry is present, the first entry to be displayed is the keys value,
+        // otherwise display starts with the first entry in Opt.
+        int topIndex = listBox.getTopIndex();
 
         float highlightBoxHeight = font.getBoundingBox().getHeight() * fontSize / FONTSCALE;
 
@@ -740,9 +827,8 @@ public class AppearanceGeneratorHelper
                     HIGHLIGHT_COLOR[2]);
 
             contents.addRect(paddingEdge.getLowerLeftX(),
-                    paddingEdge.getUpperRightY()
-                            - highlightBoxHeight * (selectedIndex - topIndex + 1) + 2,
-                    paddingEdge.getWidth(), highlightBoxHeight);
+                    paddingEdge.getUpperRightY() - highlightBoxHeight * (selectedIndex - topIndex
+                            + 1) + 2, paddingEdge.getWidth(), highlightBoxHeight);
             contents.fill();
         }
         contents.setNonStrokingColor(0f);
@@ -779,17 +865,19 @@ public class AppearanceGeneratorHelper
         float yTextPos = contentRect.getUpperRightY();
 
         int topIndex = ((PDListBox) field).getTopIndex();
+        float ascent = font.getFontDescriptor().getAscent();
+        float height = font.getBoundingBox().getHeight();
 
         for (int i = topIndex; i < numOptions; i++)
         {
 
             if (i == topIndex)
             {
-                yTextPos = yTextPos - font.getFontDescriptor().getAscent() / FONTSCALE * fontSize;
+                yTextPos = yTextPos - ascent / FONTSCALE * fontSize;
             }
             else
             {
-                yTextPos = yTextPos - font.getBoundingBox().getHeight() / FONTSCALE * fontSize;
+                yTextPos = yTextPos - height / FONTSCALE * fontSize;
                 contents.beginText();
             }
 
@@ -803,7 +891,7 @@ public class AppearanceGeneratorHelper
             }
         }
     }
-    
+
     private String replaceTabChars(String input)
     {
         // avoid java.lang.IllegalArgumentException: U+0009 ('controlHT') is not available in this font XYZ
@@ -813,8 +901,23 @@ public class AppearanceGeneratorHelper
     private float calculateLineHeight(PDFont font, float fontScaleY) throws IOException
     {
         float fontBoundingBoxAtSize = font.getBoundingBox().getHeight() * fontScaleY;
-        float fontCapAtSize = font.getFontDescriptor().getCapHeight() * fontScaleY;
-        float fontDescentAtSize = font.getFontDescriptor().getDescent() * fontScaleY;
+        float fontCapAtSize = 0;
+        float fontDescentAtSize = 0;
+
+        if (font.getFontDescriptor() != null)
+        {
+            fontCapAtSize = font.getFontDescriptor().getCapHeight() * fontScaleY;
+            fontDescentAtSize = font.getFontDescriptor().getDescent() * fontScaleY;
+        }
+        else
+        {
+            float fontCapHeight = resolveCapHeight(font);
+            float fontDescent = resolveDescent(font);
+            LOG.debug("missing font descriptor - resolved Cap/Descent to " + fontCapHeight + "/"
+                    + fontDescent);
+            fontCapAtSize = fontCapHeight * fontScaleY;
+            fontDescentAtSize = fontDescent * fontScaleY;
+        }
 
         float lineHeight = fontCapAtSize - fontDescentAtSize;
         if (lineHeight < 0)
@@ -826,8 +929,9 @@ public class AppearanceGeneratorHelper
     }
 
     /**
-     * My "not so great" method for calculating the fontsize. It does not work superb, but it handles ok.
-     * 
+     * My "not so great" method for calculating the fontsize. It does not work superb, but it
+     * handles ok.
+     *
      * @return the calculated font-size
      * @throws IOException If there is an error getting the font information.
      */
@@ -850,8 +954,8 @@ public class AppearanceGeneratorHelper
             float lineHeight = calculateLineHeight(font, font.getFontMatrix().getScaleY());
             float scaledContentHeight = contentRect.getHeight() * yScalingFactor;
 
-            boolean looksLikeFauxMultiline = calculateLineHeight(font,
-                    DEFAULT_FONT_SIZE / FONTSCALE) > scaledContentHeight;
+            boolean looksLikeFauxMultiline =
+                    calculateLineHeight(font, DEFAULT_FONT_SIZE / FONTSCALE) > scaledContentHeight;
             int userTypedLinesCount = new PlainText(value).getParagraphs().size();
             boolean userTypedMultipleLines = userTypedLinesCount > 1;
 
@@ -867,8 +971,8 @@ public class AppearanceGeneratorHelper
             {
                 // calculate a font size which fits at least x lines
                 int minimumLinesToFitInAMultilineField = Math.max(2, userTypedLinesCount);
-                float fontSize = scaledContentHeight
-                        / (minimumLinesToFitInAMultilineField * lineHeight);
+                float fontSize =
+                        scaledContentHeight / (minimumLinesToFitInAMultilineField * lineHeight);
                 // don't return a font size larger than the default
                 return Math.min(fontSize, DEFAULT_FONT_SIZE);
             }
@@ -886,10 +990,89 @@ public class AppearanceGeneratorHelper
         return Math.min(heightBasedFontSize, widthBasedFontSize);
     }
 
+    /*
+     * Resolve the cap height.
+     *
+     * This is a very basic implementation using the height of "H" as reference.
+     */
+    private float resolveCapHeight(PDFont font) throws IOException
+    {
+        return resolveGlyphHeight(font, "H".codePointAt(0));
+    }
+
+    /*
+     * Resolve the descent.
+     *
+     * This is a very basic implementation using the height of "y" - "a" as reference.
+     */
+    private float resolveDescent(PDFont font) throws IOException
+    {
+        return resolveGlyphHeight(font, "y".codePointAt(0)) - resolveGlyphHeight(font,
+                "a".codePointAt(0));
+    }
+
+    // this calculates the real (except for type 3 fonts) individual glyph bounds
+    private float resolveGlyphHeight(PDFont font, int code) throws IOException
+    {
+        GeneralPath path = null;
+        if (font instanceof PDType3Font)
+        {
+            // It is difficult to calculate the real individual glyph bounds for type 3
+            // fonts
+            // because these are not vector fonts, the content stream could contain almost
+            // anything
+            // that is found in page content streams.
+            PDType3Font t3Font = (PDType3Font) font;
+            PDType3CharProc charProc = t3Font.getCharProc(code);
+            if (charProc != null)
+            {
+                BoundingBox fontBBox = t3Font.getBoundingBox();
+                PDRectangle glyphBBox = charProc.getGlyphBBox();
+                if (glyphBBox != null)
+                {
+                    // PDFBOX-3850: glyph bbox could be larger than the font bbox
+                    glyphBBox.setLowerLeftX(
+                            Math.max(fontBBox.getLowerLeftX(), glyphBBox.getLowerLeftX()));
+                    glyphBBox.setLowerLeftY(
+                            Math.max(fontBBox.getLowerLeftY(), glyphBBox.getLowerLeftY()));
+                    glyphBBox.setUpperRightX(
+                            Math.min(fontBBox.getUpperRightX(), glyphBBox.getUpperRightX()));
+                    glyphBBox.setUpperRightY(
+                            Math.min(fontBBox.getUpperRightY(), glyphBBox.getUpperRightY()));
+                    path = glyphBBox.toGeneralPath();
+                }
+            }
+        }
+        else if (font instanceof PDVectorFont)
+        {
+            PDVectorFont vectorFont = (PDVectorFont) font;
+            path = vectorFont.getPath(code);
+        }
+        else if (font instanceof PDSimpleFont)
+        {
+            PDSimpleFont simpleFont = (PDSimpleFont) font;
+
+            // these two lines do not always work, e.g. for the TT fonts in file 032431.pdf
+            // which is why PDVectorFont is tried first.
+            String name = simpleFont.getEncoding().getName(code);
+            path = simpleFont.getPath(name);
+        }
+        else
+        {
+            // shouldn't happen, please open issue in JIRA
+            LOG.warn("Unknown font class: " + font.getClass());
+        }
+        if (path == null)
+        {
+            return -1;
+        }
+        return (float) path.getBounds2D().getHeight();
+    }
+
     /**
      * Resolve the bounding box.
-     * 
-     * @param fieldWidget the annotation widget.
+     *
+     * @param fieldWidget      the annotation widget.
      * @param appearanceStream the annotations appearance stream.
      * @return the resolved boundingBox.
      */
