@@ -28,6 +28,7 @@ import java.io.IOException;
 import org.sejda.sambox.cos.COSName;
 import org.sejda.sambox.pdmodel.PDDocument;
 import org.sejda.sambox.pdmodel.PDPage;
+import org.sejda.sambox.pdmodel.PDPageTree;
 import org.sejda.sambox.pdmodel.PDResources;
 import org.sejda.sambox.pdmodel.common.PDRectangle;
 import org.sejda.sambox.pdmodel.graphics.blend.BlendMode;
@@ -38,7 +39,8 @@ import org.sejda.sambox.pdmodel.interactive.annotation.AnnotationFilter;
 import org.sejda.sambox.pdmodel.interactive.annotation.PDAnnotation;
 
 /**
- * Renders a PDF document to an AWT BufferedImage. This class may be overridden in order to perform custom rendering.
+ * Renders a PDF document to an AWT BufferedImage. This class may be overridden in order to perform
+ * custom rendering.
  *
  * @author John Hewson
  */
@@ -67,14 +69,17 @@ public class PDFRenderer
 
     private BufferedImage pageImage;
 
+    private final PDPageTree pageTree;
+
     /**
      * Creates a new PDFRenderer.
-     * 
+     *
      * @param document the document to render
      */
     public PDFRenderer(PDDocument document)
     {
         this.document = document;
+        this.pageTree = document.getPages();
     }
 
     /**
@@ -101,11 +106,11 @@ public class PDFRenderer
     }
 
     /**
-     * Value indicating if the renderer is allowed to subsample images before drawing, according to image dimensions and
-     * requested scale.
-     *
-     * Subsampling may be faster and less memory-intensive in some cases, but it may also lead to loss of quality,
-     * especially in images with high spatial frequency.
+     * Value indicating if the renderer is allowed to subsample images before drawing, according to
+     * image dimensions and requested scale.
+     * <p>
+     * Subsampling may be faster and less memory-intensive in some cases, but it may also lead to
+     * loss of quality, especially in images with high spatial frequency.
      *
      * @return true if subsampling of images is allowed, false otherwise.
      */
@@ -115,11 +120,12 @@ public class PDFRenderer
     }
 
     /**
-     * Sets a value instructing the renderer whether it is allowed to subsample images before drawing. The subsampling
-     * frequency is determined according to image size and requested scale.
-     *
-     * Subsampling may be faster and less memory-intensive in some cases, but it may also lead to loss of quality,
-     * especially in images with high spatial frequency.
+     * Sets a value instructing the renderer whether it is allowed to subsample images before
+     * drawing. The subsampling frequency is determined according to image size and requested
+     * scale.
+     * <p>
+     * Subsampling may be faster and less memory-intensive in some cases, but it may also lead to
+     * loss of quality, especially in images with high spatial frequency.
      *
      * @param subsamplingAllowed The new value indicating if subsampling is allowed.
      */
@@ -155,8 +161,9 @@ public class PDFRenderer
     }
 
     /**
-     * Set the rendering hints. Use this to influence rendering quality and speed. If you don't set them yourself or
-     * pass null, PDFBox will decide <b><u>at runtime</u></b> depending on the destination.
+     * Set the rendering hints. Use this to influence rendering quality and speed. If you don't set
+     * them yourself or pass null, PDFBox will decide <b><u>at runtime</u></b> depending on the
+     * destination.
      *
      * @param renderingHints
      */
@@ -167,7 +174,7 @@ public class PDFRenderer
 
     /**
      * Returns the given page as an RGB image at 72 DPI
-     * 
+     *
      * @param pageIndex the zero-based index of the page to be converted.
      * @return the rendered page image
      * @throws IOException if the PDF cannot be read
@@ -178,10 +185,11 @@ public class PDFRenderer
     }
 
     /**
-     * Returns the given page as an RGB image at the given scale. A scale of 1 will render at 72 DPI.
-     * 
+     * Returns the given page as an RGB image at the given scale. A scale of 1 will render at 72
+     * DPI.
+     *
      * @param pageIndex the zero-based index of the page to be converted
-     * @param scale the scaling factor, where 1 = 72 DPI
+     * @param scale     the scaling factor, where 1 = 72 DPI
      * @return the rendered page image
      * @throws IOException if the PDF cannot be read
      */
@@ -192,9 +200,9 @@ public class PDFRenderer
 
     /**
      * Returns the given page as an RGB image at the given DPI.
-     * 
+     *
      * @param pageIndex the zero-based index of the page to be converted
-     * @param dpi the DPI (dots per inch) to render at
+     * @param dpi       the DPI (dots per inch) to render at
      * @return the rendered page image
      * @throws IOException if the PDF cannot be read
      */
@@ -205,9 +213,9 @@ public class PDFRenderer
 
     /**
      * Returns the given page as an RGB image at the given DPI.
-     * 
+     *
      * @param pageIndex the zero-based index of the page to be converted
-     * @param dpi the DPI (dots per inch) to render at
+     * @param dpi       the DPI (dots per inch) to render at
      * @param imageType the type of image to return
      * @return the rendered page image
      * @throws IOException if the PDF cannot be read
@@ -220,9 +228,9 @@ public class PDFRenderer
 
     /**
      * Returns the given page as an RGB or ARGB image at the given scale.
-     * 
+     *
      * @param pageIndex the zero-based index of the page to be converted
-     * @param scale the scaling factor, where 1 = 72 DPI
+     * @param scale     the scaling factor, where 1 = 72 DPI
      * @param imageType the type of image to return
      * @return the rendered page image
      * @throws IOException if the PDF cannot be read
@@ -236,10 +244,10 @@ public class PDFRenderer
 
     /**
      * Returns the given page as an RGB or ARGB image at the given scale.
-     * 
-     * @param pageIndex the zero-based index of the page to be converted
-     * @param scale the scaling factor, where 1 = 72 DPI
-     * @param imageType the type of image to return
+     *
+     * @param pageIndex   the zero-based index of the page to be converted
+     * @param scale       the scaling factor, where 1 = 72 DPI
+     * @param imageType   the type of image to return
      * @param destination controlling visibility of optional content groups
      * @return the rendered page image
      * @throws IOException if the PDF cannot be read
@@ -247,11 +255,11 @@ public class PDFRenderer
     public BufferedImage renderImage(int pageIndex, float scale, ImageType imageType,
             RenderDestination destination) throws IOException
     {
-        PDPage page = document.getPage(pageIndex);
+        PDPage page = pageTree.get(pageIndex);
 
-        PDRectangle cropbBox = page.getCropBox();
-        float widthPt = cropbBox.getWidth();
-        float heightPt = cropbBox.getHeight();
+        PDRectangle cropBox = page.getCropBox();
+        float widthPt = cropBox.getWidth();
+        float heightPt = cropBox.getHeight();
 
         // PDFBOX-4306 avoid single blank pixel line on the right or on the bottom
         int widthPx = (int) Math.max(Math.floor(widthPt * scale), 1);
@@ -260,21 +268,25 @@ public class PDFRenderer
         // PDFBOX-4518 the maximum size (w*h) of a buffered image is limited to Integer.MAX_VALUE
         if ((long) widthPx * (long) heightPx > Integer.MAX_VALUE)
         {
-            throw new IOException("Maximum size of image exceeded (w * h * scale) = "//
-                    + widthPt + " * " + heightPt + " * " + scale + " > " + Integer.MAX_VALUE);
+            throw new IOException("Maximum size of image exceeded (w * h * scale ^ 2) = "//
+                    + widthPt + " * " + heightPt + " * " + scale + " ^ 2 > " + Integer.MAX_VALUE);
         }
 
         int rotationAngle = page.getRotation();
 
-        int bimType = imageType.toBufferedImageType();
+        int bimType;
         if (imageType != ImageType.ARGB && hasBlendMode(page))
         {
             // PDFBOX-4095: if the PDF has blending on the top level, draw on transparent background
-            // Inpired from PDF.js: if a PDF page uses any blend modes other than Normal,
+            // Inspired from PDF.js: if a PDF page uses any blend modes other than Normal,
             // PDF.js renders everything on a fully transparent RGBA canvas.
             // Finally when the page has been rendered, PDF.js draws the RGBA canvas on a white canvas.
             bimType = BufferedImage.TYPE_INT_ARGB;
         }
+        else
+        {
+            bimType = imageType.toBufferedImageType();
+        }
 
         // swap width and height
         BufferedImage image;
@@ -300,15 +312,15 @@ public class PDFRenderer
         }
         g.clearRect(0, 0, image.getWidth(), image.getHeight());
 
-        transform(g, page, scale, scale);
+        transform(g, page.getRotation(), cropBox, scale, scale);
 
         // the end-user may provide a custom PageDrawer
-        RenderingHints actualRenderingHints = renderingHints == null
-                ? createDefaultRenderingHints(g) : renderingHints;
+        RenderingHints actualRenderingHints =
+                renderingHints == null ? createDefaultRenderingHints(g) : renderingHints;
         PageDrawerParameters parameters = new PageDrawerParameters(this, page, subsamplingAllowed,
                 destination, actualRenderingHints);
         PageDrawer drawer = createPageDrawer(parameters);
-        drawer.drawPage(g, page.getCropBox());
+        drawer.drawPage(g, cropBox);
 
         g.dispose();
 
@@ -330,9 +342,9 @@ public class PDFRenderer
 
     /**
      * Renders a given page to an AWT Graphics2D instance.
-     * 
+     *
      * @param pageIndex the zero-based index of the page to be converted
-     * @param graphics the Graphics2D on which to draw the page
+     * @param graphics  the Graphics2D on which to draw the page
      * @throws IOException if the PDF cannot be read
      */
     public void renderPageToGraphics(int pageIndex, Graphics2D graphics) throws IOException
@@ -342,10 +354,10 @@ public class PDFRenderer
 
     /**
      * Renders a given page to an AWT Graphics2D instance.
-     * 
+     *
      * @param pageIndex the zero-based index of the page to be converted
-     * @param graphics the Graphics2D on which to draw the page
-     * @param scale the scale to draw the page at
+     * @param graphics  the Graphics2D on which to draw the page
+     * @param scale     the scale to draw the page at
      * @throws IOException if the PDF cannot be read
      */
     public void renderPageToGraphics(int pageIndex, Graphics2D graphics, float scale)
@@ -356,11 +368,11 @@ public class PDFRenderer
 
     /**
      * Renders a given page to an AWT Graphics2D instance.
-     * 
+     *
      * @param pageIndex the zero-based index of the page to be converted
-     * @param graphics the Graphics2D on which to draw the page
-     * @param scaleX the scale to draw the page at for the x-axis
-     * @param scaleY the scale to draw the page at for the y-axis
+     * @param graphics  the Graphics2D on which to draw the page
+     * @param scaleX    the scale to draw the page at for the x-axis
+     * @param scaleY    the scale to draw the page at for the y-axis
      * @throws IOException if the PDF cannot be read
      */
     public void renderPageToGraphics(int pageIndex, Graphics2D graphics, float scaleX, float scaleY)
@@ -384,28 +396,27 @@ public class PDFRenderer
      * </ul>
      * If you encounter these problems, then you should render into an image by using the {@link #renderImage(int)
      * renderImage} methods.
-     * 
-     * @param pageIndex the zero-based index of the page to be converted
-     * @param graphics the Graphics2D on which to draw the page
-     * @param scaleX the scale to draw the page at for the x-axis, where 1 = 72 DPI
-     * @param scaleY the scale to draw the page at for the y-axis, where 1 = 72 DPI
+     *
+     * @param pageIndex   the zero-based index of the page to be converted
+     * @param graphics    the Graphics2D on which to draw the page
+     * @param scaleX      the scale to draw the page at for the x-axis, where 1 = 72 DPI
+     * @param scaleY      the scale to draw the page at for the y-axis, where 1 = 72 DPI
      * @param destination controlling visibility of optional content groups
      * @throws IOException if the PDF cannot be read
      */
     public void renderPageToGraphics(int pageIndex, Graphics2D graphics, float scaleX, float scaleY,
             RenderDestination destination) throws IOException
     {
-        PDPage page = document.getPage(pageIndex);
+        PDPage page = pageTree.get(pageIndex);
         // TODO need width/height calculations? should these be in PageDrawer?
 
-        transform(graphics, page, scaleX, scaleY);
-
         PDRectangle cropBox = page.getCropBox();
+        transform(graphics, page.getRotation(), cropBox, scaleX, scaleY);
         graphics.clearRect(0, 0, (int) cropBox.getWidth(), (int) cropBox.getHeight());
 
         // the end-user may provide a custom PageDrawer
-        RenderingHints actualRenderingHints = renderingHints == null
-                ? createDefaultRenderingHints(graphics) : renderingHints;
+        RenderingHints actualRenderingHints =
+                renderingHints == null ? createDefaultRenderingHints(graphics) : renderingHints;
         PageDrawerParameters parameters = new PageDrawerParameters(this, page, subsamplingAllowed,
                 destination, actualRenderingHints);
         PageDrawer drawer = createPageDrawer(parameters);
@@ -414,7 +425,7 @@ public class PDFRenderer
 
     /**
      * Indicates whether an optional content group is enabled.
-     * 
+     *
      * @param group the group
      * @return true if the group is enabled
      */
@@ -425,14 +436,12 @@ public class PDFRenderer
     }
 
     // scale rotate translate
-    private void transform(Graphics2D graphics, PDPage page, float scaleX, float scaleY)
+    private void transform(Graphics2D graphics, int rotationAngle, PDRectangle cropBox,
+            float scaleX, float scaleY)
     {
         graphics.scale(scaleX, scaleY);
 
         // TODO should we be passing the scale to PageDrawer rather than messing with Graphics?
-        int rotationAngle = page.getRotation();
-        PDRectangle cropBox = page.getCropBox();
-
         if (rotationAngle != 0)
         {
             float translateX = 0;
@@ -480,12 +489,11 @@ public class PDFRenderer
     private RenderingHints createDefaultRenderingHints(Graphics2D graphics)
     {
         RenderingHints r = new RenderingHints(null);
-        r.put(RenderingHints.KEY_INTERPOLATION,
-                isBitonal(graphics) ? RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR
-                        : RenderingHints.VALUE_INTERPOLATION_BICUBIC);
+        r.put(RenderingHints.KEY_INTERPOLATION, isBitonal(
+                graphics) ? RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR : RenderingHints.VALUE_INTERPOLATION_BICUBIC);
         r.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
-        r.put(RenderingHints.KEY_ANTIALIASING, isBitonal(graphics)
-                ? RenderingHints.VALUE_ANTIALIAS_OFF : RenderingHints.VALUE_ANTIALIAS_ON);
+        r.put(RenderingHints.KEY_ANTIALIASING, isBitonal(
+                graphics) ? RenderingHints.VALUE_ANTIALIAS_OFF : RenderingHints.VALUE_ANTIALIAS_ON);
         return r;
     }
 
@@ -510,24 +518,23 @@ public class PDFRenderer
         for (COSName name : resources.getExtGStateNames())
         {
             PDExtendedGraphicsState extGState = resources.getExtGState(name);
-            if (extGState == null)
+            if (extGState != null)
             {
-                // can happen if key exists but no value
+                // extGState null can happen if key exists but no value
                 // see PDFBOX-3950-23EGDHXSBBYQLKYOKGZUOVYVNE675PRD.pdf
-                continue;
-            }
-            BlendMode blendMode = extGState.getBlendMode();
-            if (blendMode != BlendMode.NORMAL)
-            {
-                return true;
+                BlendMode blendMode = extGState.getBlendMode();
+                if (blendMode != BlendMode.NORMAL)
+                {
+                    return true;
+                }
             }
         }
         return false;
     }
 
     /**
-     * Returns the image to which the current page is being rendered. May be null if the page is rendered to a
-     * Graphics2D object instead of a BufferedImage.
+     * Returns the image to which the current page is being rendered. May be null if the page is
+     * rendered to a Graphics2D object instead of a BufferedImage.
      */
     BufferedImage getPageImage()
     {
